/*---------------------------------------------------------------------------*
  Project:  NintendoWare
  File:     eft_Particle.vsh
  
  Copyright (C)2011-2013 Nintendo All rights reserved.
  
  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.
  
  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/


 //---------------------------------------------------
// ローカル座標系の法線ベクトル取得( カスタマイズ可能 )
//---------------------------------------------------
#ifndef USE_USR_GET_LOCAL_NORMAL
vec3 GetLocalNormal()
{
#ifdef _PARTICLE
    return vec3( 0, 0, 1 );
#endif
#ifdef _PRIMITIVE
    return sysNormalAttr;          // プリミティブの場合は入力法線を利用
#endif
}
#endif

//---------------------------------------------------
// ローカル座標系の接線ベクトル取得( カスタマイズ可能 )
//---------------------------------------------------
#ifndef USE_USR_GET_LOCAL_TANGENT
vec3 GetLocalTangent()
{
#ifdef _PARTICLE
    return vec3( 1, 0, 0 );
#endif

// プリミティブの場合は入力接線を利用
#ifdef _PRIMITIVE

// OpenGL版ではアトリビュートに余裕がある場合に利用可
#if defined( TARGET_WINDOWS )

#if defined( _ALPHA0_INHERIT ) || defined( _ALPHA1_INHERIT ) || defined( _COLOR0_INHERIT ) || defined( _COLOR1_INHERIT )
    return vec3( 1, 0, 0 );
#else
    return sysTangentAttr;
#endif

#else

    return sysTangentAttr;

#endif

#endif
}
#endif

//---------------------------------------------------
// ローカル座標系の従法線ベクトル取得( カスタマイズ可能 )
//---------------------------------------------------
#ifndef USE_USR_GET_LOCAL_BINORMAL
vec3 GetLocalBinormal()
{
#ifdef _PARTICLE
    return vec3( 0, 1, 0 );
#endif
#if defined( _PRIMITIVE ) && defined( TARGET_CAFE )
    return cross( sysNormalAttr, sysTangentAttr );
#else
    return vec3( 0, 1, 0 );
#endif


// プリミティブの場合は入力接線を利用
#ifdef _PRIMITIVE

// OpenGL版ではアトリビュートに余裕がある場合に利用可
#if defined( TARGET_WINDOWS )

#if defined( _ALPHA0_INHERIT ) || defined( _ALPHA1_INHERIT ) || defined( _COLOR0_INHERIT ) || defined( _COLOR1_INHERIT )
    return cross( sysNormalAttr, sysTangentAttr );
#else
    return vec3( 1, 1, 0 );
#endif

#else

    return cross( sysNormalAttr, sysTangentAttr );

#endif

#endif
}
#endif


//---------------------------------------------------
// Zバッファの奥行きを取得して、指定のz値が隠れているならを、見えているなら1を返す
//---------------------------------------------------
#ifndef USE_USR_CHECK_DEPTH_BUFFER
float CheckDepthBuffer( vec2 ofs, vec4 outDepthTexCoordProj, float pz )
{
    float tz = GetDepthValueFromTexture( outDepthTexCoordProj + vec4(ofs,0,0) );
    float dis = tz - pz;
    return step( 0,dis );
}
#endif

//---------------------------------------------------
// プリミティブのスケールZを取得( カスタマイズ可能 )
// X/Y どちらを適用するか分岐します。
//---------------------------------------------------

#if 0
#ifndef USE_USR_GET_PRIMITIVE_SCALE_Z
float GetPrimitiveScaleZ()
{
    float f = GET_BIT_FLAG( BIT_FLAG_PRIMITIVE_SCALE_Y_TO_Z );
    return f * sysPtclScale.y + ( 1 - f ) * sysPtclScale.x;
}
#endif
#else
#ifndef USE_USR_GET_PRIMITIVE_SCALE_Z
float GetPrimitiveScaleZ()
{
    if ( CHECK_BIT_FLAG( BIT_FLAG_PRIMITIVE_SCALE_Y_TO_Z ) )
    {
        return sysPtclScale.y;
    }
    else
    {
        return sysPtclScale.x;
    }
}
#endif
#endif

//--------------------------------------------------------------
// カスタムシェーダ処理
//--------------------------------------------------------------
#ifndef USE_USR_FINAL_ADJUSTMENT_VERTEX_PROCSS
void FinalAdjustmentVertexProcess(){}
#endif

#ifndef USE_USR_ADJUSTMENT_WORLD_POSITION
void AdjustmentWorldPosition() {}
#endif

//--------------------------------------------------------------
// エミッタプラグイン処理
//--------------------------------------------------------------
#ifndef USE_EMITTER_PLUGIN_FINAL_ADJUSTMENT_VERTEX_PROCSS
void EP_FinalAdjustmentVertexProcess(){}
#endif

#ifndef USE_EMITTER_PLUGIN_ADJUSTMENT_WORLD_POSITION
void EP_AdjustmentWorldPosition(){}
#endif

//---------------------------------------------------
// 遮蔽チェック
//---------------------------------------------------
#ifndef USE_USR_VERTEX_MASKING_CHECK
float CalcMaskingCheck()
{
    float alphaRate = 1.0;

    vec4 outDepthTexCoordProj = sysPtclFlagCoordCenter;

    // ピクセルZを取得する
    float cpz = sysPtclVertexDepthValueCenter;

    // 外側と内側のサンプリング点のデプスバッファとの前後関係を取得
    float r       = alphaFunc0.y;
    float out_sum = 0;
    float in_sum  = 0;

    // 外側の点
    out_sum += 2 * CheckDepthBuffer( vec2(-0.8,-0.7)*1.0*r, outDepthTexCoordProj, cpz );
    out_sum += 1 * CheckDepthBuffer( vec2(  -1,   0)*1.0*r, outDepthTexCoordProj, cpz );
    out_sum += 2 * CheckDepthBuffer( vec2( 0.7, 0.8)*1.0*r, outDepthTexCoordProj, cpz );
    out_sum += 1 * CheckDepthBuffer( vec2(   0,  -1)*1.0*r, outDepthTexCoordProj, cpz );

    out_sum += 1 * CheckDepthBuffer( vec2(   0,   1)*1.0*r, outDepthTexCoordProj, cpz );
    out_sum += 2 * CheckDepthBuffer( vec2( 0.7,-0.8)*1.0*r, outDepthTexCoordProj, cpz );
    out_sum += 1 * CheckDepthBuffer( vec2(   1,   0)*1.0*r, outDepthTexCoordProj, cpz );
    out_sum += 2 * CheckDepthBuffer( vec2( 0.8, 0.7)*1.0*r, outDepthTexCoordProj, cpz );

    out_sum += 2 * CheckDepthBuffer( vec2(-0.8,-0.7)*0.7*r, outDepthTexCoordProj, cpz );
    out_sum += 1 * CheckDepthBuffer( vec2(  -1,   0)*0.7*r, outDepthTexCoordProj, cpz );
    out_sum += 2 * CheckDepthBuffer( vec2( 0.7, 0.8)*0.7*r, outDepthTexCoordProj, cpz );
    out_sum += 1 * CheckDepthBuffer( vec2(   0,  -1)*0.7*r, outDepthTexCoordProj, cpz );

    out_sum += 1 * CheckDepthBuffer( vec2(   0,   1)*0.7*r, outDepthTexCoordProj, cpz );
    out_sum += 2 * CheckDepthBuffer( vec2( 0.7,-0.8)*0.7*r, outDepthTexCoordProj, cpz );
    out_sum += 1 * CheckDepthBuffer( vec2(   1,   0)*0.7*r, outDepthTexCoordProj, cpz );
    out_sum += 2 * CheckDepthBuffer( vec2( 0.8, 0.7)*0.7*r, outDepthTexCoordProj, cpz );

    out_sum += 2 * CheckDepthBuffer( vec2(-0.8,-0.7)*0.4*r, outDepthTexCoordProj, cpz );
    out_sum += 1 * CheckDepthBuffer( vec2(  -1,   0)*0.4*r, outDepthTexCoordProj, cpz );
    out_sum += 2 * CheckDepthBuffer( vec2( 0.7, 0.8)*0.4*r, outDepthTexCoordProj, cpz );
    out_sum += 1 * CheckDepthBuffer( vec2(   0,  -1)*0.4*r, outDepthTexCoordProj, cpz );

    out_sum += 1 * CheckDepthBuffer( vec2(   0,   1)*0.4*r, outDepthTexCoordProj, cpz );
    out_sum += 2 * CheckDepthBuffer( vec2( 0.7,-0.8)*0.4*r, outDepthTexCoordProj, cpz );
    out_sum += 1 * CheckDepthBuffer( vec2(   1,   0)*0.4*r, outDepthTexCoordProj, cpz );
    out_sum += 2 * CheckDepthBuffer( vec2( 0.8, 0.7)*0.4*r, outDepthTexCoordProj, cpz );

    // 内側の点
    in_sum += 3 * CheckDepthBuffer( vec2(-0.2, -0.4)*1.2*r, outDepthTexCoordProj, cpz );
    in_sum += 3 * CheckDepthBuffer( vec2( 0.2,  0.4)*1.2*r, outDepthTexCoordProj, cpz );
    in_sum += 3 * CheckDepthBuffer( vec2( 0.4, -0.2)*1.2*r, outDepthTexCoordProj, cpz );
    in_sum += 3 * CheckDepthBuffer( vec2(-0.4,  0.2)*1.2*r, outDepthTexCoordProj, cpz );

    in_sum += 3 * CheckDepthBuffer( vec2(-0.1, -0.2)*1.0*r, outDepthTexCoordProj, cpz );
    in_sum += 3 * CheckDepthBuffer( vec2( 0.1,  0.2)*1.0*r, outDepthTexCoordProj, cpz );
    in_sum += 3 * CheckDepthBuffer( vec2( 0.2, -0.1)*1.0*r, outDepthTexCoordProj, cpz );
    in_sum += 3 * CheckDepthBuffer( vec2(-0.2,  0.1)*1.0*r, outDepthTexCoordProj, cpz );

    in_sum += 5 * CheckDepthBuffer( vec2(   0,    0)*r, outDepthTexCoordProj, cpz );

    // 外側の遮蔽率と、内側の遮蔽率を個別に乗算して減衰(内側が全点隠れたら消えるようにするため)
    alphaRate *= clamp( out_sum/35.0, 0, 1 );
    alphaRate *= clamp( in_sum/7.0,   0, 1 );

    return alphaRate;
}
#endif

//---------------------------------------------------
// ソフトエッジ処理( カスタマイズ可能 )
//---------------------------------------------------
#ifndef USE_USR_VERTEX_SOFT_EDGE
float CalcVertexSoftEdge()
{
    float alphaRate = 1.0;

    // デプスオフセット版
    float sum       = 0.0;
    float rad       = alphaFunc0.y;
    float offset    = alphaFunc0.x;

    // パーティクル中心
    float cpz  = sysPtclVertexDepthValue;
    float ctz  = sysPtclTextureDepthValue;

    float cdis = ctz - cpz;
    cdis /= offset;
    cdis = clamp( cdis, 0.0, 1.0 );

    alphaRate = cdis;
/*
    // 円形サンプリング
    vec4 nearPos[16];
    nearPos[0]  = vec4(  1.0   ,  0.0   , offset, 0.0 );
    nearPos[1]  = vec4(  0.9239,  0.3827, offset, 0.0 );
    nearPos[2]  = vec4(  0.7071,  0.7071, offset, 0.0 );
    nearPos[3]  = vec4(  0.3827,  0.9239, offset, 0.0 );
    nearPos[4]  = vec4(  0.0   ,  1.0   , offset, 0.0 );
    nearPos[5]  = vec4( -0.9239,  0.3827, offset, 0.0 );
    nearPos[6]  = vec4( -0.7071,  0.7071, offset, 0.0 );
    nearPos[7]  = vec4( -0.3827,  0.9239, offset, 0.0 );
    nearPos[8]  = vec4( -1.0   ,  0.0   , offset, 0.0 );
    nearPos[9]  = vec4( -0.9239, -0.3827, offset, 0.0 );
    nearPos[10] = vec4( -0.7071, -0.7071, offset, 0.0 );
    nearPos[11] = vec4( -0.3827, -0.9239, offset, 0.0 );
    nearPos[12] = vec4(  0.0   , -1.0   , offset, 0.0 );
    nearPos[13] = vec4(  0.9239, -0.3827, offset, 0.0 );
    nearPos[14] = vec4(  0.7071, -0.7071, offset, 0.0 );
    nearPos[15] = vec4(  0.3827, -0.9239, offset, 0.0 );

    for( int i = 0; i < 16; ++i )
    {
        vec4 pos = sysFlagCoordVary + nearPos[i] * rad;

        float pz = GetDepthValue( pos );
        float tz = GetDepthValueFromTexture( pos );

        float dis = tz - pz;

        dis = clamp( dis, 0.0, 1.0 );
        sum += dis;
    }

    alphaRate = mix( 0.0, 1.0, sum / 16 );
*/

    return alphaRate;
}
#endif

#ifdef TARGET_CAFE

// ループ数を含めた現在の時間位置を[0,1]で取得する
float Calc8KeyAnimLocalTime( float time, float loopRate, float isLoopRandom )
{
    float flag = sign( loopRate );    // 0なら0、0より大なら1
    float localTime = fract( time / loopRate + isLoopRandom * sysPtclRandomX );
    return flag * localTime + ( 1 - flag ) * ( time / sysPtclLife );
}

// 指定区間の線形補間

#if 0
vec3 Get8KeyIntermediateValue( float localTime, vec4 param0, vec4 param1 )
{
    return vec3(1,1,1);
    vec3 vsub = param1.xyz - param0.xyz;
    float tsub = (param1.w - param0.w);
    float tbase = localTime - param0.w;
    return param0.xyz + vsub / tsub * tbase;
}
#else
#define Get8KeyIntermediateValue( localTime, param0, param1 ) \
    ( param0.xyz + ( param1.xyz - param0.xyz ) / ( param1.w - param0.w ) * ( localTime - param0.w ) )
#endif


#if 1
// キー数に応じて処理を軽い方に逃がすバージョン（1,2,4,8キーで対応）

#define DEFINE_CALC_1KEY_ANIM( NAME, PARAM )                                                \
vec3 Calculate1KeyAnim##NAME( int keyNum, float time, float loopRate, float isLoopRandom )  \
{                                                                                           \
    return PARAM.v[ 0 ].xyz;                                                            \
}     

#define DEFINE_CALC_2KEY_ANIM( NAME, PARAM )                                                \
vec3 Calculate2KeyAnim##NAME( int keyNum, float time, float loopRate, float isLoopRandom )  \
{                                                                                           \
    float tLocal = Calc8KeyAnimLocalTime( time, loopRate, isLoopRandom );                   \
    vec3 r1 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 0 ], PARAM.v[ 1 ] ).xyz;   \
    vec3 r2 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 1 ], PARAM.v[ 2 ] ).xyz;   \
    float f0 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 0 ].w );                           \
    float f1 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 1 ].w );                           \
    return ( 1 - f0 ) * PARAM.v[ 0 ].xyz                                                \
         + ( 1 - f1 ) * f0 * r1                                                             \
         + ( f1 )     * PARAM.v[ 1 ].xyz;                                               \
}     

#define DEFINE_CALC_3KEY_ANIM( NAME, PARAM )                                                \
vec3 Calculate3KeyAnim##NAME( int keyNum, float time, float loopRate, float isLoopRandom )  \
{                                                                                           \
    float tLocal = Calc8KeyAnimLocalTime( time, loopRate, isLoopRandom );                   \
    vec3 r1 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 0 ], PARAM.v[ 1 ] ).xyz;   \
    vec3 r2 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 1 ], PARAM.v[ 2 ] ).xyz;   \
    vec3 r3 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 2 ], PARAM.v[ 3 ] ).xyz;   \
    float f0 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 0 ].w );                           \
    float f1 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 1 ].w );                           \
    float f2 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 2 ].w );                           \
    return ( 1 - f0 ) * PARAM.v[ 0 ].xyz                                                \
         + ( 1 - f1 ) * f0 * r1                                                             \
         + ( 1 - f2 ) * f1 * r2                                                             \
         + ( f2 )     * PARAM.v[ 2 ].xyz;                                               \
}     

#define DEFINE_CALC_4KEY_ANIM( NAME, PARAM )                                                \
vec3 Calculate4KeyAnim##NAME( int keyNum, float time, float loopRate, float isLoopRandom )  \
{                                                                                           \
    float tLocal = Calc8KeyAnimLocalTime( time, loopRate, isLoopRandom );                   \
    vec3 r1 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 0 ], PARAM.v[ 1 ] ).xyz;   \
    vec3 r2 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 1 ], PARAM.v[ 2 ] ).xyz;   \
    vec3 r3 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 2 ], PARAM.v[ 3 ] ).xyz;   \
    float f0 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 0 ].w );                           \
    float f1 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 1 ].w );                           \
    float f2 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 2 ].w );                           \
    float f3 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 3 ].w );                           \
    return ( 1 - f0 ) * PARAM.v[ 0 ].xyz                                                \
         + ( 1 - f1 ) * f0 * r1                                                             \
         + ( 1 - f2 ) * f1 * r2                                                             \
         + ( 1 - f3 ) * f2 * r3                                                             \
         + ( f3 )     * PARAM.v[ 3 ].xyz;                                               \
}     

#define DEFINE_CALC_5KEY_ANIM( NAME, PARAM )                                                \
vec3 Calculate5KeyAnim##NAME( int keyNum, float time, float loopRate, float isLoopRandom )  \
{                                                                                           \
    float tLocal = Calc8KeyAnimLocalTime( time, loopRate, isLoopRandom );                   \
    vec3 r1 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 0 ], PARAM.v[ 1 ] ).xyz;   \
    vec3 r2 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 1 ], PARAM.v[ 2 ] ).xyz;   \
    vec3 r3 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 2 ], PARAM.v[ 3 ] ).xyz;   \
    vec3 r4 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 3 ], PARAM.v[ 4 ] ).xyz;   \
    float f0 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 0 ].w );                           \
    float f1 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 1 ].w );                           \
    float f2 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 2 ].w );                           \
    float f3 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 3 ].w );                           \
    float f4 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 4 ].w );                           \
    return ( 1 - f0 ) * PARAM.v[ 0 ].xyz                                                \
         + ( 1 - f1 ) * f0 * r1                                                             \
         + ( 1 - f2 ) * f1 * r2                                                             \
         + ( 1 - f3 ) * f2 * r3                                                             \
         + ( 1 - f4 ) * f3 * r4                                                             \
         + ( f4 )     * PARAM.v[ 4 ].xyz;                                               \
}     

#define DEFINE_CALC_6KEY_ANIM( NAME, PARAM )                                                \
vec3 Calculate6KeyAnim##NAME( int keyNum, float time, float loopRate, float isLoopRandom )  \
{                                                                                           \
    float tLocal = Calc8KeyAnimLocalTime( time, loopRate, isLoopRandom );                   \
    vec3 r1 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 0 ], PARAM.v[ 1 ] ).xyz;   \
    vec3 r2 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 1 ], PARAM.v[ 2 ] ).xyz;   \
    vec3 r3 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 2 ], PARAM.v[ 3 ] ).xyz;   \
    vec3 r4 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 3 ], PARAM.v[ 4 ] ).xyz;   \
    vec3 r5 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 4 ], PARAM.v[ 5 ] ).xyz;   \
    float f0 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 0 ].w );                           \
    float f1 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 1 ].w );                           \
    float f2 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 2 ].w );                           \
    float f3 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 3 ].w );                           \
    float f4 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 4 ].w );                           \
    float f5 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 5 ].w );                           \
    return ( 1 - f0 ) * PARAM.v[ 0 ].xyz                                                \
         + ( 1 - f1 ) * f0 * r1                                                             \
         + ( 1 - f2 ) * f1 * r2                                                             \
         + ( 1 - f3 ) * f2 * r3                                                             \
         + ( 1 - f4 ) * f3 * r4                                                             \
         + ( 1 - f5 ) * f4 * r5                                                             \
         + ( f5 )     * PARAM.v[ 5 ].xyz;                                               \
}     

#define DEFINE_CALC_7KEY_ANIM( NAME, PARAM )                                                \
vec3 Calculate7KeyAnim##NAME( int keyNum, float time, float loopRate, float isLoopRandom )  \
{                                                                                           \
    float tLocal = Calc8KeyAnimLocalTime( time, loopRate, isLoopRandom );                   \
    vec3 r1 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 0 ], PARAM.v[ 1 ] ).xyz;   \
    vec3 r2 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 1 ], PARAM.v[ 2 ] ).xyz;   \
    vec3 r3 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 2 ], PARAM.v[ 3 ] ).xyz;   \
    vec3 r4 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 3 ], PARAM.v[ 4 ] ).xyz;   \
    vec3 r5 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 4 ], PARAM.v[ 5 ] ).xyz;   \
    vec3 r6 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 5 ], PARAM.v[ 6 ] ).xyz;   \
    float f0 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 0 ].w );                           \
    float f1 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 1 ].w );                           \
    float f2 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 2 ].w );                           \
    float f3 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 3 ].w );                           \
    float f4 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 4 ].w );                           \
    float f5 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 5 ].w );                           \
    float f6 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 6 ].w );                           \
    return ( 1 - f0 ) * PARAM.v[ 0 ].xyz                                                \
         + ( 1 - f1 ) * f0 * r1                                                             \
         + ( 1 - f2 ) * f1 * r2                                                             \
         + ( 1 - f3 ) * f2 * r3                                                             \
         + ( 1 - f4 ) * f3 * r4                                                             \
         + ( 1 - f5 ) * f4 * r5                                                             \
         + ( 1 - f6 ) * f5 * r6                                                             \
         + ( f6 )     * PARAM.v[ 6 ].xyz;                                               \
}     

#define DEFINE_CALC_8KEY_ANIM( NAME, PARAM )                                                \
vec3 Calculate8KeyAnim##NAME( int keyNum, float time, float loopRate, float isLoopRandom )  \
{                                                                                           \
    float tLocal = Calc8KeyAnimLocalTime( time, loopRate, isLoopRandom );                   \
    vec3 r1 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 0 ], PARAM.v[ 1 ] ).xyz;   \
    vec3 r2 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 1 ], PARAM.v[ 2 ] ).xyz;   \
    vec3 r3 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 2 ], PARAM.v[ 3 ] ).xyz;   \
    vec3 r4 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 3 ], PARAM.v[ 4 ] ).xyz;   \
    vec3 r5 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 4 ], PARAM.v[ 5 ] ).xyz;   \
    vec3 r6 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 5 ], PARAM.v[ 6 ] ).xyz;   \
    vec3 r7 = Get8KeyIntermediateValue( tLocal, PARAM.v[ 6 ], PARAM.v[ 7 ] ).xyz;   \
    float f0 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 0 ].w );                           \
    float f1 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 1 ].w );                           \
    float f2 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 2 ].w );                           \
    float f3 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 3 ].w );                           \
    float f4 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 4 ].w );                           \
    float f5 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 5 ].w );                           \
    float f6 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 6 ].w );                           \
    float f7 = GET_FLAG_GREATER_EQ( tLocal, PARAM.v[ 7 ].w );                           \
    return ( 1 - f0 ) * PARAM.v[ 0 ].xyz                                                \
         + ( 1 - f1 ) * f0 * r1                                                             \
         + ( 1 - f2 ) * f1 * r2                                                             \
         + ( 1 - f3 ) * f2 * r3                                                             \
         + ( 1 - f4 ) * f3 * r4                                                             \
         + ( 1 - f5 ) * f4 * r5                                                             \
         + ( 1 - f6 ) * f5 * r6                                                             \
         + ( 1 - f7 ) * f6 * r7                                                             \
         + ( f7 )     * PARAM.v[ 7 ].xyz;                                               \
}     

#else
#define DEFINE_CALC_8KEY_ANIM( NAME, PARAM )                                                                              \
vec3 Calculate8KeyAnim##NAME( int keyNum, float time, float loopRate, float isLoopRandom )                                \
{                                                                                                                         \
    return PARAM.v[0].xyz;                                                                                            \
}                                                                                                                       
#endif

// 8キーアニメの計算関数の実体定義。
DEFINE_CALC_8KEY_ANIM( Color0, color0Anim )
DEFINE_CALC_8KEY_ANIM( Color1, color1Anim )
DEFINE_CALC_8KEY_ANIM( Alpha0, alpha0Anim )
DEFINE_CALC_8KEY_ANIM( Alpha1, alpha1Anim )
DEFINE_CALC_8KEY_ANIM( Scale, scaleAnim )
DEFINE_CALC_8KEY_ANIM( Shader, shaderAnim )

DEFINE_CALC_7KEY_ANIM( Color0, color0Anim )
DEFINE_CALC_7KEY_ANIM( Color1, color1Anim )
DEFINE_CALC_7KEY_ANIM( Alpha0, alpha0Anim )
DEFINE_CALC_7KEY_ANIM( Alpha1, alpha1Anim )
DEFINE_CALC_7KEY_ANIM( Scale, scaleAnim )
DEFINE_CALC_7KEY_ANIM( Shader, shaderAnim )

DEFINE_CALC_6KEY_ANIM( Color0, color0Anim )
DEFINE_CALC_6KEY_ANIM( Color1, color1Anim )
DEFINE_CALC_6KEY_ANIM( Alpha0, alpha0Anim )
DEFINE_CALC_6KEY_ANIM( Alpha1, alpha1Anim )
DEFINE_CALC_6KEY_ANIM( Scale, scaleAnim )
DEFINE_CALC_6KEY_ANIM( Shader, shaderAnim )

DEFINE_CALC_5KEY_ANIM( Color0, color0Anim )
DEFINE_CALC_5KEY_ANIM( Color1, color1Anim )
DEFINE_CALC_5KEY_ANIM( Alpha0, alpha0Anim )
DEFINE_CALC_5KEY_ANIM( Alpha1, alpha1Anim )
DEFINE_CALC_5KEY_ANIM( Scale, scaleAnim )
DEFINE_CALC_5KEY_ANIM( Shader, shaderAnim )

DEFINE_CALC_4KEY_ANIM( Color0, color0Anim )
DEFINE_CALC_4KEY_ANIM( Color1, color1Anim )
DEFINE_CALC_4KEY_ANIM( Alpha0, alpha0Anim )
DEFINE_CALC_4KEY_ANIM( Alpha1, alpha1Anim )
DEFINE_CALC_4KEY_ANIM( Scale, scaleAnim )
DEFINE_CALC_4KEY_ANIM( Shader, shaderAnim )

DEFINE_CALC_3KEY_ANIM( Color0, color0Anim )
DEFINE_CALC_3KEY_ANIM( Color1, color1Anim )
DEFINE_CALC_3KEY_ANIM( Alpha0, alpha0Anim )
DEFINE_CALC_3KEY_ANIM( Alpha1, alpha1Anim )
DEFINE_CALC_3KEY_ANIM( Scale, scaleAnim )
DEFINE_CALC_3KEY_ANIM( Shader, shaderAnim )

DEFINE_CALC_2KEY_ANIM( Color0, color0Anim )
DEFINE_CALC_2KEY_ANIM( Color1, color1Anim )
DEFINE_CALC_2KEY_ANIM( Alpha0, alpha0Anim )
DEFINE_CALC_2KEY_ANIM( Alpha1, alpha1Anim )
DEFINE_CALC_2KEY_ANIM( Scale, scaleAnim )
DEFINE_CALC_2KEY_ANIM( Shader, shaderAnim )

DEFINE_CALC_1KEY_ANIM( Color0, color0Anim )
DEFINE_CALC_1KEY_ANIM( Color1, color1Anim )
DEFINE_CALC_1KEY_ANIM( Alpha0, alpha0Anim )
DEFINE_CALC_1KEY_ANIM( Alpha1, alpha1Anim )
DEFINE_CALC_1KEY_ANIM( Scale, scaleAnim )
DEFINE_CALC_1KEY_ANIM( Shader, shaderAnim )

#endif

#ifdef TARGET_WINDOWS 

#if 1

// ループ数を含めた現在の時間位置を[0,1]で取得する
float Calc8KeyAnimLocalTime( float time, float loopRate, float isLoopRandom )
{
    // time: フレーム数
    // loopRate: このフレーム数でループする
    float localTime;  
    if ( loopRate > 0 )                                                      
    {                                                                       
        // ループあり（フレーム数loopRateでループする）                     
        localTime = time + isLoopRandom * sysPtclRandomX * loopRate;       
        return fract( localTime / loopRate );
    }                                                                       
    else
    {                                                                       
        // ループ無し（寿命のフレーム数までの[0,1]）                        
        return time / sysPtclLife;                                     
    }    
    return 0.0f;
}

// 指定区間の線形補間
vec3 Get8KeyIntermediateValue( float localTime, float t0, float t1, vec3 v0, vec3 v1 )
{
    return v0 + ( v1 - v0 ) / ( t1 - t0 ) * ( localTime - t0 );
}

#if 1
// キー数に応じて処理を軽い方に逃がすバージョン（1,2,4,8キーで対応）
#define DEFINE_CALC_8KEY_ANIM( NAME, PARAM )                                                                              \
vec3 Calculate8KeyAnim##NAME( int keyNum, float time, float loopRate, float isLoopRandom )                                \
{                                                                                                                         \
    float tLocal = Calc8KeyAnimLocalTime( time, loopRate, isLoopRandom );                                                      \
    float t0 = PARAM.v[ 0 ].w;                                                                                             \
    float t1 = PARAM.v[ 1 ].w;                                                                                             \
    float t2 = PARAM.v[ 2 ].w;                                                                                             \
    float t3 = PARAM.v[ 3 ].w;                                                                                             \
    float t4 = PARAM.v[ 4 ].w;                                                                                             \
    float t5 = PARAM.v[ 5 ].w;                                                                                             \
    float t6 = PARAM.v[ 6 ].w;                                                                                             \
    float t7 = PARAM.v[ 7 ].w;                                                                                             \
    vec3 v0 = PARAM.v[ 0 ].xyz;                                                                                             \
    vec3 v1 = PARAM.v[ 1 ].xyz;                                                                                             \
    vec3 v2 = PARAM.v[ 2 ].xyz;                                                                                             \
    vec3 v3 = PARAM.v[ 3 ].xyz;                                                                                             \
    vec3 v4 = PARAM.v[ 4 ].xyz;                                                                                             \
    vec3 v5 = PARAM.v[ 5 ].xyz;                                                                                             \
    vec3 v6 = PARAM.v[ 6 ].xyz;                                                                                             \
    vec3 v7 = PARAM.v[ 7 ].xyz;                                                                                             \
    if( keyNum > 4 )                                                                                                      \
    {                                                                                                                     \
        // 8キーアニメとして計算                                                                                                   \
        if ( tLocal < t0 ){ return v0; }                                                                         \
                                                                                                                                   \
        int lastKeyIndex = keyNum - 1;                                                                                             \
        float lastKeyTime = PARAM.v[ lastKeyIndex ].w;                                                                         \
        if ( lastKeyTime <= tLocal ){ return PARAM.v[ lastKeyIndex ].xyz; }                                                    \
                                                                                                                                   \
        if ( tLocal < t1 ){ return Get8KeyIntermediateValue( tLocal, t0, t1, v0, v1 ); }       \
        if ( tLocal < t2 ){ return Get8KeyIntermediateValue( tLocal, t1, t2, v1, v2 ); }  \
        if ( tLocal < t3 ){ return Get8KeyIntermediateValue( tLocal, t2, t3, v2, v3 ); }  \
        if ( tLocal < t4 ){ return Get8KeyIntermediateValue( tLocal, t3, t4, v3, v4 ); }  \
        if ( tLocal < t5 ){ return Get8KeyIntermediateValue( tLocal, t4, t5, v4, v5 ); }  \
        if ( tLocal < t6 ){ return Get8KeyIntermediateValue( tLocal, t5, t6, v5, v6 ); }  \
        if ( tLocal < t7 ){ return Get8KeyIntermediateValue( tLocal, t6, t7, v6, v7 ); }  \
        return v7;                                                                                                 \
    }                                                                                                                     \
    else if( keyNum == 3 || keyNum == 4 )                                                                                 \
    {                                                                                                                     \
        // 4キーアニメとして計算                                                                                                   \
        if ( tLocal < t0 ){ return v0; }                                                                            \
        if ( tLocal < t1 ){ return Get8KeyIntermediateValue( tLocal, t0, t1, v0, v1 ); }  \
        if ( tLocal < t2 ){ return Get8KeyIntermediateValue( tLocal, t1, t2, v1, v2 ); }  \
        if ( tLocal < t3 ){ return Get8KeyIntermediateValue( tLocal, t2, t3, v2, v3 ); }  \
        return v3;                                                                                                 \
    }                                                                                                                     \
    else if( keyNum == 2 )                                                                                                \
    {                                                                                                                     \
        // 2キーアニメを計算                                                                                              \
        if( tLocal < t0 ){ return v0; }                                                       \
        else if( tLocal < t1 ){ return Get8KeyIntermediateValue( tLocal, t0, t1, v0, v1 ); }  \
        return v1;                                                                                                        \
    }                                                                                                                     \
    else if( keyNum == 1 )                                                                                                \
    {                                                                                                                     \
        // 1キーはそのまま返す                                                                                            \
        return v0;                                                                                        \
    }                                                                                                                     \
}                                                                                                                       

#else
#define DEFINE_CALC_8KEY_ANIM( NAME, PARAM )                                                                              \
vec3 Calculate8KeyAnim##NAME( int keyNum, float time, float loopRate, float isLoopRandom )                                \
{                                                                                                                         \
    return PARAM.v[0].xyz;                                                                                            \
}                                                                                                                       
#endif

// 8キーアニメの計算関数の実体定義。
DEFINE_CALC_8KEY_ANIM( Color0, color0Anim )
DEFINE_CALC_8KEY_ANIM( Color1, color1Anim )
DEFINE_CALC_8KEY_ANIM( Alpha0, alpha0Anim )
DEFINE_CALC_8KEY_ANIM( Alpha1, alpha1Anim )
DEFINE_CALC_8KEY_ANIM( Scale, scaleAnim )
DEFINE_CALC_8KEY_ANIM( Shader, shaderAnim )

#else


//------------------------------------------------------------------------------
// 8キーアニメの計算（マクロ版）：ひな形
// かなり大きいマクロなので注意！
//------------------------------------------------------------------------------
#define CALC_8KEY_ANIM_DEFINE( NAME, PARAM_ANIM )                                           \
vec3 Calculate8KeyAnim##NAME( int keyNum, float time, float loopRate, float isLoopRandom )  \
{                                                                           \
    vec3 start  = vec3( 0 );                                                \
    vec3 end    = vec3( 0 );                                                \
    int  term   = 0;                                                        \
    float dis   = 0;                                                        \
    int lastKeyIndex = keyNum - 1;                                          \
    // キーなし                                                             \
    if ( keyNum == 0 )                                                      \
    {                                                                       \
        return vec3( 1 );                                                   \
    }                                                                       \
    // キーが一つのみ                                                       \
    if ( keyNum == 1 )                                                      \
    {                                                                       \
        return PARAM_ANIM.v[0].xyz;                                     \
    }                                                                       \
    // ループを加味した寿命[0, 1]に計算し直す                               \
    float localTime;                                                        \
    if( loopRate > 0 )                                                      \
    {                                                                       \
        // ループあり（フレーム数loopRateでループする）                     \
        localTime = time + isLoopRandom * sysPtclRandomX * loopRate;        \
        localTime = ( mod( localTime, loopRate ) ) / float( loopRate ) ;    \
    }                                                                       \
    else                                                                    \
    {                                                                       \
        // ループ無し（寿命のフレーム数までの[0,1]）                        \
        localTime = time / sysPtclLife;                                     \
    }                                                                       \
    // 既に最終キーの時間を過ぎている                                       \
    if ( PARAM_ANIM.v[ lastKeyIndex ].w <= localTime )                  \
    {                                                                       \
        // 最終キーのカラーでClampする                                      \
        return PARAM_ANIM.v[ lastKeyIndex ].xyz;                        \
    }                                                                       \
    // まだ最初のキーの時間に到達していない                                 \
    if ( localTime < PARAM_ANIM.v[0].w )                                \
    {                                                                       \
        // 最初のキーのカラーでClampする                                    \
        return PARAM_ANIM.v[0].xyz;                                     \
    }                                                                       \
    // TODO: 平たく書いた方がよいかも。                                     \
    for ( term = 0; term < lastKeyIndex; term++ )                           \
    {                                                                       \
        int index1 = term;                                                  \
        int index2 = term + 1;                                              \
        float t1   = PARAM_ANIM.v[ index1 ].w;                          \
        float t2   = PARAM_ANIM.v[ index2 ].w;                          \
        if ( ( t1 <= localTime ) && ( localTime < t2 ) )                    \
        {                                                                   \
            start = PARAM_ANIM.v[ index1 ].xyz;                         \
            end   = PARAM_ANIM.v[ index2 ].xyz;                         \
            dis   = t2 - t1;                                                \
            return ( start + ( end - start )/dis * ( localTime - t1 ) );    \
        }                                                                   \
    }                                                                       \
    return vec3( 0,0,0 );                                                   \
}                                                                        

CALC_8KEY_ANIM_DEFINE( Color0, color0Anim )
CALC_8KEY_ANIM_DEFINE( Color1, color1Anim )
CALC_8KEY_ANIM_DEFINE( Alpha0, alpha0Anim )
CALC_8KEY_ANIM_DEFINE( Alpha1, alpha1Anim )
CALC_8KEY_ANIM_DEFINE( Scale, scaleAnim )
CALC_8KEY_ANIM_DEFINE( Shader, shaderAnim )

#endif

#endif


//------------------------------------------------------------------------------
// YZX 回転行列を生成します。
//------------------------------------------------------------------------------
mat4 MakeRotationMatrixYZX( vec3 rotate )
{
    //回転の軸は常にWorld空間に固定されます

    mat4 rotMat;

    float sinX = sin( rotate.x );
    float cosX = cos( rotate.x );

    float sinY = sin( rotate.y );
    float cosY = cos( rotate.y );

    float sinZ = sin( rotate.z );
    float cosZ = cos( rotate.z );

    float opt1 = sinX * sinY;
    float opt2 = cosX * sinY;
    float opt3 = sinX * cosY;
    float opt4 = cosX * cosY;

    rotMat[0][0] =  cosY * cosZ;        // (cosY * cosZ)
    rotMat[0][1] =  opt4 * sinZ + opt1; // (cosX * cosY * sinZ) + (sinX * sinY)
    rotMat[0][2] =  opt3 * sinZ - opt2; // (sinX * cosY * sinZ) - (cosX * sinY)
    rotMat[0][3] =  0.0;

    rotMat[1][0] = -sinZ;               //-(sinZ)
    rotMat[1][1] =  cosZ * cosX;        // (cosX * cosZ)
    rotMat[1][2] =  cosZ * sinX;        // (sinX * cosZ)
    rotMat[1][3] =  0.0;

    rotMat[2][0] =  sinY * cosZ;        // (sinY * cosZ)
    rotMat[2][1] =  opt2 * sinZ - opt3; // (cosX * sinY * sinZ) - (sinX * cosY)
    rotMat[2][2] =  opt1 * sinZ + opt4; //-(sinX * sinY * sinZ) + (cosX * cosY)
    rotMat[2][3] =  0.0;

    rotMat[3][0] = 0.0;
    rotMat[3][1] = 0.0;
    rotMat[3][2] = 0.0;
    rotMat[3][3] = 1.0;

    return rotMat;
}

//------------------------------------------------------------------------------
// XYZ 回転行列を生成します。
//------------------------------------------------------------------------------
mat4 MakeRotationMatrixXYZ( vec3 rotate )
{
    //回転の軸は常にWorld空間に固定されます

    mat4 rotMat;

    float sinX = sin( rotate.x );
    float cosX = cos( rotate.x );

    float sinY = sin( rotate.y );
    float cosY = cos( rotate.y );

    float sinZ = sin( rotate.z );
    float cosZ = cos( rotate.z );

    float opt1 = cosX * sinZ;
    float opt2 = cosX * cosZ;
    float opt3 = sinX * sinZ;
    float opt4 = sinX * cosZ;

    rotMat[0][0] = cosY * cosZ;         // (cosY * cosZ)
    rotMat[0][1] = cosY * sinZ;         // (cosY * sinZ)
    rotMat[0][2] = -sinY;               //-(sinY)
    rotMat[0][3] = 0.0; 

    rotMat[1][0] = opt4 * sinY - opt1;  // (sinX * sinY * cosZ) - (cosX * sinZ)
    rotMat[1][1] = opt3 * sinY + opt2;  // (sinX * sinY * sinZ) + (cosX * cosZ)
    rotMat[1][2] = sinX * cosY;         // (sinX * cosY)
    rotMat[1][3] = 0.0; 

    rotMat[2][0] = opt2 * sinY + opt3;  // (cosX * sinY * cosZ) + (sinX * sinZ)
    rotMat[2][1] = opt1 * sinY - opt4;  // (cosX * sinY * sinZ) - (sinX * sinZ)
    rotMat[2][2] = cosX * cosY;         // (cosX * cosY)
    rotMat[2][3] = 0.0; 

    rotMat[3][0] = 0.0; 
    rotMat[3][1] = 0.0; 
    rotMat[3][2] = 0.0; 
    rotMat[3][3] = 1.0; 

    return rotMat;
}

//------------------------------------------------------------------------------
// ZXY 回転行列を生成します。
//------------------------------------------------------------------------------
mat4 MakeRotationMatrixZXY( vec3 rotate )
{
    //回転の軸は常にWorld空間に固定されます

    mat4 rotMat;

    float sinX = sin( rotate.x );
    float cosX = cos( rotate.x );

    float sinY = sin( rotate.y );
    float cosY = cos( rotate.y );

    float sinZ = sin( rotate.z );
    float cosZ = cos( rotate.z );

    float opt1 = cosY * cosZ;
    float opt2 = cosY * sinZ;
    float opt3 = sinY * cosZ;
    float opt4 = sinY * sinZ;

    rotMat[0][0] = sinX * opt4 + opt1;  // (sinX * sinY * sinZ) + (cosY * cosZ)
    rotMat[0][1] = cosX * sinZ;         // (cosX * sinZ)
    rotMat[0][2] = sinX * opt2 - opt3;  // (sinX * cosY * sinZ) - (sinY * cosZ)
    rotMat[0][3] = 0.0;

    rotMat[1][0] = sinX * opt3 - opt2;  // (sinX * sinY * cosZ) - (cosY * sinZ)
    rotMat[1][1] = cosX * cosZ;         // (cosX * cosZ)
    rotMat[1][2] = sinX * opt1 + opt4;  // (sinX * cosY * cosZ) + (sinY * sinZ)
    rotMat[1][3] = 0.0;

    rotMat[2][0] = cosX * sinY;         // (cosX * sinY)
    rotMat[2][1] = -sinX;               //-(sinX)
    rotMat[2][2] = cosX * cosY;         // (cosX * cosY)
    rotMat[2][3] = 0.0;

    rotMat[3][0] = 0.0;
    rotMat[3][1] = 0.0;
    rotMat[3][2] = 0.0;
    rotMat[3][3] = 1.0;

    return rotMat;
}

//------------------------------------------------------------------------------
//  揺らぎ 処理
//------------------------------------------------------------------------------
// 揺らぎ : Sin波
float CalcFluctuation_Sin( vec4 fluctuation, float time )
{
    #define amplitude    fluctuation.x
    #define cycle        fluctuation.y
    #define phaseRandom  fluctuation.z
    #define phaseInit    fluctuation.w

    float t = ( ( time + phaseInit ) / cycle ) + ( phaseRandom * sysPtclRandomX );   //現在フレーム位置（割合）

    // 正弦波の揺らぎ量を計算。
    // 掛け率 1.0 から amplitude 分減算する。
    // amplitude := [0.0, 1.0]

    float flucValue =  1 - ( ( cos( t * 6.2831853 ) + 1 ) * 0.5f * amplitude );   //揺らぎ量

    return flucValue;
}

// 揺らぎ : のこぎり波
float CalcFluctuation_SawTooth( vec4 fluctuation, float time )
{
    #define amplitude    fluctuation.x
    #define cycle        fluctuation.y
    #define phaseRandom  fluctuation.z
    #define phaseInit    fluctuation.w

    float t = (( time + phaseInit ) / cycle ) + ( phaseRandom * sysPtclRandomX );    //現在フレーム位置（割合）
    float flucValue = abs( ( 1 - fract( t ) * amplitude ) );                        //揺らぎ量
    
    return flucValue;
}

// 揺らぎ : 矩形波
float CalcFluctuation_Rect( vec4 fluctuation, float time )
{
    #define amplitude    fluctuation.x
    #define cycle        fluctuation.y
    #define phaseRandom  fluctuation.z
    #define phaseInit    fluctuation.w

    float t = (( time + phaseInit ) / cycle ) + ( phaseRandom * sysPtclRandomX );    //現在フレーム位置（割合）
    float flucValue = abs( 1 - ( 1 - step( 0.5, fract( t ) ) ) * amplitude );       //揺らぎ量

    return flucValue;
}


//------------------------------------------------------------------------------
// ディレクショナル計算
//------------------------------------------------------------------------------
void CalcDirectionalParticleType( vec3 localPos, vec3 upVector )
{
    vec3 basisX;
    vec3 basisY;
    vec3 basisZ;

    mat4 matVelLook = mat4( 1.0, 0.0, 0.0, 0.0,
                            0.0, 1.0, 0.0, 0.0,
                            0.0, 0.0, 1.0, 0.0,
                            0.0, 0.0, 0.0, 1.0 );

    basisY.xyz = sysPtclWorldPosDelta;
    basisY = normalize( basisY );

    basisX = cross( upVector, basisY );
    basisX = normalize( basisX );

    basisZ = cross( basisX, basisY );

    matVelLook[0][0] = basisX.x;
    matVelLook[1][0] = basisY.x;
    matVelLook[2][0] = basisZ.x;
    matVelLook[3][0] = 0.0;
    matVelLook[0][1] = basisX.y;
    matVelLook[1][1] = basisY.y;
    matVelLook[2][1] = basisZ.y;
    matVelLook[3][1] = 0.0;
    matVelLook[0][2] = basisX.z;
    matVelLook[1][2] = basisY.z;
    matVelLook[2][2] = basisZ.z;
    matVelLook[3][2] = 0.0;

#ifdef _USE_ROTATE
    matVelLook *= sysPtclRotateMatrix;
#endif

    gl_Position.xyz = ( matVelLook * vec4( localPos, 0 ) ).xyz + sysPtclWorldPos;

    sysWorldNormalVary    = GetLocalNormal();
    sysWorldTangentVary   = GetLocalTangent();
    sysWorldBinormalVary  = GetLocalBinormal();
    sysWorldNormalVary    = ( matVelLook * vec4( sysWorldNormalVary,   0.0 ) ).xyz;
    sysWorldTangentVary   = ( matVelLook * vec4( sysWorldTangentVary,  0.0 ) ).xyz;
    sysWorldBinormalVary  = ( matVelLook * vec4( sysWorldBinormalVary, 0.0 ) ).xyz;
}

//------------------------------------------------------------------------------
// 各ビルボードタイプごとの処理 - ビルボードパーティクル
//------------------------------------------------------------------------------
#ifdef _PARTICLE_TYPE_BILLBOARD
void ProcessParticleType( vec3 localPos )
{
    sysWorldNormalVary    = GetLocalNormal();
    sysWorldTangentVary   = GetLocalTangent();
    sysWorldBinormalVary  = GetLocalBinormal();

#ifdef _USE_ROTATE
    mat4 transposeMat   = transpose( bldMat ) * sysPtclRotateMatrix;
#else
    mat4 transposeMat   = transpose( bldMat );
#endif
    sysWorldNormalVary    = ( transposeMat * vec4( sysWorldNormalVary,   0.0 ) ).xyz;
    sysWorldTangentVary   = ( transposeMat * vec4( sysWorldTangentVary,  0.0 ) ).xyz;
    sysWorldBinormalVary  = ( transposeMat * vec4( sysWorldBinormalVary, 0.0 ) ).xyz;

#ifdef _USE_ROTATE
    vec3 temp = ( sysPtclRotateMatrix * vec4( localPos, 1 ) ).xyz;
    gl_Position.xyz = ( vec4( temp, 1 ) * bldMat ).xyz + sysPtclWorldPos;
#else
    gl_Position.xyz = ( vec4( localPos, 1 ) * bldMat ).xyz + sysPtclWorldPos;
#endif
    
}
#endif

//------------------------------------------------------------------------------
// 各ビルボードタイプごとの処理 - 高機能ビルボードパーティクル
//------------------------------------------------------------------------------
#ifdef _PARTICLE_TYPE_COMPLEX_BILLBOARD
void ProcessParticleType( vec3 localPos )
{
    sysWorldNormalVary    = GetLocalNormal();
    sysWorldTangentVary   = GetLocalTangent();
    sysWorldBinormalVary  = GetLocalBinormal();

    // ビルボードマトリクス
    vec3 viewUp = vec3( sysViewMatrix[1].x, sysViewMatrix[1].y, sysViewMatrix[1].z );
    
    vec3 zAxis = normalize( sysEyePosition.xyz - sysPtclWorldPos.xyz );
    vec3 xAxis = normalize( cross( zAxis, -viewUp ) );
    vec3 yAxis = cross( zAxis, xAxis );

    mat4 cmpBillboardMat = mat4(
        xAxis.x, yAxis.x, zAxis.x, 0,
        xAxis.y, yAxis.y, zAxis.y, 0,
        xAxis.z, yAxis.z, zAxis.z, 0,
              0,       0,       0, 1
        );

#ifdef _USE_ROTATE
    mat4 transposeMat   = transpose( cmpBillboardMat ) * sysPtclRotateMatrix;
#else
    mat4 transposeMat   = transpose( cmpBillboardMat );
#endif

    sysWorldNormalVary    = ( transposeMat * vec4( sysWorldNormalVary,   0.0 ) ).xyz;
    sysWorldTangentVary   = ( transposeMat * vec4( sysWorldTangentVary,  0.0 ) ).xyz;
    sysWorldBinormalVary  = ( transposeMat * vec4( sysWorldBinormalVary, 0.0 ) ).xyz;

#ifdef _USE_ROTATE
    vec3 temp = ( sysPtclRotateMatrix * vec4( localPos, 1 ) ).xyz;
    gl_Position.xyz = ( vec4( temp, 1 ) * cmpBillboardMat ).xyz + sysPtclWorldPos;
#else
    gl_Position.xyz = ( vec4( localPos, 1 ) * cmpBillboardMat ).xyz + sysPtclWorldPos;
#endif
}
#endif

//------------------------------------------------------------------------------
// 各ビルボードタイプごとの処理 - Yビルボードパーティクル（カメラを向く）
//------------------------------------------------------------------------------
#ifdef _PARTICLE_TYPE_Y_BILLBOARD
void ProcessParticleType( vec3 localPos )
{
    sysWorldNormalVary    = GetLocalNormal();
    sysWorldTangentVary   = GetLocalTangent();
    sysWorldBinormalVary  = GetLocalBinormal();

    sysWorldNormalVary    = ( sysPtclRotateMatrix * vec4( sysWorldNormalVary,   0.0 ) ).xyz;
    sysWorldTangentVary   = ( sysPtclRotateMatrix * vec4( sysWorldTangentVary,  0.0 ) ).xyz;
    sysWorldBinormalVary  = ( sysPtclRotateMatrix * vec4( sysWorldBinormalVary, 0.0 ) ).xyz;

    vec3 temp = ( sysPtclRotateMatrix * vec4( localPos, 1 ) ).xyz;
    gl_Position.xyz = temp + sysPtclWorldPos;
}
#endif

//------------------------------------------------------------------------------
// 各ビルボードタイプごとの処理 - Yビルボードパーティクル（カメラと平行）
//------------------------------------------------------------------------------
#ifdef _PARTICLE_TYPE_Y_BILLBOARD_PARALLEL
void ProcessParticleType( vec3 localPos )
{
    sysWorldNormalVary    = GetLocalNormal();
    sysWorldTangentVary   = GetLocalTangent();
    sysWorldBinormalVary  = GetLocalBinormal();

    sysWorldNormalVary    = ( sysPtclRotateMatrix * vec4( sysWorldNormalVary,   0.0 ) ).xyz;
    sysWorldTangentVary   = ( sysPtclRotateMatrix * vec4( sysWorldTangentVary,  0.0 ) ).xyz;
    sysWorldBinormalVary  = ( sysPtclRotateMatrix * vec4( sysWorldBinormalVary, 0.0 ) ).xyz;

    vec3 temp = ( sysPtclRotateMatrix * vec4( localPos, 1 ) ).xyz;
    gl_Position.xyz = temp + sysPtclWorldPos;
}
#endif

//------------------------------------------------------------------------------
// 各ビルボードタイプごとの処理 - 立体風ビルボード
//------------------------------------------------------------------------------
#ifdef _PARTICLE_TYPE_WITH_SCALE_Z
void ProcessParticleType( vec3 localPos )
{
    // 基本的な向きは標準ビルボードと同じ
    sysWorldNormalVary    = GetLocalNormal();
    sysWorldTangentVary   = GetLocalTangent();
    sysWorldBinormalVary  = GetLocalBinormal();

#ifdef _USE_ROTATE
    mat4 transposeMat   = transpose( bldMat ) * sysPtclRotateMatrix;
#else
    mat4 transposeMat   = transpose( bldMat );
#endif
    sysWorldNormalVary    = ( transposeMat * vec4( sysWorldNormalVary,   0.0 ) ).xyz;
    sysWorldTangentVary   = ( transposeMat * vec4( sysWorldTangentVary,  0.0 ) ).xyz;
    sysWorldBinormalVary  = ( transposeMat * vec4( sysWorldBinormalVary, 0.0 ) ).xyz;

#ifdef _USE_ROTATE
    vec3 temp = ( sysPtclRotateMatrix * vec4( localPos, 1 ) ).xyz;
    gl_Position.xyz = ( vec4( temp, 1 ) * bldMat ).xyz + sysPtclWorldPos;
#else
    gl_Position.xyz = ( vec4( localPos, 1 ) * bldMat ).xyz + sysPtclWorldPos;
#endif
}
#endif


//------------------------------------------------------------------------------
// 各ビルボードタイプごとの処理 - XYパーティクル
//------------------------------------------------------------------------------
#ifdef _PARTICLE_TYPE_POLYGON_XY
void ProcessParticleType( vec3 localPos )
{
    sysWorldNormalVary    = GetLocalNormal();
    sysWorldTangentVary   = GetLocalTangent();
    sysWorldBinormalVary  = GetLocalBinormal();


    mat4    tempMat;
    tempMat[0]            = vec4( sysEmitterRT[0].xyz, sysPtclWorldPos.x );
    tempMat[1]            = vec4( sysEmitterRT[1].xyz, sysPtclWorldPos.y );
    tempMat[2]            = vec4( sysEmitterRT[2].xyz, sysPtclWorldPos.z );
    tempMat[3]            = vec4( 0.0, 0.0, 0.0, 1.0 );

#ifdef _USE_ROTATE
    gl_Position           = ( sysPtclRotateMatrix * vec4( localPos, 1 ) );
    gl_Position           = gl_Position * tempMat;
    sysWorldNormalVary    = ( sysPtclRotateMatrix * vec4( sysWorldNormalVary,   0.0 ) ).xyz;
    sysWorldTangentVary   = ( sysPtclRotateMatrix * vec4( sysWorldTangentVary,  0.0 ) ).xyz;
    sysWorldBinormalVary  = ( sysPtclRotateMatrix * vec4( sysWorldBinormalVary, 0.0 ) ).xyz;

#else                     
    gl_Position           = vec4( localPos, 1 ) * tempMat;
#endif

    sysWorldNormalVary    = ( vec4( sysWorldNormalVary,   0.0 ) * tempMat ).xyz;
    sysWorldTangentVary   = ( vec4( sysWorldTangentVary,  0.0 ) * tempMat ).xyz;
    sysWorldBinormalVary  = ( vec4( sysWorldBinormalVary, 0.0 ) * tempMat ).xyz;
}
#endif

//------------------------------------------------------------------------------
// 各ビルボードタイプごとの処理 - XZパーティクル
//------------------------------------------------------------------------------
#ifdef _PARTICLE_TYPE_POLYGON_XZ
void ProcessParticleType( vec3 localPos )
{
    sysWorldNormalVary    = GetLocalNormal();
    sysWorldTangentVary   = GetLocalTangent();
    sysWorldBinormalVary  = GetLocalBinormal();

    vec4    yzFlipPos;
    yzFlipPos.xyz             = localPos.xzy;
    yzFlipPos.w               = 1.0;
    sysWorldNormalVary.xyz    = sysWorldNormalVary.xzy;
    sysWorldTangentVary.xyz   = sysWorldTangentVary.xzy;
    sysWorldBinormalVary.xyz  = sysWorldBinormalVary.xzy;
    yzFlipPos.z               = -yzFlipPos.z;
    sysWorldNormalVary.z      = -sysWorldNormalVary.z;
    sysWorldTangentVary.z     = -sysWorldTangentVary.z;
    sysWorldBinormalVary.z    = -sysWorldBinormalVary.z;

#ifdef _USE_ROTATE
    gl_Position           = sysPtclRotateMatrix * yzFlipPos;
    sysWorldNormalVary    = ( sysPtclRotateMatrix * vec4( sysWorldNormalVary,   0.0 ) ).xyz;
    sysWorldTangentVary   = ( sysPtclRotateMatrix * vec4( sysWorldTangentVary,  0.0 ) ).xyz;
    sysWorldBinormalVary  = ( sysPtclRotateMatrix * vec4( sysWorldBinormalVary, 0.0 ) ).xyz;
#else
    gl_Position           = yzFlipPos;
#endif

    mat4    tempMat;
    tempMat[0]          = vec4( sysEmitterRT[0].xyz, sysPtclWorldPos.x );
    tempMat[1]          = vec4( sysEmitterRT[1].xyz, sysPtclWorldPos.y );
    tempMat[2]          = vec4( sysEmitterRT[2].xyz, sysPtclWorldPos.z );
    tempMat[3]          = vec4( 0.0, 0.0, 0.0, 1.0 );

    gl_Position         = gl_Position * tempMat;
    sysWorldNormalVary    = ( vec4( sysWorldNormalVary,   0.0 ) * tempMat ).xyz;
    sysWorldTangentVary   = ( vec4( sysWorldTangentVary,  0.0 ) * tempMat ).xyz;
    sysWorldBinormalVary  = ( vec4( sysWorldBinormalVary, 0.0 ) * tempMat ).xyz;
}
#endif

//------------------------------------------------------------------------------
// 各ビルボードタイプごとの処理 - ディレクショナルYパーティクル
//------------------------------------------------------------------------------
#ifdef _PARTICLE_TYPE_VEL_LOOK
void ProcessParticleType( vec3 localPos )
{
    vec3 emitterUp = -eyeVec.xyz;
    CalcDirectionalParticleType( localPos, emitterUp );
}
#endif

//------------------------------------------------------------------------------
// 各ビルボードタイプごとの処理 - ディレクショナルポリゴン
//------------------------------------------------------------------------------
#ifdef _PARTICLE_TYPE_VEL_LOOK_POLYGON
void ProcessParticleType( vec3 localPos )
{
    vec3 emitterUp;
    emitterUp.x      = sysEmitterRT[0].y;
    emitterUp.y      = sysEmitterRT[1].y;
    emitterUp.z      = sysEmitterRT[2].y;

    CalcDirectionalParticleType( localPos, emitterUp );
}
#endif

//------------------------------------------------------------------------------
// 各ビルボードタイプごとの処理 - 立体風ビルボードディレクショナル
//------------------------------------------------------------------------------
#ifdef _PARTICLE_TYPE_WITH_SCALE_Z_DIRECTIONAL
void ProcessParticleType( vec3 localPos )
{
    vec3 emitterUp;
    emitterUp.x      = sysEmitterRT[0].y;
    emitterUp.y      = sysEmitterRT[1].y;
    emitterUp.z      = sysEmitterRT[2].y;

    CalcDirectionalParticleType( localPos, emitterUp );
}
#endif

//------------------------------------------------------------------------------
// タイムからベクトルを生成
//------------------------------------------------------------------------------
vec3 GetVectorFromTime( vec3 localVec, float time )
{
    #define airRegist               ubVec.x             // 空気抵抗
    #define gravityX                ubGrav.x            // 重力X
    #define gravityY                ubGrav.y            // 重力Y 
    #define gravityZ                ubGrav.z            // 重力Z
    #define gravityScale            ubGrav.w            // 重力スケール

    float k = airRegist;
    float t = time;
    vec3  gravity   = vec3 ( gravityX, gravityY, gravityZ );
    
    if ( k == 1.0 )
    {
        // 重力のみ
        gravity *= ( gravityScale * ( t * t * 0.5 ) );
    }
    else
    {
        // 重力 + 空気抵抗
        float klog = log( k );
        float kpow = pow( k, t );
        float tempK = 1.0 / ( 1 - k );
        float tempKPow = ( t - ( ( kpow - 1 ) / klog ) );
        float integration = tempK * tempKPow;

        gravity *= ( gravityScale * integration );
    }
    
#ifdef _WORLD_GRAVITY
    {
        float gravX = gravity.x;
        float gravY = gravity.y;
        float gravZ = gravity.z;
        gravity.x =  gravX * sysEmitterRT[0].x + gravY * sysEmitterRT[1].x + gravZ * sysEmitterRT[2].x;
        gravity.y =  gravX * sysEmitterRT[0].y + gravY * sysEmitterRT[1].y + gravZ * sysEmitterRT[2].y;
        gravity.z =  gravX * sysEmitterRT[0].z + gravY * sysEmitterRT[1].z + gravZ * sysEmitterRT[2].z;
    }
#endif

    // 空気抵抗値を加味した時間位置
    float velTime = ( airRegist == 1.0 ) ? time : ( 1 - pow( airRegist, time ) )/( 1 - airRegist );

    // 運動量ランダムを含める。
    vec3 ret = ( localVec * velTime ) + gravity;    
    
    // 移動量の積算を計算
    return ret;
}

//------------------------------------------------------------------------------
// 回転行列生成
//------------------------------------------------------------------------------
#if defined( _USE_ROTATE ) || defined( _PARTICLE_TYPE_Y_BILLBOARD ) || defined( _PARTICLE_TYPE_Y_BILLBOARD_PARALLEL )
mat4 CalcRotationMatrix( float time )
{
    #define rotInitX             ubRot0.x       // 初期値 X
    #define rotInitY             ubRot0.y       // 初期値 Y
    #define rotInitZ             ubRot0.z       // 初期値 Z
    #define rotInitRndX          ubRot1.x       // 初期値ランダム X
    #define rotInitRndY          ubRot1.y       // 初期値ランダム Y
    #define rotInitRndZ          ubRot1.z       // 初期値ランダム Z
    #define rotVelX              ubRot2.x       // 加算値 X
    #define rotVelY              ubRot2.y       // 加算値 Y
    #define rotVelZ              ubRot2.z       // 加算値 Z
    #define rotVelRegist         ubRot2.w       // 減衰値
    #define rotVelRandX          ubRot3.x       // 加算値ランダム X
    #define rotVelRandY          ubRot3.y       // 加算値ランダム Y
    #define rotVelRandZ          ubRot3.z       // 加算値ランダム Z

    // ※sysPtclRandom変数は初回に乱数を与えられて以降は固定

    // 乱数を増やす
    float tempRandX = ( sysPtclRandomX + sysPtclRandomY ) / 2;
    float tempRandY = ( sysPtclRandomY + sysPtclRandomZ ) / 2;
    float tempRandZ = ( sysPtclRandomZ + sysPtclRandomX ) / 2;
    // [-1.0, 1.0]
    tempRandX = (tempRandX - 0.5f) * 2;
    tempRandY = (tempRandY - 0.5f) * 2;
    tempRandZ = (tempRandZ - 0.5f) * 2;

    float tempRandX2 = sysPtclRandomX;
    float tempRandY2 = sysPtclRandomY;
    float tempRandZ2 = sysPtclRandomZ;
    // [-0.5, 0.5]
    tempRandX2 = (tempRandX2 - 0.5f);
    tempRandY2 = (tempRandY2 - 0.5f);
    tempRandZ2 = (tempRandZ2 - 0.5f);

    // 回転速度（ランダム量込み）
    float rVelX = rotVelX + ( tempRandX * rotVelRandX );
    float rVelY = rotVelY + ( tempRandY * rotVelRandY );
    float rVelZ = rotVelZ + ( tempRandZ * rotVelRandZ );

    // 初期回転
    float rInitX = sysInitRotateAttr.x;
    float rInitY = sysInitRotateAttr.y;
    float rInitZ = sysInitRotateAttr.z;

    // 回転方向ランダム
#if 0
    if ( CHECK_BIT_FLAG( BIT_FLAG_ROTATE_DIR_RANDOM_X ) )
    {
        if ( sysPtclRandomY > 0.5 )
        {
            rVelX = -rVelX;
            rInitX = -rInitX;
        }
    }
    if ( CHECK_BIT_FLAG( BIT_FLAG_ROTATE_DIR_RANDOM_Y ) )
    {
        if ( sysPtclRandomZ > 0.5 )
        {
            rVelY = -rVelY;
            rInitY = -rInitY;
        }
    }
    if ( CHECK_BIT_FLAG( BIT_FLAG_ROTATE_DIR_RANDOM_Z ) )
    {
        if ( sysPtclRandomX > 0.5 ) 
        {
            rVelZ = -rVelZ;
            rInitZ = -rInitZ;
        }
    }
#else
    {
        //float fx = 1.0;
        //float fy = 1.0;
        //float fz = 1.0;
        //fx *= sysPtclRandomY;
        //fy *= sysPtclRandomZ;
        //fz *= sysPtclRandomX;
        float fx = GET_BIT_FLAG( BIT_FLAG_ROTATE_DIR_RANDOM_X );
        float fy = GET_BIT_FLAG( BIT_FLAG_ROTATE_DIR_RANDOM_Y );
        float fz = GET_BIT_FLAG( BIT_FLAG_ROTATE_DIR_RANDOM_Z );
        fx *= GET_FLAG_RANDOM_HALF( sysPtclRandomY );
        fy *= GET_FLAG_RANDOM_HALF( sysPtclRandomZ );
        fz *= GET_FLAG_RANDOM_HALF( sysPtclRandomX );

        rVelX  += -2 * fx * rVelX;
        rInitX += -2 * fx * rInitX;
        rVelY  += -2 * fy * rVelY;
        rInitY += -2 * fy * rInitY;
        rVelZ  += -2 * fz * rVelZ;
        rInitZ += -2 * fz * rInitZ;
    }
#endif

    //回転初期値（ランダム量込み）
    float rotX = rInitX + ( tempRandX2 * rotInitRndX );
    float rotY = rInitY + ( tempRandY2 * rotInitRndY );
    float rotZ = rInitZ + ( tempRandZ2 * rotInitRndZ );

    //減衰率を計算
    float rotRegist = pow( rotVelRegist, time );
    float rotTime = ( rotVelRegist == 1.0 ) ? time : ( 1 - rotRegist )/( 1 - rotVelRegist );

    //現在時刻の回転量を計算
    float nRotX = ( rVelX * rotTime ) + rotX;
    float nRotY = ( rVelY * rotTime ) + rotY;
    float nRotZ = ( rVelZ * rotTime ) + rotZ;

    // ビルボードY値を計算
#ifdef _PARTICLE_TYPE_Y_BILLBOARD
    float billboardY = atan( -( eyePos.x - sysPtclWorldPos.x ), ( eyePos.z - sysPtclWorldPos.z ) );
    nRotY -= billboardY;
#endif

    // ビルボードY（カメラに平行）の値を計算
#ifdef _PARTICLE_TYPE_Y_BILLBOARD_PARALLEL
    float billboardY = atan( eyeVec.x, eyeVec.z );
    nRotY += billboardY;
#endif

    // 回転タイプにより分岐
#ifdef _ROTATE_YZX
    mat4 ret = MakeRotationMatrixYZX( vec3( nRotX, nRotY, nRotZ ) );
#endif
#ifdef _ROTATE_XYZ
    mat4 ret = MakeRotationMatrixXYZ( vec3( nRotX, nRotY, nRotZ ) );
#endif
#ifdef _ROTATE_ZXY
    mat4 ret = MakeRotationMatrixZXY( vec3( nRotX, nRotY, nRotZ ) );
#endif



    return ret;
}
#endif

//------------------------------------------------------------------------------
// main
//------------------------------------------------------------------------------
#ifndef USE_USR_VERTEX_MAIN
void main()
{

#define M_PI 3.14159265358979323846264

#if 0

    // パーティクルのローカル位置をCPUで計算済み入力値を利用
    sysPtclLocalPos = sysLocalPosAttr.xyz;

    // パーティクル中心世界座標
    sysPtclWorldPos = ApplyEmitterMatrix( sysPtclLocalPos );

    gl_Position.xyz = sysPosAttr.xyz + sysPtclWorldPos;
    gl_Position.w   = 1;

    // スクリーン座標系へ変換
    gl_Position = gl_Position * sysViewProjctionMatrix;

    sysColor0Vary.r = 1.0;
    sysColor0Vary.g = 1.0;
    sysColor0Vary.b = 1.0;
    sysColor0Vary.a = 1.0;
    sysColor1Vary.r = 1.0;
    sysColor1Vary.g = 1.0;
    sysColor1Vary.b = 1.0;
    sysColor1Vary.a = 1.0;

    sysFlagParamVary.x = 1.0;
    sysFlagParamVary.y = 1.0;

    // 計算結果を書き込み
    sysTexCoord01Vary.x = sysPosAttr.x + 0.5;
    sysTexCoord01Vary.y = sysPosAttr.y - 0.5;
    sysTexCoord01Vary.z = sysPosAttr.x + 0.5;
    sysTexCoord01Vary.w = sysPosAttr.y - 0.5;
    sysTexCoord2Vary.x  = sysPosAttr.x + 0.5;
    sysTexCoord2Vary.y  = sysPosAttr.y - 0.5;

#else

    #define color0AnimKeyNum    animKeyNum0.x   // カラー0アニメキー数
    #define alpha0AnimKeyNum    animKeyNum0.y   // アルファ0アニメキー数
    #define color1AnimKeyNum    animKeyNum0.z   // カラー1アニメキー数
    #define alpha1AnimKeyNum    animKeyNum0.w   // アルファ1アニメキー数
    #define scaleAnimKeyNum     animKeyNum1.x   // スケールアニメキー数
    #define color0LoopRate animLoopParam0.x
    #define alpha0LoopRate animLoopParam0.y
    #define color1LoopRate animLoopParam0.z
    #define alpha1LoopRate animLoopParam0.w
    #define scaleLoopRate animLoopParam1.x
    #define color0IsLoopInitRandom animLoopParam1.y
    #define alpha0IsLoopInitRandom animLoopParam1.z
    #define color1IsLoopInitRandom animLoopParam1.w
    #define alpha1IsLoopInitRandom animLoopParam2.x
    #define scaleIsLoopInitRandom  animLoopParam2.y
    #define colorScale          ubCol0.x        // カラースケール

    // パーティクル時間
    sysPtclTime = sysEmitterTime - sysPtclEmitTime;

    // CPUパーティクル時には、sysPtclEmitTimeが InitRot.w に書かれるため、
    // パーティクル時間が負になり、不正なタイミングで描画される可能性がある。
    if ( sysPtclTime < 0 ) 
    {
        DiscardVertex();
        return;
    }

    // 寿命を全うしたパーティクルは描画しない
    if ( sysPtclTime >= sysPtclLife ) 
    {
        DiscardVertex();
        return;
    }

    sysMasterAlpha = 1.0;   // 最終アルファを1.0からスタート。

    // 頂点インデックス
    sysIndexAttr = uint( sysPosAttr.w );

    // エミッタRTマトリクス生成
    MakeEmitterRTMatrix();

    //--------------------------------------------------------------
    // 揺らぎ
    //--------------------------------------------------------------
    float ptclFluctuationX = 1.0;
    float ptclFluctuationY = 1.0;

#if 0
#if defined( _FLUCTUATION_SCALE_ENABLE ) || defined( _FLUCTUATION_ALPHA_ENABLE )
    vec4 flucX = vec4 ( fluc.pm0.x, fluc.pm0.z, fluc.pm1.x, fluc.pm1.z );
    if ( CHECK_BIT_FLAG( BIT_FLAG_FLUCTUATION_TYPE_SIN   ) )      ptclFluctuationX = CalcFluctuation_Sin( flucX, sysPtclTime );
    else if ( CHECK_BIT_FLAG( BIT_FLAG_FLUCTUATION_TYPE_SAW_TOOTH ) )  ptclFluctuationX = CalcFluctuation_SawTooth( flucX, sysPtclTime );
    else if ( CHECK_BIT_FLAG( BIT_FLAG_FLUCTUATION_TYPE_RECT  ) )      ptclFluctuationX = CalcFluctuation_Rect( flucX, sysPtclTime );
    ptclFluctuationY = ptclFluctuationX;
#endif
#ifdef _FLUCTUATION_SCALE_Y_SEPARATE
    vec4 flucY = vec4 ( fluc.pm0.y, fluc.pm0.w, fluc.pm1.y, fluc.pm1.w );
    if ( CHECK_BIT_FLAG( BIT_FLAG_FLUCTUATION_TYPE_SIN  ) )       ptclFluctuationY = CalcFluctuation_Sin( flucY, sysPtclTime );
    else if ( CHECK_BIT_FLAG( BIT_FLAG_FLUCTUATION_TYPE_SAW_TOOTH ) )  ptclFluctuationY = CalcFluctuation_SawTooth( flucY, sysPtclTime );
    else if ( CHECK_BIT_FLAG( BIT_FLAG_FLUCTUATION_TYPE_RECT ) )       ptclFluctuationY = CalcFluctuation_Rect( flucY, sysPtclTime );
#endif
#else

#if defined( _FLUCTUATION_SCALE_ENABLE ) || defined( _FLUCTUATION_ALPHA_ENABLE )
    {
        vec4 flucX = vec4 ( fluc.pm0.x, fluc.pm0.z, fluc.pm1.x, fluc.pm1.z );
        vec3 v;
        v.x = GET_BIT_FLAG( BIT_FLAG_FLUCTUATION_TYPE_SIN );
        v.y = GET_BIT_FLAG( BIT_FLAG_FLUCTUATION_TYPE_SAW_TOOTH );
        v.z = GET_BIT_FLAG( BIT_FLAG_FLUCTUATION_TYPE_RECT );
        vec3 ret;
        ret.x = CalcFluctuation_Sin( flucX, sysPtclTime );
        ret.y = CalcFluctuation_SawTooth( flucX, sysPtclTime );
        ret.z = CalcFluctuation_Rect( flucX, sysPtclTime );
        ptclFluctuationX = dot( v, ret );

#ifndef _FLUCTUATION_SCALE_Y_SEPARATE
        ptclFluctuationY = ptclFluctuationX;
#else
        vec4 flucY = vec4 ( fluc.pm0.y, fluc.pm0.w, fluc.pm1.y, fluc.pm1.w );
        ret.x = CalcFluctuation_Sin( flucY, sysPtclTime );
        ret.y = CalcFluctuation_SawTooth( flucY, sysPtclTime );
        ret.z = CalcFluctuation_Rect( flucY, sysPtclTime );
        ptclFluctuationY = dot( v, ret );
#endif
    }
#endif
#endif

    //--------------------------------------------------------------
    // パーティクルスケール
    //--------------------------------------------------------------
    // 入力スケール
    sysPtclScale.xyz = sysScaleAttr.xyz;

#ifdef TARGET_CAFE
#ifdef _SCALE_ANIM_1_KEY
    sysPtclScale.xyz *= scaleAnim.v[0].xyz;
#endif
#ifdef _SCALE_ANIM_2_KEY
    sysPtclScale.xyz *= Calculate2KeyAnimScale( scaleAnimKeyNum, sysPtclTime, scaleLoopRate, scaleIsLoopInitRandom ).xyz;
#endif
#ifdef _SCALE_ANIM_3_KEY
    sysPtclScale.xyz *= Calculate3KeyAnimScale( scaleAnimKeyNum, sysPtclTime, scaleLoopRate, scaleIsLoopInitRandom ).xyz;
#endif
#ifdef _SCALE_ANIM_4_KEY
    sysPtclScale.xyz *= Calculate4KeyAnimScale( scaleAnimKeyNum, sysPtclTime, scaleLoopRate, scaleIsLoopInitRandom ).xyz;
#endif
#ifdef _SCALE_ANIM_5_KEY
    sysPtclScale.xyz *= Calculate5KeyAnimScale( scaleAnimKeyNum, sysPtclTime, scaleLoopRate, scaleIsLoopInitRandom ).xyz;
#endif
#ifdef _SCALE_ANIM_6_KEY
    sysPtclScale.xyz *= Calculate6KeyAnimScale( scaleAnimKeyNum, sysPtclTime, scaleLoopRate, scaleIsLoopInitRandom ).xyz;
#endif
#ifdef _SCALE_ANIM_7_KEY
    sysPtclScale.xyz *= Calculate7KeyAnimScale( scaleAnimKeyNum, sysPtclTime, scaleLoopRate, scaleIsLoopInitRandom ).xyz;
#endif
#ifdef _SCALE_ANIM_8_KEY
    sysPtclScale.xyz *= Calculate8KeyAnimScale( scaleAnimKeyNum, sysPtclTime, scaleLoopRate, scaleIsLoopInitRandom ).xyz;
#endif
#else
#ifdef _SCALE_ANIM
    sysPtclScale.xyz *= Calculate8KeyAnimScale( scaleAnimKeyNum, sysPtclTime, scaleLoopRate, scaleIsLoopInitRandom ).xyz;
#endif
#endif

    // スケール揺らぎ反映
#if defined( _FLUCTUATION_SCALE_ENABLE )
    sysPtclScale.x *= ptclFluctuationX;
    sysPtclScale.y *= ptclFluctuationY;
#endif
    // エミッタパーティクルスケールを適用
    sysPtclScale.x *= sysEmitterParticleScaleX;
    sysPtclScale.y *= sysEmitterParticleScaleY;
    sysPtclScale.z *= sysEmitterParticleScaleZ;

#ifdef _PARTICLE
#if defined( _PARTICLE_TYPE_WITH_SCALE_Z ) || defined( _PARTICLE_TYPE_WITH_SCALE_Z_DIRECTIONAL )
    // 立体風ビルボード系の場合
    // スケール値を保存
    vec3 scaleOrigin = sysPtclScale.xyz;    
#else
    // それ以外の場合、パーティクルのZスケールは1.0fにしておく
    sysPtclScale.z = 1.0f;
#endif
#endif

    //--------------------------------------------------------------
    // GPUパーティクル タイム版
    //--------------------------------------------------------------
#ifdef _CALC_TYPE_GPU_TIME
    // 現時間のベクトルを生成
    vec3 ptclLocalVec = GetVectorFromTime( sysLocalVecAttr.xyz, sysPtclTime );

    // sysPtclTime-frameRate 時間のベクトルを生成
    vec3 ptclPreLocalVec = GetVectorFromTime( sysLocalVecAttr.xyz, sysPtclTime - 1.0 );
    if( sysPtclTime < 0.001 )
    {
        // これ以上前のフレームは存在しないので、方向だけ与える
        vec3 norm = normalize( sysLocalVecAttr.xyz ) * 0.001;
        ptclPreLocalVec = ptclLocalVec - norm; 
    }   

    // パーティクルのローカル位置をタイムから生成
    float dynamicsRand = sysScaleAttr.w;
    sysPtclLocalPos = sysLocalPosAttr.xyz + ptclLocalVec * dynamicsRand ;
    //sysPtclLocalPos = sysLocalPosAttr.xyz + ptclLocalVec;

    // パーティクル世界座標移動差分
    sysPtclWorldPosDelta = ApplyEmitterRotateMatrix( ptclLocalVec - ptclPreLocalVec ).xyz;

#if defined(_PARTICLE_TYPE_VEL_LOOK) || defined(_PARTICLE_TYPE_VEL_LOOK_POLYGON) 
    // 座標差分が一定値以下の場合は、初速を姿勢として扱う
    float scalar = length( sysPtclWorldPosDelta );
    if ( scalar < 0.001 )
    {
        sysPtclWorldPosDelta = ApplyEmitterRotateMatrix( ptclLocalVec );
    }
#endif

#ifdef _FIELD_RANDOM
    // フィールド：ランダム
    #define airRegist               ubVec.x             // 空気抵抗
    // 空気抵抗値を加味した時間位置
    
    float velTime = sysPtclTime;
    if( rnd3.w == 1 )
    {
        velTime = ( airRegist == 1.0 ) ? sysPtclTime : ( 1 - pow( airRegist, sysPtclTime ) )/( 1 - airRegist );
    }

    vec3 randomAdd = vec3( 0, 0, 0 );   // ランダムによって動く変位
    vec3 randomVel = rnd0.xyz;   // ランダム適用量

    float RandCycle = rnd0.w;    // 乱数の基本周期のフレーム数
    vec4 seed = vec4( sysPtclRandomX, sysPtclRandomY, sysPtclRandomZ, sysPtclRandomW ); //乱数の種（初期位相）

    // 位相を統一
    if( rnd3.x == 1 )
    {
        seed.x = 0;
        seed.y = 0.31415f;    // x,y,z=0の時のために適当にずらす
        seed.z = 0.59265f;    // x,y,z=0の時のために適当にずらす
        seed.w = 0.58979f;    // x,y,z=0の時のために適当にずらす
    }

    seed *= RandCycle;

    // 位相を統一
    float speed = ( rnd4.x / 100.0f ) * RandCycle;   // 位相変化速度
    if( rnd3.x == 1 )
    {
        float randWidth = ( rnd4.y / 100.0f ) * RandCycle;   // 位相のばらつき
        seed.x += ( sysEmitterTime * speed ) + randWidth * ((sysPtclRandomX - 0.5f) * 2 );
        seed.y += ( sysEmitterTime * speed ) + randWidth * ((sysPtclRandomY - 0.5f) * 2 );
        seed.z += ( sysEmitterTime * speed ) + randWidth * ((sysPtclRandomZ - 0.5f) * 2 );
        seed.w += ( sysEmitterTime * speed ) + randWidth * ((sysPtclRandomW - 0.5f) * 2 );
    }

    vec3 randomValue = vec3( 0, 0, 0 );         // 乱数（これまでの積算値）
    float delta = velTime;                      // 時間経過の量（空気抵抗による減衰も加味している）

    // 乱数の積算値を取得
    randomValue.x = GetRandSum( seed.x, seed.x + delta );
    randomValue.y = GetRandSum( seed.y, seed.y + delta );
    randomValue.z = GetRandSum( seed.z, seed.z + delta );

    // 変位量を求める
    randomAdd.x = randomValue.x * randomVel.x;
    randomAdd.y = randomValue.y * randomVel.y;
    randomAdd.z = randomValue.z * randomVel.z;

    sysPtclLocalPos += randomAdd;
    sysPtclWorldPosDelta += ApplyEmitterRotateMatrix( randomAdd ).xyz;
#endif
#endif

    //--------------------------------------------------------------
    // GPUパーティクル ストリームアウト版
    //--------------------------------------------------------------
#ifdef _CALC_TYPE_GPU_SO
    // パーティクルのローカル位置をCPUで計算済み入力値を利用
    if ( sysPtclTime < 0.001 )
    {
        sysPtclLocalPos = sysLocalPosAttr.xyz;
    }
    else
    {
        sysPtclLocalPos = sysInPos.xyz;
    }

    // TODO : パーティクル世界座標移動差分をとりあえずベクトルを入力
    sysPtclWorldPosDelta = ApplyEmitterRotateMatrix( sysInVec.xyz ).xyz;

#ifdef _VELOCITY_SCALE_Y
    // フィールドの計算はStreamOutシェーダで済ませているので、
    // ここではすぐに計算できる物だけをdeltaに足し込む。
    // TODO: 根本的な解決方法は無いか検討
    if ( CHECK_BIT_FLAG_1( BIT_FLAG_FIELD_POSADD ) )
    {
        // ランダム（位置に合成）
        sysPtclWorldPosDelta.xyz += CalcAddPosField( sysFrameRate );
    }
#endif

    // 位置差分の値により、ディレクショナル系が描画されない症状への対応
    sysPtclWorldPosDelta.z += 0.000001;
#endif

    //--------------------------------------------------------------
    // CPUパーティクル
    //--------------------------------------------------------------
#ifdef _CALC_TYPE_CPU
    // パーティクルのローカル位置をCPUで計算済み入力値を利用
    sysPtclLocalPos = sysLocalPosAttr.xyz;

#if defined( _PARTICLE_TYPE_VEL_LOOK ) || defined( _PARTICLE_TYPE_VEL_LOOK_POLYGON ) || defined( _PARTICLE_TYPE_Y_BILLBOARD ) || defined( _PARTICLE_TYPE_Y_BILLBOARD_PARALLEL ) || defined( _VELOCITY_SCALE_Y )

    // パーティクル世界座標移動差分をCPUで計算済みベクトルを利用
    sysPtclWorldPosDelta = ApplyEmitterRotateMatrix( sysLocalDiffAttr.xyz ).xyz;

    // 位置差分の値により、ディレクショナル系が描画されない症状への対応
    sysPtclWorldPosDelta.z += 0.000001;

#endif

#endif


    // パーティクル中心世界座標
    sysPtclWorldPos = ApplyEmitterMatrix( sysPtclLocalPos );

    // パーティクルの回転値を生成
#if defined( _USE_ROTATE ) || defined( _PARTICLE_TYPE_Y_BILLBOARD ) || defined( _PARTICLE_TYPE_Y_BILLBOARD_PARALLEL )
    sysPtclRotateMatrix = CalcRotationMatrix( sysPtclTime );
#endif

    //--------------------------------------------------------------
    // 頂点処理
    //--------------------------------------------------------------
    // カメラオフセット
#ifdef _CAMERA_OFFSET
    vec3 cameraOffset = CalculateCameraOffset( sysPtclWorldPos );
#endif

    // 各ビルボードタイプごとの処理
    // 内部でgl_Position.xyzの更新が行われます。
    ProcessParticleType( vec3( ( sysPosAttr.xy + ( ubCenter.xy * 0.5 ) ) * sysPtclScale.xy, sysPosAttr.z * sysPtclScale.z ) );

    {
        // 世界座標を調整する。
        EP_AdjustmentWorldPosition();   // エミッタプラグインのコールバック呼び出し
        AdjustmentWorldPosition();      // カスタムシェーダのコールバック呼び出し
    }

    // 以降、Ptclのワールド座標は確定した前提。
#ifdef _PARTICLE
#if defined( _PARTICLE_TYPE_WITH_SCALE_Z ) || defined( _PARTICLE_TYPE_WITH_SCALE_Z_DIRECTIONAL )

#define DOT(A, B) ( A.x * B.x + A.y * B.y + A.z * B.z )

#define worldAxisY = vec3( 0, 1, 0 );                                                                   // ワールドY軸方向

#ifdef _PARTICLE_TYPE_WITH_SCALE_Z
    vec3 localAxisX = ( vec3( sysEmitterRT[0][0], sysEmitterRT[1][0], sysEmitterRT[2][0] ) );  // 仮想箱系のX軸方向
    vec3 localAxisY = ( vec3( sysEmitterRT[0][1], sysEmitterRT[1][1], sysEmitterRT[2][1] ) );  // 仮想箱系のY軸方向
    vec3 localAxisZ = ( vec3( sysEmitterRT[0][2], sysEmitterRT[1][2], sysEmitterRT[2][2] ) );  // 仮想箱系のZ軸方向
#endif
#ifdef _PARTICLE_TYPE_WITH_SCALE_Z_DIRECTIONAL
    vec3 localAxisZ = normalize( sysPtclWorldPosDelta.xyz );        // 仮想箱系のZ軸方向
    vec3 localAxisX = ( cross( vec3( 0, 1, 0 ), localAxisZ ) ); // 仮想箱系のX軸方向
    vec3 localAxisY = ( cross( localAxisZ, localAxisX ) ); // 仮想箱系のY軸方向
#endif
    vec3 iOffset = sysPtclWorldPos;//エミッタ位置＆Ptcl位置

    // カメラ方向（仮想箱座標系）を計算
    vec3 camLook  = normalize( iOffset - eyePos.xyz );              // 視線方向
    vec3 camLookH = normalize( DOT( camLook, localAxisX ) * localAxisX + DOT( camLook, localAxisZ ) * localAxisZ );   // 視線方向（水平面）
    vec3 camRight = normalize( cross( camLook, localAxisY ) );      // カメラ右方向
    vec3 camUp    = ( cross( camRight, camLook ) );        // カメラ上方向
    
    // 各種三角関数の値
    float cosY = DOT( camLook, localAxisY );
    float cosXH = DOT( camLookH, localAxisX );
    float cosZH = DOT( camLookH, localAxisZ );

    float sqCosX = cosXH * cosXH;
    float sq4CosX = sqCosX * sqCosX;

    float sqSinX = 1.0 - sqCosX;
    float sinX = sqrt( sqSinX );
    float sq4SinX = sqSinX * sqSinX;

    float sinY = sqrt( 1.0 - cosY * cosY );
    float sinZ = sqrt( 1.0 - cosZH * cosZH );
    
    // 俯瞰したときの矩形位置を計算: 正面～右側～背後～左側
    float a0 = -clamp( cosZH, -1, 0 );
    float a1 = -clamp( cosXH, -1, 0 );
    float a2 = clamp( cosZH, 0, 1 );
    float a3 = clamp( cosXH, 0, 1 );
   
    // 俯角からのスクリーンY方向の高さを決める
    float cX = DOT( camLook, localAxisX );
    float cZ = DOT( camLook, localAxisZ );
    float sX = 1.0 - cX * cX;
    float sZ = 1.0 - cZ * cZ;

    // 水平から見た時の矩形位置を計算
    //ToDo: b0, b1 の値は要調整
    float b0 = ( ( sinX * scaleOrigin.x ) + ( sinZ * scaleOrigin.z ) ) * 0.5 * sinY * sinY;
    float b1 = ( ( scaleOrigin.y * abs( sinY ) ) + ( ( sX * scaleOrigin.x ) * abs( sq4CosX ) + ( sZ * scaleOrigin.z ) * abs( sq4SinX ) ) * abs( cosY ) ) * 0.5;

    vec3 bR = camRight * b0;
    vec3 bU = camUp * b1;

    // y
    // 0-3 
    // 1-2 x

    // n: x y
    // 0: - +
    // 1: - -
    // 2: + -
    // 3: + +

    int signU = ( gl_VertexID & 0x2 ) - 1;
    int signV = -( ( gl_VertexID + 1 ) & 0x2 ) + 1;

    vec3 pHorizon = signU * bR + signV * bU;

    float d = sign( DOT( camLook, localAxisY ) );
    if( d == 0 ){ d = 1; }
    vec3 tempX = localAxisX * scaleOrigin.x * 0.5;
    vec3 tempZ = localAxisZ * scaleOrigin.z * 0.5;
    vec3 pVertical =
        (      signU * tempX * a0 +  d * signV * tempZ * a0 ) +
        (  d * signV * tempX * a1 -      signU * tempZ * a1 ) + 
        (     -signU * tempX * a2 -  d * signV * tempZ * a2 ) + 
        ( -d * signV * tempX * a3 +      signU * tempZ * a3 ) ;

    vec3 pMorph= pHorizon * abs( sinY ) + pVertical * abs( cosY );
    gl_Position.xyz += pMorph + iOffset - gl_Position.xyz;

#endif
#endif

    // 速度スケールY
#ifdef _VELOCITY_SCALE_Y
    vec3 ndiff = normalize( sysPtclWorldPosDelta );
    vec3 npos  = normalize( gl_Position.xyz - sysPtclWorldPos );
    float power = dot( ndiff, npos );
    gl_Position.xyz += sysPtclWorldPosDelta * sysEmitterParticleScaleXYZ * addVecToScale.x * power;
#endif

    // カメラオフセット値を加算
#ifdef _CAMERA_OFFSET
    sysPtclWorldPos += cameraOffset;
    gl_Position.xyz += cameraOffset;
#endif
    gl_Position.w   = 1;

    // カメラ前でのスケールを制限
#ifdef _PARTICLE_SCALE_LIMIT
    {
        #define scaleLimitDistNear ubCamScaleLimit.x
        #define scaleLimitDistFar ubCamScaleLimit.y

        vec3 offsetVec = gl_Position.xyz - sysPtclWorldPos.xyz;
        float dist = length( sysPtclWorldPos.xyz - sysEyePosition.xyz );

        //処理の反映率を計算	
        float rate = ( scaleLimitDistFar - dist ) / ( scaleLimitDistFar - scaleLimitDistNear );
        rate = clamp( rate, 0.0, 1.0 );

        vec3 minPos = offsetVec * ( min( scaleLimitDistNear , dist ) / scaleLimitDistNear ) + sysPtclWorldPos.xyz;
        gl_Position.xyz = mix( gl_Position.xyz, minPos, rate );
    }
#endif

    //v_outWorldPosを記憶
    sysWorldPositionVary = gl_Position.xyz;

    // 視線ベクトル
    sysEyeVecVary.xyz = sysWorldPositionVary - eyePos.xyz;

    // スクリーン座標系へ変換
    gl_Position = gl_Position * sysViewProjctionMatrix;

    // デプスオフセット
#ifdef _DEPTH_OFFSET
    float offset = offsetParam - ( offsetParam * gl_Position.z ) / gl_Position.w;
    gl_Position.z -= offset;
#endif

    // フラグメントシェーダでのスクリーン座標参照用
    sysFlagCoordVary = transformTexture * gl_Position;

    // パーティクルの四隅のデプス値を計算
    sysPtclVertexDepthValue = GetDepthValue( sysFlagCoordVary );

    // デプスバッファテクスチャからデプス値を取得
    sysPtclTextureDepthValue = GetDepthValueFromTexture( sysFlagCoordVary );       

    // パーティクルの中心座標がスクリーン座標でどこに置かれるか
#ifdef _DEPTH_OFFSET
    sysPtclFlagCoordCenter = transformTexture * ( vec4( sysPtclWorldPos, 1 ) * sysViewProjctionMatrix );
    float coffset = offsetParam - ( offsetParam * sysPtclFlagCoordCenter.z ) / sysPtclFlagCoordCenter.w;
    sysPtclFlagCoordCenter.z -= coffset;
#else
    sysPtclFlagCoordCenter = transformTexture * ( vec4( sysPtclWorldPos, 1 ) * sysViewProjctionMatrix );
#endif

    // パーティクルの中心のデプス値を計算
    sysPtclVertexDepthValueCenter = GetDepthValue( sysPtclFlagCoordCenter );


    //--------------------------------------------------------------
    // カラー/アルファ 0 処理
    //--------------------------------------------------------------
    // カラー0

#ifdef _COLOR_0_FIXED
    sysColor0Vary.rgb = color0Anim.v[0].xyz;
#endif
#ifdef _COLOR_0_RANDOM
    int index0 = int ( sysPtclRandomX * color0AnimKeyNum );
    sysColor0Vary.rgb = color0Anim.v[index0].xyz;
#endif

#ifdef TARGET_CAFE
#ifdef _COLOR_0_ANIM_1_KEY
    sysColor0Vary.rgb = color0Anim.v[0].xyz;
#endif
#ifdef _COLOR_0_ANIM_2_KEY
    sysColor0Vary.rgb = Calculate2KeyAnimColor0( color0AnimKeyNum, sysPtclTime, color0LoopRate, color0IsLoopInitRandom );
#endif
#ifdef _COLOR_0_ANIM_3_KEY
    sysColor0Vary.rgb = Calculate3KeyAnimColor0( color0AnimKeyNum, sysPtclTime, color0LoopRate, color0IsLoopInitRandom );
#endif
#ifdef _COLOR_0_ANIM_4_KEY
    sysColor0Vary.rgb = Calculate4KeyAnimColor0( color0AnimKeyNum, sysPtclTime, color0LoopRate, color0IsLoopInitRandom );
#endif
#ifdef _COLOR_0_ANIM_5_KEY
    sysColor0Vary.rgb = Calculate5KeyAnimColor0( color0AnimKeyNum, sysPtclTime, color0LoopRate, color0IsLoopInitRandom );
#endif
#ifdef _COLOR_0_ANIM_6_KEY
    sysColor0Vary.rgb = Calculate6KeyAnimColor0( color0AnimKeyNum, sysPtclTime, color0LoopRate, color0IsLoopInitRandom );
#endif
#ifdef _COLOR_0_ANIM_7_KEY
    sysColor0Vary.rgb = Calculate7KeyAnimColor0( color0AnimKeyNum, sysPtclTime, color0LoopRate, color0IsLoopInitRandom );
#endif
#ifdef _COLOR_0_ANIM_8_KEY
    sysColor0Vary.rgb = Calculate8KeyAnimColor0( color0AnimKeyNum, sysPtclTime, color0LoopRate, color0IsLoopInitRandom );
#endif
#else
#ifdef _COLOR_0_ANIM
    sysColor0Vary.rgb = Calculate8KeyAnimColor0( color0AnimKeyNum, sysPtclTime, color0LoopRate, color0IsLoopInitRandom );
#endif
#endif
    // アルファ0
#ifdef _ALPHA_0_FIXED
    sysColor0Vary.a = alpha0Anim.v[0].x;
#endif
#ifdef TARGET_CAFE
#ifdef _ALPHA_0_ANIM_1_KEY
    sysColor0Vary.a = alpha0Anim.v[0].x;
#endif
#ifdef _ALPHA_0_ANIM_2_KEY
    sysColor0Vary.a = Calculate2KeyAnimAlpha0( alpha0AnimKeyNum, sysPtclTime, alpha0LoopRate, alpha0IsLoopInitRandom ).x;
#endif
#ifdef _ALPHA_0_ANIM_3_KEY
    sysColor0Vary.a = Calculate3KeyAnimAlpha0( alpha0AnimKeyNum, sysPtclTime, alpha0LoopRate, alpha0IsLoopInitRandom ).x;
#endif
#ifdef _ALPHA_0_ANIM_4_KEY
    sysColor0Vary.a = Calculate4KeyAnimAlpha0( alpha0AnimKeyNum, sysPtclTime, alpha0LoopRate, alpha0IsLoopInitRandom ).x;
#endif
#ifdef _ALPHA_0_ANIM_5_KEY
    sysColor0Vary.a = Calculate5KeyAnimAlpha0( alpha0AnimKeyNum, sysPtclTime, alpha0LoopRate, alpha0IsLoopInitRandom ).x;
#endif
#ifdef _ALPHA_0_ANIM_6_KEY
    sysColor0Vary.a = Calculate6KeyAnimAlpha0( alpha0AnimKeyNum, sysPtclTime, alpha0LoopRate, alpha0IsLoopInitRandom ).x;
#endif
#ifdef _ALPHA_0_ANIM_7_KEY
    sysColor0Vary.a = Calculate7KeyAnimAlpha0( alpha0AnimKeyNum, sysPtclTime, alpha0LoopRate, alpha0IsLoopInitRandom ).x;
#endif
#ifdef _ALPHA_0_ANIM_8_KEY
    sysColor0Vary.a = Calculate8KeyAnimAlpha0( alpha0AnimKeyNum, sysPtclTime, alpha0LoopRate, alpha0IsLoopInitRandom ).x;
#endif
#else
#ifdef _ALPHA_0_ANIM
    sysColor0Vary.a = Calculate8KeyAnimAlpha0( alpha0AnimKeyNum, sysPtclTime, alpha0LoopRate, alpha0IsLoopInitRandom ).x;
#endif
#endif

    // カラー0 / アルファ0 継承
#ifdef _COLOR0_INHERIT
    sysColor0Vary.rgb *= sysColor0Attr.rgb;
#endif
#ifdef _ALPHA0_INHERIT
    sysColor0Vary.a *= sysColor0Attr.a;
#endif

    // エミッタカラーを乗算
    sysColor0Vary *= sysEmitterColor0;

    // カラースケール
    sysColor0Vary.rgb *= colorScale;

    //--------------------------------------------------------------
    // カラー/アルファ 1 処理
    //--------------------------------------------------------------
    // カラー1
#ifdef _COLOR_1_FIXED
    sysColor1Vary.rgb = color1Anim.v[0].rgb;
#endif
#ifdef _COLOR_1_RANDOM
    int index1 = int ( sysPtclRandomX * color1AnimKeyNum );
    sysColor1Vary.rgb = color1Anim.v[index1].rgb;
#endif

#ifdef TARGET_CAFE
#ifdef _COLOR_1_ANIM_1_KEY
    sysColor1Vary.rgb = color1Anim.v[0].rgb;
#endif
#ifdef _COLOR_1_ANIM_2_KEY
    sysColor1Vary.rgb = Calculate2KeyAnimColor1( color1AnimKeyNum, sysPtclTime, color1LoopRate, color1IsLoopInitRandom );
#endif
#ifdef _COLOR_1_ANIM_3_KEY
    sysColor1Vary.rgb = Calculate3KeyAnimColor1( color1AnimKeyNum, sysPtclTime, color1LoopRate, color1IsLoopInitRandom );
#endif
#ifdef _COLOR_1_ANIM_4_KEY
    sysColor1Vary.rgb = Calculate4KeyAnimColor1( color1AnimKeyNum, sysPtclTime, color1LoopRate, color1IsLoopInitRandom );
#endif
#ifdef _COLOR_1_ANIM_5_KEY
    sysColor1Vary.rgb = Calculate5KeyAnimColor1( color1AnimKeyNum, sysPtclTime, color1LoopRate, color1IsLoopInitRandom );
#endif
#ifdef _COLOR_1_ANIM_6_KEY
    sysColor1Vary.rgb = Calculate6KeyAnimColor1( color1AnimKeyNum, sysPtclTime, color1LoopRate, color1IsLoopInitRandom );
#endif
#ifdef _COLOR_1_ANIM_7_KEY
    sysColor1Vary.rgb = Calculate7KeyAnimColor1( color1AnimKeyNum, sysPtclTime, color1LoopRate, color1IsLoopInitRandom );
#endif
#ifdef _COLOR_1_ANIM_8_KEY
    sysColor1Vary.rgb = Calculate8KeyAnimColor1( color1AnimKeyNum, sysPtclTime, color1LoopRate, color1IsLoopInitRandom );
#endif
#else
#ifdef _COLOR_1_ANIM
    sysColor1Vary.rgb = Calculate8KeyAnimColor1( color1AnimKeyNum, sysPtclTime, color1LoopRate, color1IsLoopInitRandom );
#endif
#endif
    // アルファ1
#ifdef _ALPHA_1_FIXED
    sysColor1Vary.a = alpha1Anim.v[0].x;
#endif

#ifdef TARGET_CAFE
#ifdef _ALPHA_1_ANIM_1_KEY
    sysColor1Vary.a = alpha1Anim.v[0].x;
#endif
#ifdef _ALPHA_1_ANIM_2_KEY
    sysColor1Vary.a = Calculate2KeyAnimAlpha1( alpha1AnimKeyNum, sysPtclTime, alpha1LoopRate, alpha1IsLoopInitRandom ).x;
#endif
#ifdef _ALPHA_1_ANIM_3_KEY
    sysColor1Vary.a = Calculate3KeyAnimAlpha1( alpha1AnimKeyNum, sysPtclTime, alpha1LoopRate, alpha1IsLoopInitRandom ).x;
#endif
#ifdef _ALPHA_1_ANIM_4_KEY
    sysColor1Vary.a = Calculate4KeyAnimAlpha1( alpha1AnimKeyNum, sysPtclTime, alpha1LoopRate, alpha1IsLoopInitRandom ).x;
#endif
#ifdef _ALPHA_1_ANIM_5_KEY
    sysColor1Vary.a = Calculate5KeyAnimAlpha1( alpha1AnimKeyNum, sysPtclTime, alpha1LoopRate, alpha1IsLoopInitRandom ).x;
#endif
#ifdef _ALPHA_1_ANIM_6_KEY
    sysColor1Vary.a = Calculate6KeyAnimAlpha1( alpha1AnimKeyNum, sysPtclTime, alpha1LoopRate, alpha1IsLoopInitRandom ).x;
#endif
#ifdef _ALPHA_1_ANIM_7_KEY
    sysColor1Vary.a = Calculate7KeyAnimAlpha1( alpha1AnimKeyNum, sysPtclTime, alpha1LoopRate, alpha1IsLoopInitRandom ).x;
#endif
#ifdef _ALPHA_1_ANIM_8_KEY
    sysColor1Vary.a = Calculate8KeyAnimAlpha1( alpha1AnimKeyNum, sysPtclTime, alpha1LoopRate, alpha1IsLoopInitRandom ).x;
#endif
#else
#ifdef _ALPHA_1_ANIM
    sysColor1Vary.a = Calculate8KeyAnimAlpha1( alpha1AnimKeyNum, sysPtclTime, alpha1LoopRate, alpha1IsLoopInitRandom ).x;
#endif
#endif

    // カラー0 / アルファ0 継承
#ifdef _COLOR1_INHERIT
    sysColor1Vary.rgb *= sysColor1Attr.rgb;
#endif
#ifdef _COLOR1_INHERIT
    sysColor1Vary.a *= sysColor1Attr.a;
#endif

    // エミッタカラーを乗算
    sysColor1Vary *= sysEmitterColor1;

    // カラースケール
    sysColor1Vary.rgb *= colorScale;

    // プリミティブ頂点カラー
#ifdef _PRIMITIVE
    sysPrimitiveColorVary = sysVertexColor0Attr;
#endif

    //--------------------------------------------------------------
    // テクスチャ  パターン・座標アニメ処理
    //--------------------------------------------------------------
    int     ptclTexturePtn0;    // テクスチャパターン 0
    int     ptclTexturePtn1;    // テクスチャパターン 1
    int     ptclTexturePtn2;    // テクスチャパターン 2
    ptclTexturePtn0 = ptclTexturePtn1 = ptclTexturePtn2 = 0;

    // スフィアマップ テクスチャ座標を計算する
#ifdef _USE_SPHERE_MAP
    vec2 sphereMapTexCoord;
    vec3 viewPosition = normalize( ( vec4( sysWorldPositionVary, 1 ) * sysViewMatrix ).xyz );
    vec3 viewNormal   = normalize( ( vec4( sysWorldNormalVary,   0 ) * sysViewMatrix ).xyz );
    viewNormal.z = clamp( viewNormal.z, 0.2, 1.0 );     // 暫定処理（zの負数を回避）
    vec3 r = reflect( viewPosition, viewNormal );
    r.z = r.z + 1.0;
    float m = 2.0 * sqrt( dot(r, r) );          
    sphereMapTexCoord.x =  ( r.x / m ) + 0.5;
    sphereMapTexCoord.y = -( r.y / m ) + 0.5;
#endif

#ifdef _PARTICLE
    vec2 defuv = vec2( sysPosAttr.x, sysPosAttr.y );
#endif
#ifdef _PRIMITIVE
    vec2 defuv = vec2( sysTexCoordAttr.x, sysTexCoordAttr.y );
#endif

    vec2 inUv0   = defuv;
    vec2 inUv1   = defuv;
    vec2 inUv2   = defuv;
    vec4 outUv0  = vec4( 0 );
    vec4 outUv1  = vec4( 0 );
    vec4 outUv2  = vec4( 0 );

    // テクスチャ描画サイズを固定
#ifdef _KEEP_TEXTURE_SIZE
    inUv0 *= sysPtclScale.xy;
    inUv1 *= sysPtclScale.xy;
    inUv2 *= sysPtclScale.xy;
#endif

#ifdef _TEX_0_SPHERE_MAP
        inUv0.xy = sphereMapTexCoord;
#endif
#ifdef _TEX_1_SPHERE_MAP
        inUv1.xy = sphereMapTexCoord;
#endif
#ifdef _TEX_2_SPHERE_MAP
        inUv2.xy = sphereMapTexCoord;
#endif

    // テクスチャ反転ランダム
#ifdef _PARTICLE
#ifdef _TEXTURE0_ENABLE
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX0_INV_RANDOM_U ) ) { if ( sysPtclRandomX > 0.5 ) inUv0.x = -inUv0.x; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX0_INV_RANDOM_V ) ) { if ( sysPtclRandomY > 0.5 ) inUv0.y = -inUv0.y; }
#endif
#ifdef _TEXTURE1_ENABLE
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX1_INV_RANDOM_U ) ) { if ( sysPtclRandomZ > 0.5 ) inUv1.x = -inUv1.x; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX1_INV_RANDOM_V ) ) { if ( sysPtclRandomW > 0.5 ) inUv1.y = -inUv1.y; }
#endif
#ifdef _TEXTURE2_ENABLE
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX2_INV_RANDOM_U ) ) { if ( sysPtclRandomY > 0.5 ) inUv2.x = -inUv2.x; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX2_INV_RANDOM_V ) ) { if ( sysPtclRandomZ > 0.5 ) inUv2.y = -inUv2.y; }
#endif
#endif

#ifdef _PRIMITIVE
#ifdef _TEXTURE0_ENABLE
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX0_INV_RANDOM_U ) ) { if ( sysPtclRandomX > 0.5 ) inUv0.x = 1.0 - inUv0.x; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX0_INV_RANDOM_V ) ) { if ( sysPtclRandomY > 0.5 ) inUv0.y = 1.0 - inUv0.y; }
#endif
#ifdef _TEXTURE1_ENABLE
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX1_INV_RANDOM_U ) ) { if ( sysPtclRandomZ > 0.5 ) inUv1.x = 1.0 - inUv1.x; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX1_INV_RANDOM_V ) ) { if ( sysPtclRandomW > 0.5 ) inUv1.y = 1.0 - inUv1.y; }
#endif
#ifdef _TEXTURE2_ENABLE
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX2_INV_RANDOM_U ) ) { if ( sysPtclRandomY > 0.5 ) inUv2.x = 1.0 - inUv2.x; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX2_INV_RANDOM_V ) ) { if ( sysPtclRandomZ > 0.5 ) inUv2.y = 1.0 - inUv2.y; }
#endif
#endif

    vec3 texPtnRandomSeed;
    texPtnRandomSeed.x = texPtnRandomSeed.y = texPtnRandomSeed.z = sysPtclRandomX;

    // テクスチャ座標アニメを計算
    // 乱数を共有するかしないかの設定
    ivec3 texShareRandom = ivec3( 0, 0, 0 );
    if( !CHECK_BIT_FLAG_1( BIT_FLAG_TEX_ANIM_FIXED_RANDOM ) )
    {
        // 乱数を共有しない
        texShareRandom.y = 1;
        texShareRandom.z = 2;        

        texPtnRandomSeed.x = sysPtclRandomX;
        texPtnRandomSeed.y = sysPtclRandomY;
        texPtnRandomSeed.z = sysPtclRandomZ;
    }

    // テクスチャ0パターンアニメ コマ値を計算
#if defined( _TEXTURE0_ENABLE ) && defined( _TEX_0_PATTERN_ANIM )
    ptclTexturePtn0 = CalculateTexPtnAnim0( sysPtclTime, sysPtclLife, texPtnRandomSeed.x );
#endif

    // テクスチャ1パターンアニメ コマ値を計算
#if defined( _TEXTURE1_ENABLE ) && defined( _TEX_1_PATTERN_ANIM )
    ptclTexturePtn1 = CalculateTexPtnAnim1( sysPtclTime, sysPtclLife, texPtnRandomSeed.y );
#endif

    // テクスチャ2パターンアニメ コマ値を計算
#if defined( _TEXTURE2_ENABLE ) && defined( _TEX_2_PATTERN_ANIM )
    ptclTexturePtn2 = CalculateTexPtnAnim2( sysPtclTime, sysPtclLife, texPtnRandomSeed.z );
#endif

    vec2 animDir = vec2( 1, 1 );
    vec4 randomSeed = vec4( sysPtclRandomX, sysPtclRandomY, sysPtclRandomZ, sysPtclRandomW );

#ifdef _TEXTURE0_ENABLE
#if defined( _TEX_0_SHIFT_ANIM ) || defined( _TEX_0_SHIFT_ANIM_ROTATE ) || defined( _TEX_0_PATTERN_ANIM )
#ifdef _TEX_0_SHIFT_ANIM_ROTATE
    outUv0 = CalculateTextureShiftAnimWithRotate( inUv0, texShiftAnim0, sysPtclTime, ptclTexturePtn0, texShareRandom.x, randomSeed );
#else
    outUv0 = CalculateTextureShiftAnim( inUv0, texShiftAnim0, sysPtclTime, ptclTexturePtn0, texShareRandom.x, randomSeed );
#endif
#else
#ifdef _PARTICLE
    outUv0.x = ( inUv0.x + 0.5 ) *  texShiftAnim0.pm4.x * ( 1 / texShiftAnim0.pm4.z );
    outUv0.y = ( inUv0.y - 0.5 ) * -texShiftAnim0.pm4.y * ( 1 / texShiftAnim0.pm4.w );
#endif
#ifdef _PRIMITIVE
    outUv0.xy = inUv0.xy * texShiftAnim0.pm4.xy * ( 1 / texShiftAnim0.pm4.zw );
#endif
#endif
#endif


#ifdef _TEXTURE1_ENABLE
#if defined( _TEX_1_SHIFT_ANIM ) || defined( _TEX_1_SHIFT_ANIM_ROTATE ) || defined( _TEX_1_PATTERN_ANIM )
#ifdef _TEX_1_SHIFT_ANIM_ROTATE
    outUv1 = CalculateTextureShiftAnimWithRotate( inUv1, texShiftAnim1, sysPtclTime, ptclTexturePtn1, texShareRandom.y, randomSeed );
#else
    outUv1 = CalculateTextureShiftAnim( inUv1, texShiftAnim1, sysPtclTime, ptclTexturePtn1, texShareRandom.y, randomSeed );
#endif
#else
#ifdef _PARTICLE
    outUv1.x = ( inUv1.x + 0.5 ) *  texShiftAnim1.pm4.x * ( 1 / texShiftAnim1.pm4.z );
    outUv1.y = ( inUv1.y - 0.5 ) * -texShiftAnim1.pm4.y * ( 1 / texShiftAnim1.pm4.w );
#endif
#ifdef _PRIMITIVE
    outUv1.xy = inUv1.xy * texShiftAnim1.pm4.xy * ( 1 / texShiftAnim1.pm4.zw );
#endif
#endif
#endif

#ifdef _TEXTURE2_ENABLE
#if defined( _TEX_2_SHIFT_ANIM ) || defined( _TEX_2_SHIFT_ANIM_ROTATE ) || defined( _TEX_2_PATTERN_ANIM )
#ifdef _TEX_2_SHIFT_ANIM_ROTATE
    outUv2 = CalculateTextureShiftAnimWithRotate( inUv2, texShiftAnim2, sysPtclTime, ptclTexturePtn2, texShareRandom.z, randomSeed );
#else
    outUv2 = CalculateTextureShiftAnim( inUv2, texShiftAnim2, sysPtclTime, ptclTexturePtn2, texShareRandom.z, randomSeed );
#endif
#else
#ifdef _PARTICLE
    outUv2.x = ( inUv2.x + 0.5 ) *  texShiftAnim2.pm4.x * ( 1 / texShiftAnim2.pm4.z );
    outUv2.y = ( inUv2.y - 0.5 ) * -texShiftAnim2.pm4.y * ( 1 / texShiftAnim2.pm4.w );
#endif
#ifdef _PRIMITIVE
    outUv2.xy = inUv2.xy * texShiftAnim2.pm4.xy * ( 1 / texShiftAnim2.pm4.zw );
#endif
#endif
#endif

    // 計算結果を書き込み
    sysTexCoord01Vary.x = outUv0.x;
    sysTexCoord01Vary.y = outUv0.y;
    sysTexCoord01Vary.z = outUv1.x;
    sysTexCoord01Vary.w = outUv1.y;
    sysTexCoord2Vary.x  = outUv2.x;
    sysTexCoord2Vary.y  = outUv2.y;


    //--------------------------------------------------------------
    // アルファ処理
    //--------------------------------------------------------------
    // アルファ係数
    sysPtclAlphaRate        = 1.0;
    float nearDistAlphaRate = 1.0;
    float farDistAlphaRate  = 1.0;
    float maskAlphaRate     = 1.0;
    float softEdgeRate      = 1.0;

    //------------------------------
    // 遮蔽チェック処理
    //------------------------------
#ifdef _MASKING_CHECK
    maskAlphaRate = CalcMaskingCheck();
    sysPtclAlphaRate *= maskAlphaRate;
#endif

    //------------------------------
    // ソフトパーティクル処理
    //------------------------------
#ifdef _VERTEX_SOFT
    softEdgeRate = CalcVertexSoftEdge();
    sysPtclAlphaRate *= softEdgeRate;
#endif

    //------------------------------
    // フレネル処理
    //------------------------------
#ifdef _VERTEX_FRESNEL_ALPHA
    float fresnel = CalcFresnelRate( sysEyeVecVary.xyz, sysWorldNormalVary );
    sysPtclAlphaRate *= fresnel;
#endif

    //------------------------------
    // 近距離アルファ
    //------------------------------
#ifdef _NEAR_DIST_ALPHA
    #define nearAlphaMin    alphaFunc1.x
    #define nearAlphaMax    alphaFunc1.y
    nearDistAlphaRate = ( sysPtclVertexDepthValueCenter - nearAlphaMin )/( nearAlphaMax - nearAlphaMin );
    nearDistAlphaRate = clamp( nearDistAlphaRate, 0.0, 1.0 );
    sysPtclAlphaRate *= nearDistAlphaRate;
#endif

    //------------------------------
    // 遠距離アルファ
    //------------------------------
#ifdef _FAR_DIST_ALPHA
    #define farAlphaMin    alphaFunc1.z
    #define farAlphaMax    alphaFunc1.w
    farDistAlphaRate = ( sysPtclVertexDepthValueCenter - farAlphaMin )/( farAlphaMax - farAlphaMin );
    farDistAlphaRate = clamp( farDistAlphaRate, 0, 1 );
    farDistAlphaRate = 1- farDistAlphaRate;
    sysPtclAlphaRate *= farDistAlphaRate;
#endif

    // アルファ係数をフラグメントに送る
    // フェードのアルファ値もここで処理
    sysMasterAlpha *= ( sysPtclAlphaRate * sysEmitterFadeAlpha );

    // アルファ 揺らぎ
#if defined( _FLUCTUATION_ALPHA_ENABLE )
    sysMasterAlpha *= ptclFluctuationX;
#endif

    // 寿命係数
    sysFlagParamVary.y = sysPtclTime / sysPtclLife;

    // シェーダアニメを計算
#ifdef _SHADER_ANIM
    sysShaderAnim = Calculate8KeyAnimShader( 4, sysPtclTime, 0, 0 ).x;
#else
    sysShaderAnim = 1.0;
#endif

    //--------------------------------------------------------------
    // 最終調整用コールバック
    //--------------------------------------------------------------
    EP_FinalAdjustmentVertexProcess();  // エミッタプラグインのコールバック
    FinalAdjustmentVertexProcess();     // カスタムシェーダのコールバック

    //--------------------------------------------------------------
    // αが0の場合、カメラの後ろに飛ばしてフラグメントシェーダから弾く。
    //--------------------------------------------------------------
    if( farDistAlphaRate <= 0 || nearDistAlphaRate <= 0 )
    {
        DiscardVertex();
        return;
    }

#ifdef _CALC_TYPE_GPU_SO

    if ( sysInVec.w >= sysPtclLife - 2 )
    {
        // 一旦他のカリング処理に合わせる。
        DiscardVertex();
    }

#endif

#endif
}
#endif




#if 0
    // PARTICLE
    {
        float BF_TEX0_INV_RANDOM_U = GET_BIT_FLAG( BIT_FLAG_TEX0_INV_RANDOM_U );
        float BF_TEX0_INV_RANDOM_V = GET_BIT_FLAG( BIT_FLAG_TEX0_INV_RANDOM_V );
        float BF_TEX1_INV_RANDOM_U = GET_BIT_FLAG( BIT_FLAG_TEX1_INV_RANDOM_U );
        float BF_TEX1_INV_RANDOM_V = GET_BIT_FLAG( BIT_FLAG_TEX1_INV_RANDOM_V );
        float BF_TEX2_INV_RANDOM_U = GET_BIT_FLAG( BIT_FLAG_TEX2_INV_RANDOM_U );
        float BF_TEX2_INV_RANDOM_V = GET_BIT_FLAG( BIT_FLAG_TEX2_INV_RANDOM_V );
        BF_TEX0_INV_RANDOM_U *= GET_FLAG_GREATER_EQ( sysPtclRandomX, 0.5 );
        BF_TEX0_INV_RANDOM_U *= GET_FLAG_GREATER_EQ( sysPtclRandomY, 0.5 );
        BF_TEX0_INV_RANDOM_U *= GET_FLAG_GREATER_EQ( sysPtclRandomZ, 0.5 );
        BF_TEX0_INV_RANDOM_U *= GET_FLAG_GREATER_EQ( sysPtclRandomW, 0.5 );
        BF_TEX0_INV_RANDOM_U *= GET_FLAG_GREATER_EQ( sysPtclRandomY, 0.5 );
        BF_TEX0_INV_RANDOM_U *= GET_FLAG_GREATER_EQ( sysPtclRandomZ, 0.5 );
        inUv0.x += 2 * BF_TEX0_INV_RANDOM_U * (-inUv0.x);
        inUv0.y += 2 * BF_TEX0_INV_RANDOM_V * (-inUv0.y);
        inUv1.x += 2 * BF_TEX1_INV_RANDOM_U * (-inUv1.x);
        inUv1.y += 2 * BF_TEX1_INV_RANDOM_V * (-inUv1.y);
        inUv2.x += 2 * BF_TEX2_INV_RANDOM_U * (-inUv2.x);
        inUv2.y += 2 * BF_TEX2_INV_RANDOM_V * (-inUv2.y);
    }
    // PRIMITIVE
    {
        float BF_TEX0_INV_RANDOM_U = GET_BIT_FLAG( BIT_FLAG_TEX0_INV_RANDOM_U );
        float BF_TEX0_INV_RANDOM_V = GET_BIT_FLAG( BIT_FLAG_TEX0_INV_RANDOM_V );
        float BF_TEX1_INV_RANDOM_U = GET_BIT_FLAG( BIT_FLAG_TEX1_INV_RANDOM_U );
        float BF_TEX1_INV_RANDOM_V = GET_BIT_FLAG( BIT_FLAG_TEX1_INV_RANDOM_V );
        float BF_TEX2_INV_RANDOM_U = GET_BIT_FLAG( BIT_FLAG_TEX2_INV_RANDOM_U );
        float BF_TEX2_INV_RANDOM_V = GET_BIT_FLAG( BIT_FLAG_TEX2_INV_RANDOM_V );
        BF_TEX0_INV_RANDOM_U *= GET_FLAG_GREATER_EQ( sysPtclRandomX, 0.5 );
        BF_TEX0_INV_RANDOM_U *= GET_FLAG_GREATER_EQ( sysPtclRandomY, 0.5 );
        BF_TEX0_INV_RANDOM_U *= GET_FLAG_GREATER_EQ( sysPtclRandomZ, 0.5 );
        BF_TEX0_INV_RANDOM_U *= GET_FLAG_GREATER_EQ( sysPtclRandomW, 0.5 );
        BF_TEX0_INV_RANDOM_U *= GET_FLAG_GREATER_EQ( sysPtclRandomY, 0.5 );
        BF_TEX0_INV_RANDOM_U *= GET_FLAG_GREATER_EQ( sysPtclRandomZ, 0.5 );
        inUv0.x = BF_TEX0_INV_RANDOM_U * ( 1.0 -inUv0.x ) + ( 1 - BF_TEX0_INV_RANDOM_U ) * ( inUv0.x );
        inUv0.y = BF_TEX0_INV_RANDOM_V * ( 1.0 -inUv0.y ) + ( 1 - BF_TEX0_INV_RANDOM_V ) * ( inUv0.y );
        inUv1.x = BF_TEX1_INV_RANDOM_U * ( 1.0 -inUv1.x ) + ( 1 - BF_TEX1_INV_RANDOM_U ) * ( inUv1.x );
        inUv1.y = BF_TEX1_INV_RANDOM_V * ( 1.0 -inUv1.y ) + ( 1 - BF_TEX1_INV_RANDOM_V ) * ( inUv1.y );
        inUv2.x = BF_TEX2_INV_RANDOM_U * ( 1.0 -inUv2.x ) + ( 1 - BF_TEX2_INV_RANDOM_U ) * ( inUv2.x );
        inUv2.y = BF_TEX2_INV_RANDOM_V * ( 1.0 -inUv2.y ) + ( 1 - BF_TEX2_INV_RANDOM_V ) * ( inUv2.y );
    }
#endif

