/*---------------------------------------------------------------------------*
  Project:  NintendoWare
  File:     eft_ParticleDeclaration.vsh
  
  Copyright (C)2011-2013 Nintendo All rights reserved.
  
  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.
  
  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

//------------------------------------------------------------------------------
// 定義
//------------------------------------------------------------------------------
// 頂点シェーダであることの定義
#define VERTEX_SHADER

// テクスチャ座標系
#ifdef TARGET_WINDOWS
#define TEXCOORD_ADJUSTMENT                         (0.5)
#endif
#ifdef TARGET_CAFE
#define TEXCOORD_ADJUSTMENT                         (-0.5)
#endif

// テクスチャディメンション
#define VERTEX_TEXTURE_DIM_2D                       (0)
#define VERTEX_TEXTURE_DIM_2D_ARRAY                 (1)

// スフィアマップを処理するか
#if defined( _TEX_0_SPHERE_MAP ) || defined( _TEX_1_SPHERE_MAP ) || defined( _TEX_2_SPHERE_MAP )
#define _USE_SPHERE_MAP
#endif

// ストライプを使用するか
#if defined( _STRIPE_BILLBOARD ) || defined( _STRIPE_EMITTER_MATRIX ) || defined( _STRIPE_EMITTER_UPDOWN )
#define _USE_STRIPE
#endif

// スケールアニメを処理するか
#if defined( _SCALE_ANIM_1_KEY ) || defined( _SCALE_ANIM_2_KEY ) || defined( _SCALE_ANIM_3_KEY ) || defined( _SCALE_ANIM_4_KEY ) || defined( _SCALE_ANIM_5_KEY ) || defined( _SCALE_ANIM_6_KEY ) || defined( _SCALE_ANIM_7_KEY ) || defined( _SCALE_ANIM_8_KEY )
#define _SCALE_ANIM
#endif
// カラー0アニメを処理するか
#if defined( _COLOR_0_ANIM_1_KEY ) || defined( _COLOR_0_ANIM_2_KEY ) || defined( _COLOR_0_ANIM_3_KEY ) || defined( _COLOR_0_ANIM_4_KEY ) || defined( _COLOR_0_ANIM_5_KEY ) || defined( _COLOR_0_ANIM_6_KEY ) || defined( _COLOR_0_ANIM_7_KEY ) || defined( _COLOR_0_ANIM_8_KEY )
#define _COLOR_0_ANIM
#endif
// カラー1アニメを処理するか
#if defined( _COLOR_1_ANIM_1_KEY ) || defined( _COLOR_1_ANIM_2_KEY ) || defined( _COLOR_1_ANIM_3_KEY ) || defined( _COLOR_1_ANIM_4_KEY ) || defined( _COLOR_1_ANIM_5_KEY ) || defined( _COLOR_1_ANIM_6_KEY ) || defined( _COLOR_1_ANIM_7_KEY ) || defined( _COLOR_1_ANIM_8_KEY )
#define _COLOR_1_ANIM
#endif
// アルファ0アニメを処理するか
#if defined( _ALPHA_0_ANIM_1_KEY ) || defined( _ALPHA_0_ANIM_2_KEY ) || defined( _ALPHA_0_ANIM_3_KEY ) || defined( _ALPHA_0_ANIM_4_KEY ) || defined( _ALPHA_0_ANIM_5_KEY ) || defined( _ALPHA_0_ANIM_6_KEY ) || defined( _ALPHA_0_ANIM_7_KEY ) || defined( _ALPHA_0_ANIM_8_KEY )
#define _ALPHA_0_ANIM
#endif
// アルファ1アニメを処理するか
#if defined( _ALPHA_1_ANIM_1_KEY ) || defined( _ALPHA_1_ANIM_2_KEY ) || defined( _ALPHA_1_ANIM_3_KEY ) || defined( _ALPHA_1_ANIM_4_KEY ) || defined( _ALPHA_1_ANIM_5_KEY ) || defined( _ALPHA_1_ANIM_6_KEY ) || defined( _ALPHA_1_ANIM_7_KEY ) || defined( _ALPHA_1_ANIM_8_KEY )
#define _ALPHA_1_ANIM
#endif




//------------------------------------------------------------------------------
// 入力アトリビュート
//------------------------------------------------------------------------------

// 頂点単位
in vec4     sysPosAttr;                                     // 入力頂点 ( -0.5, 0.5 ) -> ( 0.5, -0.5 )
in vec2     sysTexCoordAttr;                                // 入力テクスチャ座標　(プリミティブ利用時)
in vec3     sysNormalAttr;                                  // 入力法線　　　　　　(プリミティブ利用時)
in vec3     sysTangentAttr;                                 // 入力接線　　　　　　(プリミティブ利用時)
in vec4     sysVertexColor0Attr;                            // 入力カラー　　　　　(プリミティブ/ストライプ利用時)
in vec4     sysVertexColor1Attr;                            // 入力カラー　　　　　(プリミティブ/ストライプ利用時)

uint        sysIndexAttr;                                   // 入力インデックス ( 0, 1, 2, 3 ) / sysPosAttr.w から取得

// パーティクル単位
in vec4     sysLocalPosAttr;                                // ローカル座標 / パーティクル寿命
in vec4     sysLocalVecAttr;                                // ローカルベクトル / エミッタ放出時タイム
in vec4     sysLocalDiffAttr;                               // ローカル位置差分 / 空き
in vec4     sysScaleAttr;                                   // パーティクルスケール(xyz) / 運動量ランダム
in vec4     sysRandomAttr;                                  // パーティクル固有ランダム
in vec4     sysInitRotateAttr;                              // パーティクル初期回転 / エミッタ放出時タイム
in vec4     sysEmtMat0Attr;                                 // エミッタマトリクス0
in vec4     sysEmtMat1Attr;                                 // エミッタマトリクス1
in vec4     sysEmtMat2Attr;                                 // エミッタマトリクス2
in vec4     sysEmtRTMat0Attr;                               // エミッタRTマトリクス0
in vec4     sysEmtRTMat1Attr;                               // エミッタRTマトリクス1
in vec4     sysEmtRTMat2Attr;                               // エミッタRTマトリクス2
in vec4     sysColor0Attr;                                  // パーティクル初期カラー0
in vec4     sysColor1Attr;                                  // パーティクル初期カラー1

// ストリームアウト
out vec4    sysOutPos;
out vec4    sysOutVec;
in  vec4    sysInPos;
in  vec4    sysInVec;

// エミッタプラグイン
in vec4     sysEmitterPluginAttr0;                          // エミッタプラグイン用入力アトリビュート0
in vec4     sysEmitterPluginAttr1;                          // エミッタプラグイン用入力アトリビュート1
in vec4     sysEmitterPluginAttr2;                          // エミッタプラグイン用入力アトリビュート2
in vec4     sysEmitterPluginAttr3;                          // エミッタプラグイン用入力アトリビュート3
in vec4     sysEmitterPluginAttr4;                          // エミッタプラグイン用入力アトリビュート4

//------------------------------------------------------------------------------
// フラグメントシェーダに流す値
//------------------------------------------------------------------------------
out vec4    sysColor0Vary;                                  // カラー0
out vec4    sysColor1Vary;                                  // カラー1
out vec4    sysTexCoord01Vary;                              // テクスチャ座標0-1
out vec2    sysTexCoord2Vary;                               // テクスチャ座標2
out vec4    sysFlagCoordVary;                               // スクリーン座標参照用
out vec4    sysFlagParamVary;                               // 各種パラメータ( alphaRate )
out vec4    sysPrimitiveColorVary;                          // プリミティブカラー

out vec3    sysWorldPositionVary;                           // ワールド座標
out vec3    sysWorldNormalVary;                             // ワールド法線
out vec3    sysWorldTangentVary;                            // ワールド接線
out vec3    sysWorldBinormalVary;                           // ワールド従法線
out vec3    sysEyeVecVary;                                  // 視線ベクトル

//out vec2  sysTexSurfaceAnimVary;                          // テクスチャサーフェイスアニメ
//out vec4  sysTex0SRTAnimProj;                             // プロジェクションテクスチャ座標0(テクスチャ0アニメ適用)
//out vec4  sysTex1SRTAnimProj;                             // プロジェクションテクスチャ座標1(テクスチャ1アニメ適用)
//out vec4  sysTex2SRTAnimProj;                             // プロジェクションテクスチャ座標2(テクスチャ1アニメ適用)
//out vec2  sysSphereTexCoordVary;                          // スフィアマップテクスチャ座標

#define sysMasterAlpha sysFlagParamVary.x                   // マスターアルファ
#define sysShaderAnim  sysFlagParamVary.z                   // シェーダアニメ

//------------------------------------------------------------------------------
// エミッタRTマトリクス MakesysEmitterRTMatrixで生成されます。
//------------------------------------------------------------------------------
vec4 sysEmitterRT[3];   // 今のEmitterの位置を表すRTマトリクス。


//------------------------------------------------------------------------------
// パーティクル情報
//------------------------------------------------------------------------------
float   sysPtclTime;                                        // パーティクル タイム
float   sysPtclAlphaRate;                                   // パーティクル アルファ係数
vec3    sysPtclScale;                                       // パーティクルスケール
vec3    sysPtclLocalPos;                                    // パーティクルローカル座標
vec3    sysPtclLocalVec;                                    // パーティクルローカル速度
vec3    sysPtclWorldPos;                                    // パーティクル世界座標
vec3    sysPtclWorldPosDelta;                               // パーティクル移動差分
mat4    sysPtclRotateMatrix;                                // パーティクル回転行列
vec4    sysPtclFlagCoordCenter;                             // スクリーン座標での位置
float   sysPtclVertexDepthValue;                            // 頂点のデプス値
float   sysPtclVertexDepthValueCenter;                      // 頂点のデプス値（パーティクルの中心）
float   sysPtclTextureDepthValue;                           // テクスチャのデプス値
#define sysPtclRandomX         sysRandomAttr.x              // パーティクル 固有ランダムX
#define sysPtclRandomY         sysRandomAttr.y              // パーティクル 固有ランダムY
#define sysPtclRandomZ         sysRandomAttr.z              // パーティクル 固有ランダムZ
#define sysPtclRandomW         sysRandomAttr.w              // パーティクル 固有ランダムW

//#ifdef _CALC_TYPE_GPU_TIME
#define sysPtclEmitTime        sysLocalVecAttr.w            // パーティクル 放出エミッタ時間
//#else
//#define sysPtclEmitTime        sysInitRotateAttr.w          // パーティクル 放出エミッタ時間
//#endif
#define sysPtclLife            sysLocalPosAttr.w            // パーティクル 寿命



//------------------------------------------------------------------------------
// テクスチャ座標変換行列
// HalfZ利用時は transformTexture を undef して、置き換えを行います。
//------------------------------------------------------------------------------
#ifdef TARGET_WINDOWS
    mat4 matTexture = mat4( 0.5,  0.0, 0.0, 0.0, 
                            0.0,  0.5, 0.0, 0.0, 
                            0.0,  0.0, 0.5, 0.0, 
                            0.5,  0.5, 0.5, 1.0 );
#endif
#ifdef TARGET_CAFE
    mat4 matTexture = mat4( 0.5,  0.0, 0.0, 0.0, 
                            0.0, -0.5, 0.0, 0.0, 
                            0.0,  0.0, 0.5, 0.0, 
                            0.5,  0.5, 0.5, 1.0 );
#endif

// HalfZ利用時は、#undef してください。
#define transformTexture matTexture 

//------------------------------------------------------------------------------
// 互換性保持
//------------------------------------------------------------------------------
// 頂点単位
#define v_inPos             sysPosAttr
#define v_inIndex           sysIndexAttr
#define v_inTexCoord        sysTexCoordAttr
#define v_inNormal          sysNormalAttr
#define v_inColor           sysVertexColor0Attr
//#define v_inTexCoord1   

// パーティクル単位
#define v_inWldPos          sysPtclWorldPos
#define v_inScl             sysScaleAttr
//#define v_inWldPosDf       
#define v_inColor0          sysColor0Attr
#define v_inColor1          sysColor1Attr
//#define v_inRot           sysInitRotateAttr
#define v_inEmtMat0         sysEmtMat0Attr
#define v_inEmtMat1         sysEmtMat1Attr
#define v_inEmtMat2         sysEmtMat2Attr
#define v_inRandom          sysRandomAttr
#define v_inVec             sysLocalVecAttr
#define inScaleX            v_inScl.x       
#define inScaleY            v_inScl.y       
#define inWorldPos          v_inWldPos.xyz  
#define inLocalPos          v_inWldPos.xyz  
//#define inWorldPosDf      v_inWldPosDf.xyz
#define inColor0            v_inColor0      
#define inColor1            v_inColor1      
//#define inRotate          v_inRot.xyz     
//#define inIniVec          v_inVec.xyz     
//#define inDynamicsRand    v_inVec.w       
#define ptclRandomX         v_inRandom.x    
#define ptclRandomY         v_inRandom.y    
#define ptclRandomZ         v_inRandom.z    
//#define ptclLife          v_inRandom.w    

// フラグメントシェーダに渡す値
//#define v_outPos                                     
#define v_outWorldPosition  sysWorldPositionVary       
#define v_outWorldNormal    sysWorldNormalVary         
#define v_outWorldTangent   sysWorldTangentVary                         
#define v_outWorldBinormal  sysWorldBinormalVary                         
#define g_worldNormal       v_outWorldNormal
#define g_worldTangent      v_outWorldTangent
#define g_worldBinormal     v_outWorldBinormal
#define g_worldVertexPos    v_outWorldPosition

#define v_outColor0         sysColor0Vary
#define v_outColor1         sysColor1Vary
#define v_outPrimitiveColor sysWorldPrimitiveColorVary 
#define v_outTexCoord       sysTexCoord01Vary
#define v_outTexCoord0      v_outTexCoord.xy           
#define v_outTexCoord1      v_outTexCoord.zw           
#define v_outTexCoord2      sysTexCoord2Vary           
#define v_outEyeVec         sysEyeVecVary              
//#define v_outDepthTexCoordProj           
//#define v_outFrameTexCoordProj           
//#define v_outSphereTexCoord              
//#define v_outProjectionTexSRT0CoordProj  
//#define v_outProjectionTexSRT1CoordProj  
//#define v_outTexSurfaceAnim              

#ifdef CUSTOM_SHADER_NONE
#define USR_VERTEX_SETTING_NONE
#endif
#ifdef CUSTOM_SHADER_INDEX_1
#define USR_VERTEX_SETTING_1
#endif
#ifdef CUSTOM_SHADER_INDEX_2
#define USR_VERTEX_SETTING_2
#endif
#ifdef CUSTOM_SHADER_INDEX_3
#define USR_VERTEX_SETTING_3
#endif
#ifdef CUSTOM_SHADER_INDEX_4
#define USR_VERTEX_SETTING_4
#endif
#ifdef CUSTOM_SHADER_INDEX_5
#define USR_VERTEX_SETTING_5
#endif
#ifdef CUSTOM_SHADER_INDEX_6
#define USR_VERTEX_SETTING_6
#endif
#ifdef CUSTOM_SHADER_INDEX_7
#define USR_VERTEX_SETTING_7
#endif
#ifdef CUSTOM_SHADER_INDEX_8
#define USR_VERTEX_SETTING_8
#endif
#ifdef CUSTOM_SHADER_FLAG_0
#define USR_VERTEX_FLAG_0
#endif
#ifdef CUSTOM_SHADER_FLAG_1
#define USR_VERTEX_FLAG_1
#endif
#ifdef CUSTOM_SHADER_FLAG_2
#define USR_VERTEX_FLAG_2
#endif
#ifdef CUSTOM_SHADER_FLAG_3
#define USR_VERTEX_FLAG_3
#endif
#ifdef CUSTOM_SHADER_FLAG_4
#define USR_VERTEX_FLAG_4
#endif
#ifdef CUSTOM_SHADER_FLAG_5
#define USR_VERTEX_FLAG_5
#endif
#ifdef CUSTOM_SHADER_FLAG_6
#define USR_VERTEX_FLAG_6
#endif
#ifdef CUSTOM_SHADER_FLAG_7
#define USR_VERTEX_FLAG_7
#endif
#ifdef CUSTOM_SHADER_FLAG_8
#define USR_VERTEX_FLAG_8
#endif
#ifdef CUSTOM_SHADER_FLAG_9
#define USR_VERTEX_FLAG_9
#endif
#ifdef CUSTOM_SHADER_FLAG_10
#define USR_VERTEX_FLAG_10
#endif
#ifdef CUSTOM_SHADER_FLAG_11
#define USR_VERTEX_FLAG_11
#endif
#ifdef CUSTOM_SHADER_FLAG_12
#define USR_VERTEX_FLAG_12
#endif
#ifdef CUSTOM_SHADER_FLAG_13
#define USR_VERTEX_FLAG_13
#endif
#ifdef CUSTOM_SHADER_FLAG_14
#define USR_VERTEX_FLAG_14
#endif
#ifdef CUSTOM_SHADER_FLAG_15
#define USR_VERTEX_FLAG_15
#endif
#ifdef CUSTOM_SHADER_FLAG_16
#define USR_VERTEX_FLAG_16
#endif
#ifdef CUSTOM_SHADER_FLAG_17
#define USR_VERTEX_FLAG_17
#endif
#ifdef CUSTOM_SHADER_FLAG_18
#define USR_VERTEX_FLAG_18
#endif
#ifdef CUSTOM_SHADER_FLAG_19
#define USR_VERTEX_FLAG_19
#endif
#ifdef CUSTOM_SHADER_FLAG_20
#define USR_VERTEX_FLAG_20
#endif
#ifdef CUSTOM_SHADER_FLAG_21
#define USR_VERTEX_FLAG_21
#endif
#ifdef CUSTOM_SHADER_FLAG_22
#define USR_VERTEX_FLAG_22
#endif
#ifdef CUSTOM_SHADER_FLAG_23
#define USR_VERTEX_FLAG_23
#endif
#ifdef CUSTOM_SHADER_FLAG_24
#define USR_VERTEX_FLAG_24
#endif
#ifdef CUSTOM_SHADER_FLAG_25
#define USR_VERTEX_FLAG_25
#endif
#ifdef CUSTOM_SHADER_FLAG_26
#define USR_VERTEX_FLAG_26
#endif
#ifdef CUSTOM_SHADER_FLAG_27
#define USR_VERTEX_FLAG_27
#endif
#ifdef CUSTOM_SHADER_FLAG_28
#define USR_VERTEX_FLAG_28
#endif
#ifdef CUSTOM_SHADER_FLAG_29
#define USR_VERTEX_FLAG_29
#endif
#ifdef CUSTOM_SHADER_FLAG_30
#define USR_VERTEX_FLAG_30
#endif
#ifdef CUSTOM_SHADER_FLAG_31
#define USR_VERTEX_FLAG_31
#endif

//---------------------------------------------------
// カスタムシェーダ／エミッタプラグイン用コールバック関数前方宣言
//---------------------------------------------------
void AdjustmentWorldPosition();             // カスタムシェーダ: 頂点位置補正
void FinalAdjustmentVertexProcess();        // カスタムシェーダ: 最終位置補正
void EP_AdjustmentWorldPosition();          // エミッタプラグイン: 頂点位置補正
void EP_FinalAdjustmentVertexProcess();     // エミッタプラグイン: 最終位置補正

//---------------------------------------------------
// デプステクスチャからデプス値を取得する( カスタマイズ可能 )
//---------------------------------------------------
#ifndef USE_USR_GET_DEPTH_VALUE_FROM_TEXTURE
float GetDepthValueFromTexture( vec4 projection )
{
    float depth = texture2DProj( sysDepthBufferTexture, projection ).r;
    depth = - ( sysScreenFarMultNearFar ) / ( depth  * ( sysScreenSubFarNear ) - sysScreenFar );
    return depth;
}
#endif

//------------------------------------------------------------------------------
// この頂点を棄却します
//------------------------------------------------------------------------------
void DiscardVertex()
{
    // カメラから見えない点に飛ばしてカリングさせる。
    // フラグメントシェーダに入れないのが目的。
    gl_Position.z = sysScreenFar * 5;
    gl_Position.x = gl_Position.y = 0;
    sysMasterAlpha = 0;
}

//---------------------------------------------------
// デプス値を取得する( カスタマイズ可能 )
//---------------------------------------------------
#ifndef USE_USR_GET_DEPTH_VALUE
float GetDepthValue( vec4 position )
{
    float depth = position.z / position.w;
    depth = - ( sysScreenFarMultNearFar ) / ( depth * ( sysScreenSubFarNear ) - sysScreenFar );
    return depth;
}
#endif

//---------------------------------------------------
// エミッタRTマトリクスを生成します。
// 生成されたマトリクスは、sysEmitterRT[3]に出力されます。
//---------------------------------------------------
void MakeEmitterRTMatrix()
{
#ifdef _CALC_TYPE_CPU
    // CPUの場合

#ifdef _EMITTER_FOLLOW_TYPE_ALL
    // 完全追従
    sysEmitterRT[0] = sysEmitterRTMatrix[0];
    sysEmitterRT[1] = sysEmitterRTMatrix[1];
    sysEmitterRT[2] = sysEmitterRTMatrix[2];
#endif

#ifdef _EMITTER_FOLLOW_TYPE_POS
    // 位置のみ追従

#if 0
    // 入ってるのはSRTマトリクスなので、RTマトリクスを生成。
    vec3 basisX = vec3( sysEmtMat0Attr.x, sysEmtMat1Attr.x, sysEmtMat2Attr.x );
    vec3 basisY = vec3( sysEmtMat0Attr.y, sysEmtMat1Attr.y, sysEmtMat2Attr.y );
    vec3 basisZ = vec3( sysEmtMat0Attr.z, sysEmtMat1Attr.z, sysEmtMat2Attr.z );

    float emScaleX = length( basisX );
    float emScaleY = length( basisY );
    float emScaleZ = length( basisZ );

    if( emScaleX > 0 )
    {
        float inv = 1.0 / emScaleX;
        sysEmitterRT[0].x = sysEmtMat0Attr[0] * inv;
        sysEmitterRT[1].x = sysEmtMat1Attr[0] * inv;
        sysEmitterRT[2].x = sysEmtMat2Attr[0] * inv;
    }
    else
    {
        sysEmitterRT[0].x = sysEmitterRT[1].x = sysEmitterRT[2].x = 0.0f;
    }

    if( emScaleY > 0 )
    {
        float inv = 1.0f / emScaleY;
        sysEmitterRT[0].y = sysEmtMat0Attr[1] * inv;
        sysEmitterRT[1].y = sysEmtMat1Attr[1] * inv;
        sysEmitterRT[2].y = sysEmtMat2Attr[1] * inv;
    }
    else
    {
        sysEmitterRT[0].y = sysEmitterRT[1].y = sysEmitterRT[2].y = 0.0f;
    }

    if( emScaleZ > 0 )
    {
        float inv = 1.0f / emScaleY;
        sysEmitterRT[0].z = sysEmtMat0Attr[2] * inv;
        sysEmitterRT[1].z = sysEmtMat1Attr[2] * inv;
        sysEmitterRT[2].z = sysEmtMat2Attr[2] * inv;
    }
    else
    {
        sysEmitterRT[0].z = sysEmitterRT[1].z = sysEmitterRT[2].z = 0.0f;
    }
    sysEmitterRT[0].w = sysEmtMat0Attr.w;
    sysEmitterRT[1].w = sysEmtMat1Attr.w;
    sysEmitterRT[2].w = sysEmtMat2Attr.w;
#else
    sysEmitterRT[0] = vec4( sysEmtRTMat0Attr.xyz, sysEmitterMatrix[0].w );
    sysEmitterRT[1] = vec4( sysEmtRTMat1Attr.xyz, sysEmitterMatrix[1].w );
    sysEmitterRT[2] = vec4( sysEmtRTMat2Attr.xyz, sysEmitterMatrix[2].w );
#endif
#endif

#ifdef _EMITTER_FOLLOW_TYPE_NONE
    // 追従なし
    sysEmitterRT[0] = sysEmtRTMat0Attr;
    sysEmitterRT[1] = sysEmtRTMat1Attr;
    sysEmitterRT[2] = sysEmtRTMat2Attr;
#endif

#else

    // GPU/GPU+SOの場合

#ifdef _EMITTER_FOLLOW_TYPE_ALL
    // 完全追従の場合、エミッタのSRTを使用

    vec3 basisX = sysEmitterRTMatrix[0].xyz;
    vec3 basisY = sysEmitterRTMatrix[1].xyz;
    vec3 basisZ = sysEmitterRTMatrix[2].xyz;

    float emScaleX = length( basisX );
    float emScaleY = length( basisY );
    float emScaleZ = length( basisZ );

    if( emScaleX > 0 )
    {
        float inv = 1.0f / emScaleX;
        sysEmitterRT[0].x = sysEmitterRTMatrix[0].x * inv;
        sysEmitterRT[1].x = sysEmitterRTMatrix[1].x * inv;
        sysEmitterRT[2].x = sysEmitterRTMatrix[2].x * inv;
    }
    else
    {
        sysEmitterRT[0].x = sysEmitterRT[1].x = sysEmitterRT[2].x = 0.0f;
    }

    if( emScaleY > 0 )
    {
        float inv = 1.0f / emScaleY;
        sysEmitterRT[0].y = sysEmitterRTMatrix[0].y * inv;
        sysEmitterRT[1].y = sysEmitterRTMatrix[1].y * inv;
        sysEmitterRT[2].y = sysEmitterRTMatrix[2].y * inv;
    }
    else
    {
        sysEmitterRT[0].y = sysEmitterRT[1].y = sysEmitterRT[2].y = 0.0f;
    }

    if( emScaleZ > 0 )
    {
        float inv = 1.0f / emScaleY;
        sysEmitterRT[0].z = sysEmitterRTMatrix[0].z * inv;
        sysEmitterRT[1].z = sysEmitterRTMatrix[1].z * inv;
        sysEmitterRT[2].z = sysEmitterRTMatrix[2].z * inv;
    }
    else
    {
        sysEmitterRT[0].z = sysEmitterRT[1].z = sysEmitterRT[2].z = 0.0f;
    }

    sysEmitterRT[0].w = sysEmitterRTMatrix[0].w;
    sysEmitterRT[1].w = sysEmitterRTMatrix[1].w;
    sysEmitterRT[2].w = sysEmitterRTMatrix[2].w;

#else
    // それ以外の場合、パーティクルごとのSRTを使用

    vec3 basisX = vec3( sysEmtMat0Attr.x, sysEmtMat1Attr.x, sysEmtMat2Attr.x );
    vec3 basisY = vec3( sysEmtMat0Attr.y, sysEmtMat1Attr.y, sysEmtMat2Attr.y );
    vec3 basisZ = vec3( sysEmtMat0Attr.z, sysEmtMat1Attr.z, sysEmtMat2Attr.z );

    float emScaleX = length( basisX );
    float emScaleY = length( basisY );
    float emScaleZ = length( basisZ );

    if( emScaleX > 0 )
    {
        float inv = 1.0f / emScaleX;
        sysEmitterRT[0].x = sysEmtMat0Attr.x * inv;
        sysEmitterRT[1].x = sysEmtMat1Attr.x * inv;
        sysEmitterRT[2].x = sysEmtMat2Attr.x * inv;
    }
    else
    {
        sysEmitterRT[0].x = sysEmitterRT[1].x = sysEmitterRT[2].x = 0.0f;
    }

    if( emScaleY > 0 )
    {
        float inv = 1.0f / emScaleY;
        sysEmitterRT[0].y = sysEmtMat0Attr.y * inv;
        sysEmitterRT[1].y = sysEmtMat1Attr.y * inv;
        sysEmitterRT[2].y = sysEmtMat2Attr.y * inv;
    }
    else
    {
        sysEmitterRT[0].y = sysEmitterRT[1].y = sysEmitterRT[2].y = 0.0f;
    }

    if( emScaleZ > 0 )
    {
        float inv = 1.0f / emScaleY;
        sysEmitterRT[0].z = sysEmtMat0Attr.z * inv;
        sysEmitterRT[1].z = sysEmtMat1Attr.z * inv;
        sysEmitterRT[2].z = sysEmtMat2Attr.z * inv;
    }
    else
    {
        sysEmitterRT[0].z = sysEmitterRT[1].z = sysEmitterRT[2].z = 0.0f;
    }

    sysEmitterRT[0].w = sysEmtMat0Attr.w;
    sysEmitterRT[1].w = sysEmtMat1Attr.w;
    sysEmitterRT[2].w = sysEmtMat2Attr.w;

#endif

#endif
}

//---------------------------------------------------
// 引数vにエミッタRTマトリクスを適用します。
//---------------------------------------------------
vec3 ApplyEmitterRotateMatrix( vec3 v )
{
    vec4 ret;
#ifdef _EMITTER_FOLLOW_TYPE_ALL
    mat4    tempMat;
    tempMat[0] = vec4( sysEmitterMatrix[0].xyz, 0.0 );
    tempMat[1] = vec4( sysEmitterMatrix[1].xyz, 0.0 );
    tempMat[2] = vec4( sysEmitterMatrix[2].xyz, 0.0 );
    tempMat[3] = vec4( 0.0, 0.0, 0.0, 1.0 );
    ret = vec4( v, 1 ) * tempMat;
#else
    mat4    tempMat;
    tempMat[0] = vec4( sysEmitterRT[0].xyz, 0.0 );
    tempMat[1] = vec4( sysEmitterRT[1].xyz, 0.0 );
    tempMat[2] = vec4( sysEmitterRT[2].xyz, 0.0 );
    tempMat[3] = vec4( 0.0, 0.0, 0.0, 1.0 );
    ret = vec4( v, 1 ) * tempMat;
#endif

    return ret.xyz;
}

//---------------------------------------------------
// 引数vにエミッタRTマトリクスの逆行列を適用します。
//---------------------------------------------------
vec3 ApplyEmitterRotateInvMatrix( vec3 v )
{
    // 回転行列なので、転置をとれば逆行列になる。
    vec4 ret;
#ifdef _EMITTER_FOLLOW_TYPE_ALL
    mat4    tempMat;
    tempMat[0] = vec4( sysEmitterMatrix[0].xyz, 0.0 );
    tempMat[1] = vec4( sysEmitterMatrix[1].xyz, 0.0 );
    tempMat[2] = vec4( sysEmitterMatrix[2].xyz, 0.0 );
    tempMat[3] = vec4( 0.0, 0.0, 0.0, 1.0 );
    ret = vec4( v, 1 ) * transpose(tempMat);
#else
    mat4    tempMat;
    tempMat[0] = vec4( sysEmitterRT[0].xyz, 0.0 );
    tempMat[1] = vec4( sysEmitterRT[1].xyz, 0.0 );
    tempMat[2] = vec4( sysEmitterRT[2].xyz, 0.0 );
    tempMat[3] = vec4( 0.0, 0.0, 0.0, 1.0 );
    ret = vec4( v, 1 ) * transpose(tempMat);
#endif
    return ret.xyz;
}


//------------------------------------------------------------------------------
// 追従設定によるエミッタマトリクス反映
//------------------------------------------------------------------------------
vec3 ApplyEmitterMatrix( vec3 v )
{
    vec4 ret;

    //--------------------------------------------------------------
    // 完全追従
    //--------------------------------------------------------------
#ifdef _EMITTER_FOLLOW_TYPE_ALL
    return ( vec4( v, 1 ) * sysEmitterMatrix ).xyz;
#endif

    //--------------------------------------------------------------
    // 位置のみ追従
    //--------------------------------------------------------------
#ifdef _EMITTER_FOLLOW_TYPE_POS
    vec4 m0 = vec4( sysEmtMat0Attr.xyz, sysEmitterMatrix[0].w );
    vec4 m1 = vec4( sysEmtMat1Attr.xyz, sysEmitterMatrix[1].w );
    vec4 m2 = vec4( sysEmtMat2Attr.xyz, sysEmitterMatrix[2].w );

    ret.x = dot( m0 , vec4( v, 1 ) );
    ret.y = dot( m1 , vec4( v, 1 ) );
    ret.z = dot( m2 , vec4( v, 1 ) );
    ret.w = 1;
    return ret.xyz;
#endif

    //--------------------------------------------------------------
    // 追従しない
    //--------------------------------------------------------------
#ifdef _EMITTER_FOLLOW_TYPE_NONE
    ret.x = dot( sysEmtMat0Attr , vec4( v, 1 ) );
    ret.y = dot( sysEmtMat1Attr , vec4( v, 1 ) );
    ret.z = dot( sysEmtMat2Attr , vec4( v, 1 ) );
    ret.w = 1;
    return ret.xyz;
#endif

    return ret.xyz;
}



//---------------------------------------------------
// エミッタローカル位置を取得します。
//---------------------------------------------------
vec3 GetEmitterLocalPos()
{
    vec4 ret;
    ret.x = sysEmitterMatrix[0].w;
    ret.y = sysEmitterMatrix[1].w;
    ret.z = sysEmitterMatrix[2].w;
    ret.w = 1;
    return ( ret * inverse( sysEmitterMatrix ) ).xyz;
}

//------------------------------------------------------------------------------
// x時点での乱数の積算値を取得
//------------------------------------------------------------------------------
float RandFunc( float x )
{
    float _2pi = 3.141592 * 2;
    float rCycle = rnd0.w;

#if 1
    // ★配列実装版（波の個数が可変）
#define NUM_OF_WAVES 4 
#define NUM_OF_RANDS 4

    // 振幅の固定幅
    float ampBase[ NUM_OF_WAVES ];
    if( rnd3.y == 1 )
    {
        ampBase[0] = rnd2.x;
        ampBase[1] = rnd2.y;
        ampBase[2] = rnd2.z;
        ampBase[3] = rnd2.w;
    }
    else
    {
        vec4 baseParam = vec4( 4, 3, 2, 1.5 );
        ampBase[0] = baseParam.x;
        ampBase[1] = baseParam.y;
        ampBase[2] = baseParam.z;
        ampBase[3] = baseParam.w;
    }

    // ★振幅のランダム幅
    float ampDelta[ NUM_OF_WAVES ];
    // ランダム幅は無しに。
    ampDelta[0] = 0;
    ampDelta[1] = 0;
    ampDelta[2] = 0;
    ampDelta[3] = 0;

    float amp[ NUM_OF_WAVES ];
    for(int i=0; i<NUM_OF_WAVES; ++i)
    {
        amp[i] = ampBase[i];
    }

    // 周期
    float tRate[ NUM_OF_WAVES ];
    if( rnd3.y == 1 )
    {
        tRate[0] = rnd1.x;
        tRate[1] = rnd1.y;
        tRate[2] = rnd1.z;
        tRate[3] = rnd1.w;
    }
    else
    {
        vec4 baseRate = vec4( 0.6f, 0.42f, 0.23f, 0.15f );
        tRate[0] = baseRate.x;
        tRate[1] = baseRate.y;
        tRate[2] = baseRate.z;
        tRate[3] = baseRate.w;
    }

    float cycle[ NUM_OF_WAVES ];
    for(int i=0; i<NUM_OF_WAVES; ++i)
    {
        cycle[i] = tRate[i] * rCycle;
    }

    // 位相:
    float phase[ NUM_OF_WAVES ];
    for(int i=0; i<NUM_OF_WAVES; ++i)
    {
        phase[i] = ( x / ( cycle[i] ) );
    }

    // 基本となる波形
    float baseWave[ NUM_OF_WAVES ];
    for(int i=0; i<NUM_OF_WAVES; ++i)
    {
        baseWave[i] = sin( phase[i] * _2pi ) * amp[i];
    }

    float aggregate = 0;
    for(int i=0; i<NUM_OF_WAVES; ++i)
    {
        aggregate += baseWave[i];
    }

    return aggregate;

#undef NUM_OF_WAVES 
#undef NUM_OF_RANDS

#else
    // ★Vec4実装版（それほど速度は向上しない？）

    // 乱数
    vec4 val = vec4( sysPtclRandomX, sysPtclRandomY, sysPtclRandomZ, sysPtclRandomW );
    if( rnd3.x == 1 && rnd3.y == 1 ){ val = rnd4; }

    // 振幅
    vec4 ampBase = rnd2;
    if( rnd3.y == 0 ){ ampBase = vec4( 4, 3, 2, 1.5f ); }

    // 周期比率
    vec4 tRate = rnd1;
    if( rnd3.y == 0 ){ tRate = vec4( 0.6f, 0.42f, 0.23f, 0.15f ); }

    // 周期
    vec4 cycle = rnd1 * rCycle;

    // 位相
    vec4 phase = vec4( x/cycle.x, x/cycle.y, x/cycle.z, x/cycle.w ) * _2pi;

    // 波形を合成
    vec4 baseWave;
    baseWave.x = sin( phase.x );
    baseWave.y = sin( phase.y );
    baseWave.z = sin( phase.z );
    baseWave.w = sin( phase.w );
    baseWave *= ampBase;

    return dot( vec4( 1 ), baseWave );
    //return dot( vec4(1), baseWave.x + baseWave.y + baseWave.z + baseWave.w );
#endif
}

//------------------------------------------------------------------------------
// x時点での乱数の積算値を取得
//------------------------------------------------------------------------------
float _GetRandSum( float x )
{
    return RandFunc( x );
} 

//------------------------------------------------------------------------------
// [x1, x2]の区間の乱数の積算を取得
//------------------------------------------------------------------------------
float GetRandSum( float x1, float x2 )
{
    return _GetRandSum( x2 ) - _GetRandSum( x1 );
}

//------------------------------------------------------------------------------
// 現在の疑似乱数を取得
//------------------------------------------------------------------------------
float GetRand( float seed, float time, float deltaS, float deltaV )
{
    float x = seed + time;
    if( deltaS == 0 )
    {
        return ( _GetRandSum( x + deltaV + deltaS ) - _GetRandSum( x ) );
    }
    else
    {
        return ( _GetRandSum( x + deltaV + deltaS ) - _GetRandSum( x ) )
            - ( _GetRandSum( seed + deltaS ) - _GetRandSum( seed ) );
    }
}


//------------------------------------------------------------------------------
// テクスチャ座標アニメの計算
//------------------------------------------------------------------------------
vec4 CalculateTextureShiftAnim( vec2 inUv, ResTexUvShiftAnim anim,
                               float time, int index, int randomMode, vec4 randomSeed )
{
    vec4 ret;

    #define uvScrollAddX        anim.pm0.x
    #define uvScrollAddY        anim.pm0.y
    #define uvScrollInitX       anim.pm0.z
    #define uvScrollInitY       anim.pm0.w
    #define uvScrollInitRandX   anim.pm1.x
    #define uvScrollInitRandY   anim.pm1.y
    #define uvScaleAddX         anim.pm1.z
    #define uvScaleAddY         anim.pm1.w
    #define uvScaleInitX        anim.pm2.x
    #define uvScaleInitY        anim.pm2.y
    #define uvScaleInitRandX    anim.pm2.z
    #define uvScaleInitRandY    anim.pm2.w
    #define uvScaleBaseX        anim.pm4.x
    #define uvScaleBaseY        anim.pm4.y
    #define uvDivX              anim.pm4.z
    #define uvDivY              anim.pm4.w

    float random0 = randomSeed.x;
    float random1 = randomSeed.y;
    float random2 = randomSeed.z;
    float random3 = randomSeed.x;
    float random4 = randomSeed.y;

    if ( randomMode == 1 )
    {
        random0 = randomSeed.y;
        random1 = randomSeed.z;
        random2 = randomSeed.x;
        random3 = randomSeed.y;
        random4 = randomSeed.z;
    }
    else if ( randomMode == 2 )
    {
        random0 = randomSeed.z;
        random1 = randomSeed.x;
        random2 = randomSeed.y;
        random3 = randomSeed.x;
        random4 = randomSeed.y;
    }

    // 分割数
    int divX = int( uvDivX );
    int divY = int( uvDivY );

    // 分割後の表示位置
    int indexX = index % divX;
    int indexY = index / divX;

    // UVスケール
    float uScale = uvScaleBaseX * (1/uvDivX);
    float vScale = uvScaleBaseY * (1/uvDivY);

    // 初期値
    float uvScrInitX  = uvScrollInitX + uvScrollInitRandX - random0 * uvScrollInitRandX * 2.0;
    float uvScrInitY  = uvScrollInitY + uvScrollInitRandY - random1 * uvScrollInitRandY * 2.0;
    float uvSclInitX  = uvScaleInitX  + uvScaleInitRandX  + random3 * uvScaleInitRandX;
    float uvSclInitY  = uvScaleInitY  + uvScaleInitRandY  + random4 * uvScaleInitRandY;

    // 非ストライプ
    float scrX =  -uvScrInitX - uvScrollAddX * time + indexX * uScale;
    float scrY =   uvScrInitY + uvScrollAddY * time - indexY * vScale;
    float sclX =   uvSclInitX + uvScaleAddX  * time;
    float sclY =   uvSclInitY + uvScaleAddY  * time;

    // 返り値のzw には、スクロール値を代入する
    ret.z = scrX;
    ret.w = scrY;

#ifdef _PARTICLE

    ret.x = ( inUv.x + 0.5 ) *  uScale + scrX - ( 1.0 - sclX ) * inUv.x;
    ret.y = ( inUv.y - 0.5 ) * -vScale - scrY + ( 1.0 - sclY ) * inUv.y;

    //ret.x = ( inUv.x * uScale + 0.5 * uScale )   + scrX - ( inUv.x - sclX * inUv.x );
    //ret.y = ( inUv.y * -vScale - 0.5 * -vScale)  - scrY + ( inUv.y - sclY * inUv.y ) ;

#endif
#ifdef _PRIMITIVE
    float texu = ( inUv.x * uScale - 0.5 );
    float texv = ( inUv.y * vScale - 0.5 );
    ret.x = texu * sclX + scrX + 0.5;
    ret.y = texv * sclY - scrY + 0.5;
#endif

    return ret;
}


vec4 CalculateTextureShiftAnimWithRotate( vec2 inUv, ResTexUvShiftAnim anim,
                               float time, int index, int randomMode, vec4 randomSeed )
{
    vec4 ret;

    #define uvScrollAddX        anim.pm0.x
    #define uvScrollAddY        anim.pm0.y
    #define uvScrollInitX       anim.pm0.z
    #define uvScrollInitY       anim.pm0.w
    #define uvScrollInitRandX   anim.pm1.x
    #define uvScrollInitRandY   anim.pm1.y
    #define uvScaleAddX         anim.pm1.z
    #define uvScaleAddY         anim.pm1.w
    #define uvScaleInitX        anim.pm2.x
    #define uvScaleInitY        anim.pm2.y
    #define uvScaleInitRandX    anim.pm2.z
    #define uvScaleInitRandY    anim.pm2.w
    #define uvRotateAdd         anim.pm3.x
    #define uvRotateInit        anim.pm3.y
    #define uvRotateInitRand    anim.pm3.z
    #define uvRotateInit        anim.pm3.y
    #define uvRotateInitRand    anim.pm3.z
    #define uvScaleBaseX        anim.pm4.x
    #define uvScaleBaseY        anim.pm4.y
    #define uvDivX              anim.pm4.z
    #define uvDivY              anim.pm4.w

    float random0 = randomSeed.x;
    float random1 = randomSeed.y;
    float random2 = randomSeed.z;
    float random3 = randomSeed.x;
    float random4 = randomSeed.y;

    if ( randomMode == 1 )
    {
        random0 = randomSeed.y;
        random1 = randomSeed.z;
        random2 = randomSeed.x;
        random3 = randomSeed.y;
        random4 = randomSeed.z;
    }
    else if ( randomMode == 2 )
    {
        random0 = randomSeed.z;
        random1 = randomSeed.x;
        random2 = randomSeed.y;
        random3 = randomSeed.x;
        random4 = randomSeed.y;
    }

    // 分割数
    int divX = int( uvDivX );
    int divY = int( uvDivY );

    // 分割後の表示位置
    int indexX = index % divX;
    int indexY = index / divX;

    // UVスケール
    float uScale = uvScaleBaseX * (1/uvDivX);
    float vScale = uvScaleBaseY * (1/uvDivY);

    // 初期値
    float uvScrInitX  = uvScrollInitX + uvScrollInitRandX - random0 * uvScrollInitRandX * 2.0;
    float uvScrInitY  = uvScrollInitY + uvScrollInitRandY - random1 * uvScrollInitRandY * 2.0;
    float uvRotInit   = uvRotateInit  + uvRotateInitRand  + random2 * uvRotateInitRand;
    float uvSclInitX  = uvScaleInitX  + uvScaleInitRandX  + random3 * uvScaleInitRandX;
    float uvSclInitY  = uvScaleInitY  + uvScaleInitRandY  + random4 * uvScaleInitRandY;

    // 時間経過考慮
#ifdef _USE_STRIPE

    // ストライプ使用時
    float scrX =  -uvScrInitX + uvScrollAddX * time + indexX * uScale;
    float scrY =  uvScrInitY + uvScrollAddY * time - indexY * vScale;
    float rot  =  uvRotInit  + uvRotateAdd  * time;
    float sclX =  uvSclInitX + uvScaleAddX  * time;
    float sclY =  uvSclInitY + uvScaleAddY  * time;

    // 返り値のzw には、スクロール値を代入する
    ret.z = scrX;
    ret.w = scrY;

    float inposx = inUv.x - 0.5f;
    float inposy = inUv.y + 0.5f;

    float sinZ = sin( -rot );
    float cosZ = cos( -rot );

    float texu = inposx * cosZ - inposy * sinZ;
    float texv = inposx * sinZ + inposy * cosZ;
    ret.x = ( texu + 0.5 ) *  uScale + scrX - ( 1.0 - sclX ) * texu;
    ret.y = ( texv - 0.5 ) * -vScale - scrY + ( 1.0 - sclY ) * texv;

#else

    // 非ストライプ
    float scrX =  -uvScrInitX - uvScrollAddX * time + indexX * uScale;
    float scrY =  uvScrInitY + uvScrollAddY * time - indexY * vScale;
    float rot  =  uvRotInit  + uvRotateAdd  * time;
    float sclX =  uvSclInitX + uvScaleAddX  * time;
    float sclY =  uvSclInitY + uvScaleAddY  * time;

    // 返り値のzw には、スクロール値を代入する
    ret.z = scrX;
    ret.w = scrY;

    float sinZ = sin( -rot );
    float cosZ = cos( -rot );

#ifdef _PARTICLE
    float texu = inUv.x * cosZ - inUv.y * sinZ;
    float texv = inUv.x * sinZ + inUv.y * cosZ;
    ret.x = ( texu + 0.5 ) *  uScale + scrX - ( 1.0 - sclX ) * texu;
    ret.y = ( texv - 0.5 ) * -vScale - scrY + ( 1.0 - sclY ) * texv;
#endif
#ifdef _PRIMITIVE
    float texu = ( inUv.x * uScale - 0.5 ) * cosZ - ( inUv.y * vScale - 0.5 ) * sinZ;
    float texv = ( inUv.x * uScale - 0.5 ) * sinZ + ( inUv.y * vScale - 0.5 ) * cosZ;
    ret.x = texu * sclX + scrX + 0.5;
    ret.y = texv * sclY - scrY + 0.5;
#endif

#endif

    return ret;
}

//------------------------------------------------------------------------------
// ライフフィットアニメ処理
int CalculateTexPtnAnim_LifeFit( int table, int freq, float time, float life )
{
    int ptnIx = int( ( time * table ) / life );
    return ptnIx;
}

// ランダムアニメ処理
int CalculateTexPtnAnim_Random( int table, float random )
{
    return int( table * random );
}

// ループアニメ処理
int CalculateTexPtnAnim_Loop( int table, int freq, float time, float random, int bit )
{
    int ptnIx   = int( time / freq );
#if 1
    int f = GET_BIT_FLAG( bit );
    return ptnIx + f * int( table * random );
#else
    if ( CHECK_BIT_FLAG( bit ) )
    {
        ptnIx += int( table * random );
    }
    return ptnIx;
#endif
}

// クランプアニメ処理
int CalculateTexPtnAnim_Clamp( int table, int freq, float time )
{
#if 0
    int ptnIx = int( time / freq );

    // クランプコマの修正。
    if ( table <= ptnIx )
    {
        ptnIx = table-1;
    }
    else
    {
        ptnIx = ptnIx % table;
    }
    return ptnIx;
#else 
    int ptnIx = int( time / freq );
    // クランプコマの修正。
    int f = int( GET_FLAG_GREATER_EQ( ptnIx, table ) );
    int retA = ( table-1 );
    int retB = ( ptnIx % table );
    return f * retA + ( 1 - f ) * retB;
#endif
}

//------------------------------------------------------------------------------
// テクスチャパタンアニメ（マクロ版）
//------------------------------------------------------------------------------

#if 0
// テクスチャ パタン テーブルから値を引く
#define GET_TEXTURE_PTN_TABLE_DEFINE( ID, TEX_PAT_ANIM )    \
int GetTexturePtnTable##ID( int ptnIx, int table )          \
{                                                           \
    int ret = 0;                                            \
    int idx = ptnIx % table;                                \
    int a = idx % 4;                                        \
    if ( a == 0 ) ret = TEX_PAT_ANIM.t[ idx/4 ].x;          \
    if ( a == 1 ) ret = TEX_PAT_ANIM.t[ idx/4 ].y;          \
    if ( a == 2 ) ret = TEX_PAT_ANIM.t[ idx/4 ].z;          \
    if ( a == 3 ) ret = TEX_PAT_ANIM.t[ idx/4 ].w;          \
    return ret;                                             \
}                                                                 
#else 
#define GET_TEXTURE_PTN_TABLE_DEFINE( ID, TEX_PAT_ANIM )                \
int GetTexturePtnTable##ID( int ptnIx, int table )                      \
{                                                                       \
    int ret = 0;                                                        \
    int idx = ptnIx % table;                                            \
    int a = idx % 4;                                                    \
    ivec4 temp = TEX_PAT_ANIM.t[ idx/4 ];                               \
    ivec4 v;                                                            \
    v.x = GET_FLAG_EQUAL( a, 0 );                                       \
    v.y = GET_FLAG_EQUAL( a, 1 );                                       \
    v.z = GET_FLAG_EQUAL( a, 2 );                                       \
    v.w = GET_FLAG_EQUAL( a, 3 );                                       \
    return v.x * temp.x + v.y * temp.y + v.z * temp.z + v.w * temp.w;   \
}                                                                   
#endif
//------------------------------------------------------------------------------
// テクスチャパタンアニメ：実体定義
//------------------------------------------------------------------------------

// テクスチャ0
GET_TEXTURE_PTN_TABLE_DEFINE( 0, texPatAnim0 )
// テクスチャ1
GET_TEXTURE_PTN_TABLE_DEFINE( 1, texPatAnim1 )
// テクスチャ2
GET_TEXTURE_PTN_TABLE_DEFINE( 2, texPatAnim2 )

//------------------------------------------------------------------------------

//------------------------------------------------------------------------------
// テクスチャパターンアニメ コマ値を計算（マクロ版）
//------------------------------------------------------------------------------
#define CALC_TEX_PTN_ANIM_DEFINE( ID, TEX_PAT_ANIM )                                                             \
int CalculateTexPtnAnim##ID( float time, float life, float random )                                              \
{                                                                                                                \
    int table   = int( TEX_PAT_ANIM.pm.x );                                                                      \
    int freq    = int( TEX_PAT_ANIM.pm.y );                                                                      \
    int randNum = int( TEX_PAT_ANIM.pm.z );                                                                      \
    int f0 = GET_BIT_FLAG( BIT_FLAG_TEX##ID##_PTN_LIFE_FIT );                                                    \
    int f1 = GET_BIT_FLAG( BIT_FLAG_TEX##ID##_PTN_LIFE_CLAMP );                                                  \
    int f2 = GET_BIT_FLAG( BIT_FLAG_TEX##ID##_PTN_LIFE_LOOP );                                                   \
    int f3 = GET_BIT_FLAG( BIT_FLAG_TEX##ID##_PTN_LIFE_RANDOM );                                                 \
    int idxLifeFit = CalculateTexPtnAnim_LifeFit( table, freq, time, life );                                     \
    int idxClamp   = CalculateTexPtnAnim_Clamp( table, freq, time );                                             \
    int idxLoop    = CalculateTexPtnAnim_Loop( table, freq, time, random, BIT_FLAG_TEX##ID##_PTN_LOOP_RANDOM );  \
    int idxRandom  = CalculateTexPtnAnim_Random( table, random );                                                \
    int idx = f0 * idxLifeFit + f1 * idxClamp + f2 * idxLoop + f3 * idxRandom;                                   \
    return GetTexturePtnTable##ID( idx, table );                                                                 \
}  

//------------------------------------------------------------------------------
// テクスチャパターンアニメ コマ値を計算：実体定義
//------------------------------------------------------------------------------

// テクスチャ0
CALC_TEX_PTN_ANIM_DEFINE( 0, texPatAnim0 )
// テクスチャ1
CALC_TEX_PTN_ANIM_DEFINE( 1, texPatAnim1 )
// テクスチャ2
CALC_TEX_PTN_ANIM_DEFINE( 2, texPatAnim2 )

//------------------------------------------------------------------------------
// カメラオフセット
//------------------------------------------------------------------------------
vec3 CalculateCameraOffset( vec3 inPos )
{
    #define offsetParam ubCenter.z

    if ( CHECK_BIT_FLAG( BIT_FLAG_OFFSET_TYPE_CAMERA ) )
    {
        return normalize( sysEyePosition.xyz - inPos ) * offsetParam;
    }

    return vec3(0);
}

//------------------------------------------------------------------------------
// フィールド：位置に加算（頂点シェーダとStreamOut両方で引くので共通化）
//------------------------------------------------------------------------------
vec3 CalcAddPosField( float rate )
{
    vec3  fieldPosAddVel = pAdd.xyz;        // 加算値
    int   isWorld = int( pAdd.w );          // ワールド座標で適用
    float dynamicsRand = sysScaleAttr.w;    // 運動量ランダム

    vec3 add = fieldPosAddVel * dynamicsRand * rate;
    if ( isWorld == 1 ) { add = ApplyEmitterRotateInvMatrix( add ); }
    
    return add;
}




