/*---------------------------------------------------------------------------*
  Project:  NintendoWare
  File:     eft_StreamOut.vsh

  Copyright (C)2011-2013 Nintendo All rights reserved.

  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.

  The content herein is highly confidential and should be handled accordingly.
*---------------------------------------------------------------------------*/

//---------------------------------------------------
// GPUコリジョン調整（仮）
//---------------------------------------------------
vec3 AdjustmentGpuCollision( vec3 srcVec, vec4 projection, float pz, float tz )
{
    return srcVec;
}


//------------------------------------------------------------------------------
// カールノイズ
//------------------------------------------------------------------------------
vec3 CalcCurlNoiseField( vec3 inPos, vec3 inVec, float rate )
{
    vec3 ret = inVec;
    vec3 tex_coord;
    
    #define fieldCurlNoisePowX      cur0.x           // パワーX
    #define fieldCurlNoisePowY      cur0.y           // パワーY
    #define fieldCurlNoisePowZ      cur0.z           // パワーZ
    #define fieldCurlNoiseScale     cur0.w           // スケール

    #define fieldCurlNoiseSpeedX    cur1.x           // スピードX
    #define fieldCurlNoiseSpeedY    cur1.y           // スピードY
    #define fieldCurlNoiseSpeedZ    cur1.z           // スピードZ
    #define fieldCurlNoiseBase      cur1.w           // オフセット

    #define fieldCurlNoiseBaseRandom cur2.x          // ノイズのオフセットランダム
    #define fieldCurlNoiseIsWorldCoord cur2.y        // ワールド座標系で適用

    vec3 ptclPos;
    // パーティクル中心世界座標
    if( fieldCurlNoiseIsWorldCoord > 0 )
    {
        ptclPos = ( vec4( sysOutPos.xyz, 1 ) * sysEmitterMatrix ).xyz;
    }
    else
    {
        ptclPos = inPos;
    }

    tex_coord.x = ( ptclPos.x * fieldCurlNoiseScale + fieldCurlNoiseSpeedX * sysEmitterTime );
    tex_coord.y = ( ptclPos.y * fieldCurlNoiseScale + fieldCurlNoiseSpeedY * sysEmitterTime );
    tex_coord.z = ( ptclPos.z * fieldCurlNoiseScale + fieldCurlNoiseSpeedZ * sysEmitterTime );
    float noiseOffset = fieldCurlNoiseBase;
    if( fieldCurlNoiseBaseRandom != 0 ) noiseOffset *= sysPtclRandomX;
    tex_coord += noiseOffset;

    if ( tex_coord.x < 0 ) tex_coord.x *= -1;
    if ( tex_coord.y < 0 ) tex_coord.y *= -1;
    if ( tex_coord.z < 0 ) tex_coord.z *= -1;
    tex_coord.xyz *= ( 1.0 / 32.0 );

    vec4 curlNoise = texture( sysCurlNoiseTextureArray, tex_coord );
    curlNoise = curlNoise * 2 - 1;
    curlNoise *= 0.5;
    curlNoise.x *= fieldCurlNoisePowX;
    curlNoise.y *= fieldCurlNoisePowY;
    curlNoise.z *= fieldCurlNoisePowZ;

    ret += curlNoise.xyz * rate;
    
    return ret;
}

//------------------------------------------------------------------------------
// シンプルコリジョン
//------------------------------------------------------------------------------
vec3 CalcSimpleCollisionField( vec3 inVec, float rate )
{
    #define COLLISION_TYPE_REFLECT_LOCAL    0
    #define COLLISION_TYPE_REFLECT_WORLD    1
    #define COLLISION_TYPE_KILL_LOCAL       2
    #define COLLISION_TYPE_KILL_WORLD       3
    #define COLLISION_TYPE_CUSTOM_LOCAL     4
    #define COLLISION_TYPE_CUSTOM_WORLD     5

    // パーティクル中心世界座標
    vec3 ptclWorldPos = ( vec4( sysOutPos.xyz, 1 ) * sysEmitterMatrix ).xyz;

    vec3 ret = inVec;

    #define fieldCollisionType      coll.x       // タイプ（ワールド座標系の可否も含む）
    #define fieldCollisionCoef      coll.y       // 反射率
    #define fieldCollisionRegist    coll.z       // 摩擦
    #define fieldCollisionCoord     coll.w       // 反射座標

    int ColType = int( fieldCollisionType );

    if ( ColType == COLLISION_TYPE_REFLECT_LOCAL && sysOutPos.y < fieldCollisionCoord )
    {
        // エミッタ座標系で反射
        sysOutPos.y = fieldCollisionCoord;                      // 反射面まで持ち上げる
        ret.y *= -fieldCollisionCoef;
        ret   *= fieldCollisionRegist;
    }
    if ( ColType == COLLISION_TYPE_REFLECT_WORLD && ptclWorldPos.y < fieldCollisionCoord )
    {
        // 世界座標系で反射
        sysOutPos.y += fieldCollisionCoord - ptclWorldPos.y;    // 反射面まで持ち上げる
        ret.y *= -fieldCollisionCoef;
        ret   *= fieldCollisionRegist;
    }
    if ( ColType == COLLISION_TYPE_KILL_LOCAL && sysOutPos.y < fieldCollisionCoord )
    {
        // エミッタ座標系で消滅
        // TODO: 今この変数は持ち越されない。Discard処理の方が良さそう
        sysPtclAlphaRate = 0;
    }
    if ( ColType == COLLISION_TYPE_KILL_WORLD && ptclWorldPos.y < fieldCollisionCoord )
    {
        // 世界座標系で消滅
        // TODO: 今この変数は持ち越されない。Discard処理の方が良さそう
        sysPtclAlphaRate = 0;
    }
    if ( ColType == COLLISION_TYPE_CUSTOM_LOCAL )
    {
        // コリジョンカスタマイズ
        vec4 pos   = vec4( sysOutPos.xyz, 1 ) * vpMat;
        pos        = transformTexture * pos;
        float cpz  = GetDepthValue( pos );
        float ctz  = GetDepthValueFromTexture( pos );
        ret.xyz    = AdjustmentGpuCollision( ret.xyz, pos, cpz, ctz );
    }
    if ( ColType == COLLISION_TYPE_CUSTOM_WORLD )
    {
        vec4 pos   = vec4( ptclWorldPos, 1 ) * vpMat;
        pos        = transformTexture * pos;
        float cpz  = GetDepthValue( pos );
        float ctz  = GetDepthValueFromTexture( pos );
        ret.xyz    = AdjustmentGpuCollision( ret.xyz, pos, cpz, ctz );
    }
    
    return ret;
}

//------------------------------------------------------------------------------
// スピン
//------------------------------------------------------------------------------
vec3 CalcSpinField( vec3 inPos, float rate )
{
    vec3 ret = inPos;

    #define fieldSpinRotRad         spin.x            // フィールドスピン：スピン回転値
    #define fieldSpinOuter          spin.y            // フィールドスピン：スピン拡散速度
    #define fieldSpinAxis           spin.z            // フィールドスピン：スピン軸
    float dynamicsRand = sysScaleAttr.w;    // 運動量ランダム

    int axis = int( fieldSpinAxis );

    if ( axis == 0 )
    {
        float sinV = sin( fieldSpinRotRad * dynamicsRand * rate );
        float cosV = cos( fieldSpinRotRad * dynamicsRand * rate );

        float v0 = ret.y *  cosV  + ret.z * sinV;
        float v1 = ret.y * -sinV  + ret.z * cosV;
        ret.y = v0;
        ret.z = v1;

        // 拡散速度
        if( fieldSpinOuter != 0.0 )
        {
            float length = v0 * v0 + v1 * v1;
            if( length > 0.0 )
            {
                float r = 1.0f / sqrt( length ) * fieldSpinOuter * dynamicsRand * rate;
                ret.y += ( v0 * r );
                ret.z += ( v1 * r );
            }
        }
    }
    if ( axis == 1 )
    {
        float sinV = sin( fieldSpinRotRad * dynamicsRand * rate );
        float cosV = cos( fieldSpinRotRad * dynamicsRand * rate );

        float v0 = ret.z *  cosV  + ret.x * sinV;
        float v1 = ret.z * -sinV  + ret.x * cosV;
        ret.z = v0;
        ret.x = v1;

        // 拡散速度
        if( fieldSpinOuter != 0.0 )
        {
            float length = v0 * v0 + v1 * v1;
            if( length > 0.0 )
            {
                float r = 1.0f / sqrt( length ) * fieldSpinOuter * dynamicsRand * rate;
                ret.z += ( v0 * r );
                ret.x += ( v1 * r );
            }
        }
    }
    if ( axis == 2 )
    {
        float sinV = sin( fieldSpinRotRad * dynamicsRand * rate );
        float cosV = cos( fieldSpinRotRad * dynamicsRand * rate );

        float v0 = ret.x *  cosV  + ret.y * sinV;
        float v1 = ret.x * -sinV  + ret.y * cosV;
        ret.x = v0;
        ret.y = v1;

        // 拡散速度
        if( fieldSpinOuter != 0.0 )
        {
            float length = v0 * v0 + v1 * v1;
            if( length > 0.0 )
            {
                float r = 1.0f / sqrt( length ) * fieldSpinOuter * dynamicsRand * rate;
                ret.x += ( v0 * r );
                ret.y += ( v1 * r );
            }
        }
    }
    
    return ret;
}

//------------------------------------------------------------------------------
// StreamOut用、エミッタローカル位置を求めるユーティリティ関数
//------------------------------------------------------------------------------
vec3 GetEmitterLocalPosForSO()
{
    vec3 emitterPos;
    emitterPos.x = sysEmitterMatrix[0].w;
    emitterPos.y = sysEmitterMatrix[1].w;
    emitterPos.z = sysEmitterMatrix[2].w;

    mat4 tempMat;
    tempMat[0] = sysEmtMat0Attr;
    tempMat[1] = sysEmtMat1Attr;
    tempMat[2] = sysEmtMat2Attr;
    tempMat[3] = vec4( 0.0, 0.0, 0.0, 1.0 );

    return ( vec4( emitterPos, 1 ) * inverse( tempMat ) ).xyz;
}

//------------------------------------------------------------------------------
// 収束
//------------------------------------------------------------------------------
vec3 CalcConvergenceField( vec3 inPos, float rate )
{
    vec3 ret = inPos;

    #define fieldConvergencePos     cnv0.xyz   // 収束位置（ローカル座標系）
    #define fieldConvergenceRatio   cnv0.w     // 収束力
    #define fieldConvergenceCrdEmt  cnv1.x     // エミッタ追従
    float dynamicsRand = sysScaleAttr.w;    // 運動量ランダム

    // エミッタに追従
    int crdEmitter = int( fieldConvergenceCrdEmt );

    // エミッタのローカル座標
    vec3 emtLocal = vec3( 0, 0, 0 );

    if ( crdEmitter > 0 )
    {
        // パーティクルが生成された時のエミッタ位置から、
        // 現在のエミッタ位置までの差分が必要（追従させるため）
        emtLocal = GetEmitterLocalPosForSO();
    }

    return ( emtLocal + fieldConvergencePos - inPos ) * fieldConvergenceRatio * dynamicsRand * rate;

    return ret;
}

//------------------------------------------------------------------------------
// 磁力
//------------------------------------------------------------------------------
vec3 CalcMagnetField( vec3 inPos, vec3 inVec, float rate )
{
    //vec3 ret = inVec;
    #define fieldMagnetPos          mgn0.xyz       // 磁力位置
    #define fieldMagnetRatio        mgn0.w         // 磁力
    #define fieldMagnetCrdEmt       mgn1.x         // エミッタ追従
    float dynamicsRand = sysScaleAttr.w;    // 運動量ランダム

    // エミッタに追従
    int crdEmitter = int( fieldMagnetCrdEmt );

    // エミッタローカル
    vec3 emtLocal = vec3( 0 );

    if ( crdEmitter != 0 )
    {
        // TODO:エミッタローカル値取得
        emtLocal = GetEmitterLocalPosForSO();
    }

    // TODO:軸ごとに設定
    return ( emtLocal + fieldMagnetPos - inPos - inVec ) * fieldMagnetRatio * dynamicsRand * rate;
}

//------------------------------------------------------------------------------
// ランダム（旧バージョン）
//------------------------------------------------------------------------------
vec3 CalcOldRandomField( vec3 inVec, float time, float rate )
{
    vec3 ret = vec3( 0, 0,0 );

    #define fieldRandomVecX         rnf0.x          // ランダム速度X
    #define fieldRandomVecY         rnf0.y          // ランダム速度Y
    #define fieldRandomVecZ         rnf0.z          // ランダム速度Z
    #define fieldRandomApplyBlank   rnf0.w          // ランダム適用間隔
    
    if ( ( int( time ) % int( fieldRandomApplyBlank ) ) == 0 )
    {
        vec3 randomVel = rnf0.xyz;
        float sqTime = time * time;
        ret.x = ( sin( sysPtclRandomZ * sqTime ) ) * fieldRandomVecX;
        ret.y = ( sin( sysPtclRandomY * sqTime ) ) * fieldRandomVecY;
        ret.z = ( sin( sysPtclRandomX * sqTime ) ) * fieldRandomVecZ;
    }
    
    return ret;
}

//------------------------------------------------------------------------------
// ランダム
//------------------------------------------------------------------------------
vec3 CalcRandomField()
{
    #define airRegist               ubVec.x             // 空気抵抗
    float invRate = 1.0 - sysFrameRate;
    float aRegist = airRegist + (1.0f - airRegist) * invRate;

    // 空気抵抗値を加味した時間位置
    float velTime = sysPtclTime;
    if( rnd3.w == 1 )
    {
        velTime = ( aRegist == 1.0 ) ? sysPtclTime : ( 1 - pow( aRegist, sysPtclTime ) )/( 1 - aRegist );
    }

    vec3 randomAdd = vec3( 0, 0, 0 );   // ランダムによって動く変位
    vec3 randomVel = rnd0.xyz;   // ランダム適用量

    float RandCycle = rnd0.w;    // 乱数の基本周期のフレーム数

    // 位相を統一
    vec4 seed; //乱数の種（初期位相）
    if( rnd3.x == 1 )
    {
        seed.x = 0;
        seed.y = 0.31415f;    // x,y,z=0の時のために適当にずらす
        seed.z = 0.92653f;    // x,y,z=0の時のために適当にずらす
        seed.w = 0.58979f;    // x,y,z=0の時のために適当にずらす
    }
    else
    {
        seed.x = sysPtclRandomX;
        seed.y = sysPtclRandomY;
        seed.z = sysPtclRandomZ;
        seed.w = sysPtclRandomW;
    }

    seed *= RandCycle;

    // 位相を統一
    float speed = ( rnd4.x / 100.0f ) * RandCycle;   // 位相変化速度
    if( rnd3.x == 1 )
    {
        float randWidth = ( rnd4.y / 100.0f ) * RandCycle;   // 位相のばらつき
        seed.x += ( sysEmitterTime * speed ) + randWidth * ((sysPtclRandomX - 0.5f) * 2 );
        seed.y += ( sysEmitterTime * speed ) + randWidth * ((sysPtclRandomY - 0.5f) * 2 );
        seed.z += ( sysEmitterTime * speed ) + randWidth * ((sysPtclRandomZ - 0.5f) * 2 );
        seed.w += ( sysEmitterTime * speed ) + randWidth * ((sysPtclRandomW - 0.5f) * 2 );
    }

    vec3 randomValue = vec3( 0, 0, 0 );         // 乱数（これまでの積算値）
    float deltaVelTime = sysFrameRate * 1.0f;

    float deltaSeed = 0;
    if( rnd3.x == 1 )
    {
        deltaSeed = sysFrameRate * speed;
    }

    // 乱数の積算値（更新差分）を取得
    randomValue.x = GetRand( seed.x, velTime, deltaSeed, deltaVelTime );
    randomValue.y = GetRand( seed.y, velTime, deltaSeed, deltaVelTime );
    randomValue.z = GetRand( seed.z, velTime, deltaSeed, deltaVelTime );

    // 変位量を求める
    randomAdd.x = randomValue.x * randomVel.x;
    randomAdd.y = randomValue.y * randomVel.y;
    randomAdd.z = randomValue.z * randomVel.z;

    return randomAdd;

    #undef airRegist
}

//------------------------------------------------------------------------------
// StreamOutシェーダ main関数
//------------------------------------------------------------------------------
void main()
{
    // エミッタRTマトリクス生成
    MakeEmitterRTMatrix();

    #define airRegist               ubVec.x             // 空気抵抗
    #define gravityX                ubGrav.x            // 重力X
    #define gravityY                ubGrav.y            // 重力Y 
    #define gravityZ                ubGrav.z            // 重力Z
    #define gravityScale            ubGrav.w            // 重力スケール

    sysPtclTime = sysEmitterTime - sysPtclEmitTime;
    sysPtclTime = max( sysPtclTime, 0 );

    vec4 inPos = sysInPos;
    vec4 inVec = sysInVec;

    if ( sysPtclTime <= 0.001 )
    {
        // 1フレーム目は、初期位置と速度を出力するのみ
        inPos = sysLocalPosAttr;
        inVec = sysLocalVecAttr;
    }

    {
        // 2フレーム目から挙動の積算を行う
        float dynamicsRand = sysScaleAttr.w;    // 運動量ランダム
        vec3 gravityVec = ubGrav.xyz;              // 重力方向

        // 位置を更新（前フレームの速度を加算）
        sysOutPos.xyz = inPos.xyz + ( inVec.xyz * dynamicsRand * sysFrameRate );
        sysOutPos.w = inPos.w;

        // 速度を更新（前フレームの結果を引き継ぐ）
        sysOutVec = inVec;

        // 空気抵抗を計算
        float invRatio = 1.0 - sysFrameRate;
        float airRegistRate = airRegist + ( 1.0f - airRegist ) * invRatio;
        sysOutVec.xyz *= airRegistRate;

        // 重力を計算
        vec3  gravityDir = gravityVec * gravityScale;
        vec3  gravity = gravityDir;

        if ( CHECK_BIT_FLAG( EFT_WORLD_GRAVITY_ENABLED ) )
        {
            // 重力をワールド座標で適用
            gravity.x = gravityDir.x * sysEmitterRT[0].x + gravityDir.y * sysEmitterRT[1].x + gravityDir.z * sysEmitterRT[2].x;
            gravity.y = gravityDir.x * sysEmitterRT[0].y + gravityDir.y * sysEmitterRT[1].y + gravityDir.z * sysEmitterRT[2].y;
            gravity.z = gravityDir.x * sysEmitterRT[0].z + gravityDir.y * sysEmitterRT[1].z + gravityDir.z * sysEmitterRT[2].z;
        }

        sysOutVec.xyz += ( gravity.xyz * sysFrameRate );
    }

    sysOutVec.w = sysPtclTime;

    if ( CHECK_BIT_FLAG_1( BIT_FLAG_FIELD_RANDOM ) )
    {
        // GPUノイズ（位置に合成）
        sysOutPos.xyz += CalcRandomField();
    }
    if ( CHECK_BIT_FLAG_1( BIT_FLAG_FIELD_RANDOM_FE1 ) )
    {
        // FE1ランダム（位置に合成）
        sysOutVec.xyz += CalcOldRandomField( sysOutVec.xyz, sysPtclTime, sysFrameRate );
    }
    if ( CHECK_BIT_FLAG_1( BIT_FLAG_FIELD_POSADD ) )
    {
        // 位置に加算（位置に合成）
        sysOutPos.xyz += CalcAddPosField( sysFrameRate );
    }
    if ( CHECK_BIT_FLAG_1( BIT_FLAG_FIELD_MAGNET ) )
    {
        // 磁力（速度に合成）
        sysOutVec.xyz += CalcMagnetField( sysOutPos.xyz, sysOutVec.xyz, sysFrameRate );
    }
    if ( CHECK_BIT_FLAG_1( BIT_FLAG_FIELD_CONVERGENCE ) )
    {
        // 収束（位置に合成）
        sysOutPos.xyz += CalcConvergenceField( sysOutPos.xyz, sysFrameRate );
    }
    if ( CHECK_BIT_FLAG_1( BIT_FLAG_FIELD_SPIN ) )
    {
        // スピン（位置を更新）
        sysOutPos.xyz = CalcSpinField( sysOutPos.xyz, sysFrameRate );
    }
    if ( CHECK_BIT_FLAG_1( BIT_FLAG_FIELD_COLLISION ) )
    {
        // コリジョン（速度と位置を更新）（※変則）
        sysOutVec.xyz = CalcSimpleCollisionField( sysOutVec.xyz, sysFrameRate );
    }
    if ( CHECK_BIT_FLAG_1( BIT_FLAG_FIELD_CURLNOISE ) )
    {
        // カールノイズ（速度を更新）
        sysOutVec.xyz = CalcCurlNoiseField( sysOutPos.xyz, sysOutVec.xyz, sysFrameRate );
    }
}


