/*---------------------------------------------------------------------------*
  Project:  NintendoWare
  File:     eft_Stripe.vsh
  
  Copyright (C)2011-2013 Nintendo All rights reserved.
  
  These coded instructions, statements, and computer programs contain proprietary
  information of Nintendo and/or its licensed developers and are protected by
  national and international copyright laws. They may not be disclosed to third
  parties or copied or duplicated in any form, in whole or in part, without the
  prior written consent of Nintendo.
  
  The content herein is highly confidential and should be handled accordingly.
 *---------------------------------------------------------------------------*/

#define sysStripeHeadAlpha      sysEmitterPluginUniformBlockParam0.x
#define sysStripeTailAlpha      sysEmitterPluginUniformBlockParam0.y
#define sysStripeTime           sysEmitterPluginUniformBlockParam0.z
#define sysStripePointSum       sysEmitterPluginUniformBlockParam0.w
#define sysStripeRandom         sysEmitterPluginUniformBlockParam1
#define sysStripeColor0         sysEmitterPluginUniformBlockParam2
#define sysStripeColor1         sysEmitterPluginUniformBlockParam3
#define sysStripeCrossMode      sysEmitterPluginUniformBlockParam4.x

//-------------------------------------------
// ストライプのメイン関数
//-------------------------------------------
#define USE_USR_VERTEX_MAIN     // デフォルトのメイン関数を上書きします。
void main()
{
    // パーティクル時間
    sysPtclTime = sysStripeTime;

    // デプス、カメラオフセット距離
    #define offsetParam ubCenter.z

    // カメラオフセット
    vec3 center;
    center.x = sysEmitterMatrix[0].w;
    center.y = sysEmitterMatrix[1].w;
    center.z = sysEmitterMatrix[2].w;
    vec3 camOffset = CalculateCameraOffset( center );

    // ビルボード
#ifdef _STRIPE_BILLBOARD
    vec3 dir = normalize( sysEmitterPluginAttr0.xyz );
    vec3 wing = normalize( cross( eyeVec.xyz, dir ) );
    gl_Position.xyz = sysPosAttr.xyz - wing * sysPosAttr.w * 0.5;
    gl_Position.w   = 1;
    sysWorldNormalVary = cross( dir, wing );    // 法線を設定
#endif

#if defined( _STRIPE_EMITTER_MATRIX ) || defined( _STRIPE_EMITTER_UPDOWN )
    vec3 matv;
    matv.x      = sysEmitterPluginAttr1.x;
    matv.y      = sysEmitterPluginAttr1.y;
    matv.z      = sysEmitterPluginAttr1.z;
#endif

    // エミッタマトリクス
#ifdef _STRIPE_EMITTER_MATRIX
    vec3 dir = normalize( sysEmitterPluginAttr0.xyz );
    vec3 wing = normalize( cross( matv, dir ) );
    if( sysStripeCrossMode == 1 )
    {
        wing = normalize( cross( wing, dir ) ); 
    }
    gl_Position.xyz = sysPosAttr.xyz - wing * sysPosAttr.w * 0.5;
    gl_Position.w   = 1;
    sysWorldNormalVary = cross( dir, wing );    // 法線を設定
#endif

    // エミッタ上下
#ifdef _STRIPE_EMITTER_UPDOWN
    vec3 dir = normalize( sysEmitterPluginAttr0.xyz );
    vec3 wing = matv;
    if( sysStripeCrossMode == 1 )
    {
        wing = normalize( cross( wing, dir ) ); 
    }
    gl_Position.xyz = sysPosAttr.xyz - wing * sysPosAttr.w * 0.5;
    gl_Position.w   = 1;
    sysWorldNormalVary = cross( dir, wing );    // 法線を設定
#endif

#ifdef _STRIPE_FOLLOW_EMITTER_MATRIX
    // エミッタに完全追従の場合、ローカル系からワールド系に変換
    gl_Position.xyz = ( vec4( gl_Position.xyz, 1 ) * sysEmitterMatrix ).xyz;
    sysWorldNormalVary.xyz = ( vec4( sysWorldNormalVary, 1 ) * sysEmitterMatrix ).xyz;
#endif

    gl_Position.xyz += camOffset;

    // ワールド座標調整
    AdjustmentWorldPosition();

    // ワールド座標
    sysWorldPositionVary = gl_Position.xyz;

    // スクリーン座標系へ変換
    gl_Position *= sysViewProjctionMatrix;

    // デプスオフセット
    if ( CHECK_BIT_FLAG( BIT_FLAG_OFFSET_TYPE_DEPTH ) )
    {
        gl_Position.z -= offsetParam * ( 1 - gl_Position.z / gl_Position.w );
    }

    // カラー
    // ストライプColor x エミッタColor
    sysColor0Vary = sysStripeColor0 * sysEmitterColor0;
    sysColor1Vary = sysStripeColor1 * sysEmitterColor1;

    //--------------------------------------------------------------
    // テクスチャ  パターン・座標アニメ処理
    //--------------------------------------------------------------

    int ptclTexturePtn0;    // テクスチャパターン 0
    int ptclTexturePtn1;    // テクスチャパターン 1
    int ptclTexturePtn2;    // テクスチャパターン 2


#define texPosV sysEmitterPluginAttr0.w
    // テクスチャ座標
    float u = 1.0;
    if ( sysPosAttr.w > 0 ) u = 0.0;

    vec2 defuv = vec2( u, -texPosV );

    vec2 inUv0   = defuv;
    vec2 inUv1   = defuv;
    vec2 inUv2   = defuv;
    vec4 outUv0  = vec4( 0 );
    vec4 outUv1  = vec4( 0 );
    vec4 outUv2  = vec4( 0 );

    // スフィアマップ テクスチャ座標を計算する
#if 0
    vec2 sphereMapTexCoord;
    vec3 viewPosition = normalize( ( vec4( sysWorldPositionVary, 1 ) * sysViewMatrix ).xyz );
    vec3 viewNormal   = normalize( ( vec4( sysWorldNormalVary,   0 ) * sysViewMatrix ).xyz );

    viewNormal.z = clamp( viewNormal.z, 0.2, 1.0 );     // 暫定処理（zの負数を回避）

    vec3 r = reflect( viewPosition, viewNormal );
    r.z = r.z + 1.0;
    float m = 2.0 * sqrt( dot(r, r) );          
    sphereMapTexCoord.x =  (r.x / m) + 0.5;
    sphereMapTexCoord.y = -(r.y / m) + 0.5;

#ifdef _TEX_0_SPHERE_MAP
        inUv0.xy = sphereMapTexCoord;
#endif
#ifdef _TEX_1_SPHERE_MAP
        inUv1.xy = sphereMapTexCoord;
#endif
#ifdef _TEX_2_SPHERE_MAP
        inUv2.xy = sphereMapTexCoord;
#endif
#endif


    // 反転ランダムを処理
#ifdef _PARTICLE
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX0_INV_RANDOM_U ) ) { if ( sysStripeRandom.x > 0.5 ) inUv0.x = -inUv0.x; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX0_INV_RANDOM_V ) ) { if ( sysStripeRandom.y > 0.5 ) inUv0.y = -inUv0.y; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX1_INV_RANDOM_U ) ) { if ( sysStripeRandom.z > 0.5 ) inUv1.x = -inUv1.x; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX1_INV_RANDOM_V ) ) { if ( sysStripeRandom.w > 0.5 ) inUv1.y = -inUv1.y; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX2_INV_RANDOM_U ) ) { if ( sysStripeRandom.y > 0.5 ) inUv2.x = -inUv2.x; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX2_INV_RANDOM_V ) ) { if ( sysStripeRandom.z > 0.5 ) inUv2.y = -inUv2.y; }
#endif
#ifdef _PRIMITIVE
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX0_INV_RANDOM_U ) ) { if ( sysStripeRandom.x > 0.5 ) inUv0.x = 1.0 - inUv0.x; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX0_INV_RANDOM_V ) ) { if ( sysStripeRandom.y > 0.5 ) inUv0.y = 1.0 - inUv0.y; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX1_INV_RANDOM_U ) ) { if ( sysStripeRandom.z > 0.5 ) inUv1.x = 1.0 - inUv1.x; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX1_INV_RANDOM_V ) ) { if ( sysStripeRandom.w > 0.5 ) inUv1.y = 1.0 - inUv1.y; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX2_INV_RANDOM_U ) ) { if ( sysStripeRandom.y > 0.5 ) inUv2.x = 1.0 - inUv2.x; }
    if ( CHECK_BIT_FLAG( BIT_FLAG_TEX2_INV_RANDOM_V ) ) { if ( sysStripeRandom.z > 0.5 ) inUv2.y = 1.0 - inUv2.y; }
#endif

    // テクスチャ パターンアニメ コマ値を計算
    ptclTexturePtn0 = ptclTexturePtn1 = ptclTexturePtn2 = 0;

    float life = 100;
    vec4 randomSeed = sysStripeRandom;

    vec3 texPtnRandomSeed;
    texPtnRandomSeed.x = texPtnRandomSeed.y = texPtnRandomSeed.z = randomSeed.x;

    // テクスチャ座標アニメを計算
    // 乱数を共有するかしないかの設定
    ivec3 texShareRandom = ivec3( 0, 0, 0 );
    if( !CHECK_BIT_FLAG_1( BIT_FLAG_TEX_ANIM_FIXED_RANDOM ) )
    {
        // 乱数を共有しない
        texShareRandom.y = 1;
        texShareRandom.z = 2;        

        texPtnRandomSeed.x = randomSeed.x;
        texPtnRandomSeed.y = randomSeed.y;
        texPtnRandomSeed.z = randomSeed.z;
    }

#if 1
#ifdef _TEX_0_PATTERN_ANIM
    ptclTexturePtn0 = CalculateTexPtnAnim0( sysPtclTime, life, texPtnRandomSeed.x );
#endif
#ifdef _TEX_1_PATTERN_ANIM
    ptclTexturePtn1 = CalculateTexPtnAnim1( sysPtclTime, life, texPtnRandomSeed.y );
#endif
#ifdef _TEX_2_PATTERN_ANIM
    ptclTexturePtn2 = CalculateTexPtnAnim2( sysPtclTime, life, texPtnRandomSeed.z );
#endif
#endif

    vec2 animDir = vec2( -1, 1 );   // テクスチャのスクロール方向補正
    vec2 texBaseOffset = vec2( -0.5, 0.5 );
#if defined( _TEX_0_SHIFT_ANIM ) || defined( _TEX_0_PATTERN_ANIM ) || defined( _TEX_0_SHIFT_ANIM_ROTATE ) 
    outUv0 = CalculateTextureShiftAnimWithRotate( inUv0, texShiftAnim0, sysPtclTime, ptclTexturePtn0, texShareRandom.x, randomSeed );
    //outUv0.x -= 0.5;    // CalculateTextureShiftAnim 内で補正されたパーティクルテクスチャ座標を補正
    //outUv0.y -= 0.5;
#else
    outUv0.x = inUv0.x *  texShiftAnim0.pm4.x * ( 1 / texShiftAnim0.pm4.z );
    outUv0.y = inUv0.y * -texShiftAnim0.pm4.y * ( 1 / texShiftAnim0.pm4.w );
#endif

#if defined( _TEX_1_SHIFT_ANIM ) || defined( _TEX_1_PATTERN_ANIM ) || defined( _TEX_1_SHIFT_ANIM_ROTATE ) 
    outUv1 = CalculateTextureShiftAnimWithRotate( inUv1, texShiftAnim1, sysPtclTime, ptclTexturePtn1, texShareRandom.y, randomSeed );
    //outUv1.x -= 0.5;    // CalculateTextureShiftAnim 内で補正されたパーティクルテクスチャ座標を補正
    //outUv1.y -= 0.5;
#else
    outUv1.x = inUv1.x *  texShiftAnim1.pm4.x * ( 1 / texShiftAnim1.pm4.z );
    outUv1.y = inUv1.y * -texShiftAnim1.pm4.y * ( 1 / texShiftAnim1.pm4.w );
#endif

#if defined( _TEX_2_SHIFT_ANIM ) || defined( _TEX_2_PATTERN_ANIM ) || defined( _TEX_2_SHIFT_ANIM_ROTATE ) 
    outUv2 = CalculateTextureShiftAnimWithRotate( inUv2, texShiftAnim2, sysPtclTime, ptclTexturePtn2, texShareRandom.z, randomSeed );
    //outUv2.x -= 0.5;    // CalculateTextureShiftAnim 内で補正されたパーティクルテクスチャ座標を補正
    //outUv2.y -= 0.5;
#else
    outUv2.x = inUv2.x *  texShiftAnim2.pm4.x * ( 1 / texShiftAnim2.pm4.z );
    outUv2.y = inUv2.y * -texShiftAnim2.pm4.y * ( 1 / texShiftAnim2.pm4.w );
#endif

    // フラグメントシェーダでのスクリーン座標参照用
    sysFlagCoordVary = transformTexture * gl_Position;

    sysTexCoord01Vary.x = outUv0.x;
    sysTexCoord01Vary.y = outUv0.y;
    sysTexCoord01Vary.z = outUv1.x;
    sysTexCoord01Vary.w = outUv1.y;
    sysTexCoord2Vary.x  = outUv2.x;
    sysTexCoord2Vary.y  = outUv2.y;

    // マスターアルファ値を計算
    int vIndex = gl_VertexID / 2;
    float alphaPos = vIndex / ( sysStripePointSum - 1 );
    sysFlagParamVary.x  = alphaPos * ( sysStripeTailAlpha - sysStripeHeadAlpha ) + sysStripeHeadAlpha;

    // TODO:シェーダアニメを計算
    sysShaderAnim = 1.0;

    //--------------------------------------------------------------
    // カスタムシェーダ処理
    //--------------------------------------------------------------
    FinalAdjustmentVertexProcess();
}



